@@ -231,6 +231,14 @@ impl TorAddr {
         matches!(&self.host, Host::Ip(_))
     }
 
+    /// If this TorAddr is an explicit IP address, return a reference to that [`IpAddr`].
+    pub fn as_ip_address(&self) -> Option<&IpAddr> {
+        match &self.host {
+            Host::Ip(a) => Some(a),
+            _ => None,
+        }
+    }
+
     /// Get instructions for how to make a stream to this address
     pub(crate) fn into_stream_instructions(
         self,
@@ -575,7 +583,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
