@@ -60,17 +60,17 @@ use tor_keymgr::{CTorClientKeystore, CTorServiceKeystore};
 
 use futures::StreamExt as _;
 use futures::lock::Mutex as AsyncMutex;
-use futures::task::SpawnExt;
 use std::net::IpAddr;
 use std::result::Result as StdResult;
 use std::sync::{Arc, Mutex};
+use tor_rtcompat::SpawnExt;
 
 use crate::err::ErrorDetail;
 use crate::{TorClientBuilder, status, util};
 #[cfg(feature = "geoip")]
 use tor_geoip::CountryCode;
 use tor_rtcompat::scheduler::TaskHandle;
-use tracing::{debug, info};
+use tracing::{debug, info, instrument};
 
 /// An active client session on the Tor network.
 ///
@@ -483,6 +483,34 @@ impl InertTorClient {
             action: "get key manager handle",
         })?)
     }
+
+    /// Create (but do not launch) a new
+    /// [`OnionService`](tor_hsservice::OnionService)
+    /// using the given configuration.
+    ///
+    /// See [`TorClient::create_onion_service`].
+    #[cfg(feature = "onion-service-service")]
+    #[instrument(skip_all, level = "trace")]
+    pub fn create_onion_service(
+        &self,
+        config: &TorClientConfig,
+        svc_config: tor_hsservice::OnionServiceConfig,
+    ) -> crate::Result<tor_hsservice::OnionService> {
+        let keymgr = self.keymgr.as_ref().ok_or(ErrorDetail::KeystoreRequired {
+            action: "create onion service",
+        })?;
+
+        let (state_dir, mistrust) = config.state_dir()?;
+        let state_dir =
+            self::StateDirectory::new(state_dir, mistrust).map_err(ErrorDetail::StateAccess)?;
+
+        Ok(tor_hsservice::OnionService::builder()
+            .config(svc_config)
+            .keymgr(keymgr.clone())
+            .state_dir(state_dir)
+            .build()
+            .map_err(ErrorDetail::OnionServiceSetup)?)
+    }
 }
 
 /// Preferences for whether a [`TorClient`] should bootstrap on its own or not.
@@ -668,14 +696,7 @@ impl StreamPrefs {
     /// If `Explicit(true)`, Onion Service connections are enabled.
     ///
     /// If `Auto`, the behaviour depends on the `address_filter.allow_onion_addrs`
-    /// configuration option, which is in turn **disabled** by default.
-    ///
-    /// **Note**: Arti currently lacks the
-    /// "vanguards" feature that Tor uses to prevent guard discovery attacks over time.
-    /// As such, you should probably stick with C Tor if you need to make a large
-    /// number of onion service connections, or if you are using the Tor protocol
-    /// in a way that lets an attacker control how many onion services connections that you make -
-    /// for example, when using Arti's SOCKS support from a web browser such as Tor Browser.
+    /// configuration option, which is in turn enabled by default.
     #[cfg(feature = "onion-service-client")]
     pub fn connect_to_onion_services(
         &mut self,
@@ -848,6 +869,7 @@ impl<R: Runtime> TorClient<R> {
 
     /// Implementation of `create_unbootstrapped`, split out in order to avoid manually specifying
     /// double error conversions.
+    #[instrument(skip_all, level = "trace")]
     pub(crate) fn create_inner(
         runtime: R,
         config: &TorClientConfig,
@@ -880,6 +902,7 @@ impl<R: Runtime> TorClient<R> {
         let statemgr = FsStateMgr::from_path_and_mistrust(&state_dir, mistrust)
             .map_err(ErrorDetail::StateMgrSetup)?;
         // Try to take state ownership early, so we'll know if we have it.
+        // Note that this `try_lock()` may return `Ok` even if we can't acquire the lock.
         // (At this point we don't yet care if we have it.)
         let _ignore_status = statemgr.try_lock().map_err(ErrorDetail::StateMgrSetup)?;
 
@@ -1087,6 +1110,7 @@ impl<R: Runtime> TorClient<R> {
     ///
     /// If the bootstrapping process fails, returns an error. This function can safely be called
     /// again later to attempt to bootstrap another time.
+    #[instrument(skip_all, level = "trace")]
     pub async fn bootstrap(&self) -> crate::Result<()> {
         self.bootstrap_inner().await.map_err(ErrorDetail::into)
     }
@@ -1161,6 +1185,7 @@ impl<R: Runtime> TorClient<R> {
     ///
     /// Check whether a bootstrap is in progress; if one is, wait until it finishes
     /// and then return. (Otherwise, return immediately.)
+    #[instrument(skip_all, level = "trace")]
     async fn wait_for_bootstrap(&self) -> StdResult<(), ErrorDetail> {
         match self.should_bootstrap {
             BootstrapBehavior::OnDemand => {
@@ -1209,6 +1234,7 @@ impl<R: Runtime> TorClient<R> {
     /// Changing some options do not take effect immediately on all open streams
     /// and circuits, but rather affect only future streams and circuits.  Those
     /// are also explicitly documented.
+    #[instrument(skip_all, level = "trace")]
     pub fn reconfigure(
         &self,
         new_config: &TorClientConfig,
@@ -1244,6 +1270,7 @@ impl<R: Runtime> TorClient<R> {
     /// This is split out from `reconfigure` so we can do the all-or-nothing
     /// check without recursion. the caller to this method must hold the
     /// `reconfigure_lock`.
+    #[instrument(level = "trace", skip_all)]
     fn reconfigure_inner(
         &self,
         new_config: &TorClientConfig,
@@ -1390,6 +1417,7 @@ impl<R: Runtime> TorClient<R> {
     /// # Ok(())
     /// # }
     /// ```
+    #[instrument(skip_all, level = "trace")]
     pub async fn connect<A: IntoTorAddr>(&self, target: A) -> crate::Result<DataStream> {
         self.connect_with_prefs(target, &self.connect_prefs).await
     }
@@ -1400,6 +1428,7 @@ impl<R: Runtime> TorClient<R> {
     /// Note that because Tor prefers to do DNS resolution on the remote
     /// side of the network, this function takes its address as a string.
     /// (See [`TorClient::connect()`] for more information.)
+    #[instrument(skip_all, level = "trace")]
     pub async fn connect_with_prefs<A: IntoTorAddr>(
         &self,
         target: A,
@@ -1533,11 +1562,13 @@ impl<R: Runtime> TorClient<R> {
     }
 
     /// On success, return a list of IP addresses.
+    #[instrument(skip_all, level = "trace")]
     pub async fn resolve(&self, hostname: &str) -> crate::Result<Vec<IpAddr>> {
         self.resolve_with_prefs(hostname, &self.connect_prefs).await
     }
 
     /// On success, return a list of IP addresses, but use prefs.
+    #[instrument(skip_all, level = "trace")]
     pub async fn resolve_with_prefs(
         &self,
         hostname: &str,
@@ -1572,6 +1603,7 @@ impl<R: Runtime> TorClient<R> {
     /// Perform a remote DNS reverse lookup with the provided IP address.
     ///
     /// On success, return a list of hostnames.
+    #[instrument(skip_all, level = "trace")]
     pub async fn resolve_ptr(&self, addr: IpAddr) -> crate::Result<Vec<String>> {
         self.resolve_ptr_with_prefs(addr, &self.connect_prefs).await
     }
@@ -1579,6 +1611,7 @@ impl<R: Runtime> TorClient<R> {
     /// Perform a remote DNS reverse lookup with the provided IP address.
     ///
     /// On success, return a list of hostnames.
+    #[instrument(level = "trace", skip_all)]
     pub async fn resolve_ptr_with_prefs(
         &self,
         addr: IpAddr,
@@ -1676,6 +1709,7 @@ impl<R: Runtime> TorClient<R> {
 
     /// Get or launch an exit-suitable circuit with a given set of
     /// exit ports.
+    #[instrument(skip_all, level = "trace")]
     async fn get_or_launch_exit_tunnel(
         &self,
         exit_ports: &[TargetPort],
@@ -1727,6 +1761,8 @@ impl<R: Runtime> TorClient<R> {
 
     /// Try to launch an onion service with a given configuration.
     ///
+    /// Returns `Ok(None)` if the service specified is disabled in the config.
+    ///
     /// This onion service will not actually handle any requests on its own: you
     /// will need to
     /// pull [`RendRequest`](tor_hsservice::RendRequest) objects from the returned stream,
@@ -1739,13 +1775,26 @@ impl<R: Runtime> TorClient<R> {
     /// If you want to forward all the requests from an onion service to a set
     /// of local ports, you may want to use the `tor-hsrproxy` crate.
     #[cfg(feature = "onion-service-service")]
+    #[instrument(skip_all, level = "trace")]
     pub fn launch_onion_service(
         &self,
         config: tor_hsservice::OnionServiceConfig,
-    ) -> crate::Result<(
-        Arc<tor_hsservice::RunningOnionService>,
-        impl futures::Stream<Item = tor_hsservice::RendRequest> + use<R>,
-    )> {
+    ) -> crate::Result<
+        Option<(
+            Arc<tor_hsservice::RunningOnionService>,
+            impl futures::Stream<Item = tor_hsservice::RendRequest> + use<R>,
+        )>,
+    > {
+        let nickname = config.nickname();
+
+        if !config.enabled() {
+            info!(
+                nickname=%nickname,
+                "Skipping onion service because it was disabled in the config"
+            );
+            return Ok(None);
+        }
+
         let keymgr = self
             .inert_client
             .keymgr
@@ -1763,16 +1812,14 @@ impl<R: Runtime> TorClient<R> {
             .state_dir(state_dir)
             .build()
             .map_err(ErrorDetail::LaunchOnionService)?;
-        let (service, stream) = service
+        Ok(service
             .launch(
                 self.runtime.clone(),
                 self.dirmgr.clone().upcast_arc(),
                 self.hs_circ_pool.clone(),
                 Arc::clone(&self.path_resolver),
             )
-            .map_err(ErrorDetail::LaunchOnionService)?;
-
-        Ok((service, stream))
+            .map_err(ErrorDetail::LaunchOnionService)?)
     }
 
     /// Try to launch an onion service with a given configuration and provided
@@ -1780,6 +1827,8 @@ impl<R: Runtime> TorClient<R> {
     /// associated `HsIdKeypair`  in this `TorClient`'s `KeyMgr`, then this operation
     /// fails rather than overwriting the existing key.
     ///
+    /// Returns `Ok(None)` if the service specified is disabled in the config.
+    ///
     /// The specified `HsIdKeypair` will be inserted in the primary keystore.
     ///
     /// **Important**: depending on the configuration of your
@@ -1800,14 +1849,17 @@ impl<R: Runtime> TorClient<R> {
     /// If you want to forward all the requests from an onion service to a set
     /// of local ports, you may want to use the `tor-hsrproxy` crate.
     #[cfg(all(feature = "onion-service-service", feature = "experimental-api"))]
+    #[instrument(skip_all, level = "trace")]
     pub fn launch_onion_service_with_hsid(
         &self,
         config: tor_hsservice::OnionServiceConfig,
         id_keypair: HsIdKeypair,
-    ) -> crate::Result<(
-        Arc<tor_hsservice::RunningOnionService>,
-        impl futures::Stream<Item = tor_hsservice::RendRequest> + use<R>,
-    )> {
+    ) -> crate::Result<
+        Option<(
+            Arc<tor_hsservice::RunningOnionService>,
+            impl futures::Stream<Item = tor_hsservice::RendRequest> + use<R>,
+        )>,
+    > {
         let nickname = config.nickname();
         let hsid_spec = HsIdKeypairSpecifier::new(nickname.clone());
         let selector = KeystoreSelector::Primary;
@@ -2008,28 +2060,31 @@ impl<R: Runtime> TorClient<R> {
     /// [`OnionService`](tor_hsservice::OnionService)
     /// using the given configuration.
     ///
+    /// This is useful for managing an onion service without needing to start a `TorClient` or the
+    /// onion service itself.
+    /// If you only wish to run the onion service, see
+    /// [`TorClient::launch_onion_service()`]
+    /// which allows you to launch an onion service from a running `TorClient`.
+    ///
     /// The returned `OnionService` can be launched using
     /// [`OnionService::launch()`](tor_hsservice::OnionService::launch).
+    /// Note that `launch()` requires a [`NetDirProvider`],
+    /// [`HsCircPool`](tor_circmgr::hspool::HsCircPool), etc,
+    /// which you should obtain from a running `TorClient`.
+    /// But these are only accessible from a `TorClient` if the "experimental-api" feature is
+    /// enabled.
+    /// The behaviour is not specified if you create the `OnionService` with
+    /// `create_onion_service()` using one [`TorClientConfig`],
+    /// but launch it using a `TorClient` generated from a different `TorClientConfig`.
+    // TODO #2249: Look into this behaviour more, and possibly error if there is a different config.
     #[cfg(feature = "onion-service-service")]
+    #[instrument(skip_all, level = "trace")]
     pub fn create_onion_service(
         config: &TorClientConfig,
         svc_config: tor_hsservice::OnionServiceConfig,
     ) -> crate::Result<tor_hsservice::OnionService> {
         let inert_client = InertTorClient::new(config)?;
-        let keymgr = inert_client.keymgr.ok_or(ErrorDetail::KeystoreRequired {
-            action: "create onion service",
-        })?;
-
-        let (state_dir, mistrust) = config.state_dir()?;
-        let state_dir =
-            self::StateDirectory::new(state_dir, mistrust).map_err(ErrorDetail::StateAccess)?;
-
-        Ok(tor_hsservice::OnionService::builder()
-            .config(svc_config)
-            .keymgr(keymgr)
-            .state_dir(state_dir)
-            .build()
-            .map_err(ErrorDetail::OnionServiceSetup)?)
+        inert_client.create_onion_service(config, svc_config)
     }
 
     /// Return a current [`status::BootstrapStatus`] describing how close this client
@@ -2067,6 +2122,7 @@ impl<R: Runtime> TorClient<R> {
     /// Return a [`Future`] which resolves
     /// once this TorClient has stopped.
     #[cfg(feature = "experimental-api")]
+    #[instrument(skip_all, level = "trace")]
     pub fn wait_for_stop(
         &self,
     ) -> impl futures::Future<Output = ()> + Send + Sync + 'static + use<R> {
@@ -2137,7 +2193,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
