@@ -43,11 +43,13 @@ pub mod circ {
 
 /// Types for configuring how Tor accesses its directory information.
 pub mod dir {
-    pub use tor_dirmgr::{
-        Authority, AuthorityBuilder, DirMgrConfig, DirTolerance, DirToleranceBuilder,
-        DownloadSchedule, DownloadScheduleConfig, DownloadScheduleConfigBuilder, FallbackDir,
-        FallbackDirBuilder, NetworkConfig, NetworkConfigBuilder,
+    pub use tor_dircommon::authority::{AuthorityContacts, AuthorityContactsBuilder};
+    pub use tor_dircommon::config::{
+        DirTolerance, DirToleranceBuilder, DownloadScheduleConfig, DownloadScheduleConfigBuilder,
+        NetworkConfig, NetworkConfigBuilder,
     };
+    pub use tor_dircommon::retry::{DownloadSchedule, DownloadScheduleBuilder};
+    pub use tor_dirmgr::{DirMgrConfig, FallbackDir, FallbackDirBuilder};
 }
 
 /// Types for configuring pluggable transports.
@@ -67,6 +69,32 @@ pub mod vanguards {
     pub use tor_guardmgr::{VanguardConfig, VanguardConfigBuilder};
 }
 
+#[cfg(not(all(
+    feature = "vanguards",
+    any(feature = "onion-service-client", feature = "onion-service-service"),
+)))]
+use {
+    std::sync::LazyLock,
+    tor_config::ExplicitOrAuto,
+    tor_guardmgr::{VanguardConfig, VanguardConfigBuilder, VanguardMode},
+};
+
+/// A [`VanguardConfig`] which is disabled.
+// It would be nice if the builder were const, but this is the best we can do.
+// Boxed so that this is guaranteed to use very little space if it's unused.
+#[cfg(not(all(
+    feature = "vanguards",
+    any(feature = "onion-service-client", feature = "onion-service-service"),
+)))]
+static DISABLED_VANGUARDS: LazyLock<Box<VanguardConfig>> = LazyLock::new(|| {
+    Box::new(
+        VanguardConfigBuilder::default()
+            .mode(ExplicitOrAuto::Explicit(VanguardMode::Disabled))
+            .build()
+            .expect("Could not build a disabled `VanguardConfig`"),
+    )
+});
+
 /// Configuration for client behavior relating to addresses.
 ///
 /// This type is immutable once constructed. To create an object of this type,
@@ -661,6 +689,8 @@ pub struct TorClientConfig {
     pub(crate) stream_timeouts: StreamTimeoutConfig,
 
     /// Information about vanguards.
+    // NOTE: Don't use `#[as_ref]` below, since we provide our own AsRef impl to handle when
+    // vanguards are disabled.
     #[builder(sub_builder)]
     #[builder_field_attr(serde(default))]
     pub(crate) vanguards: vanguards::VanguardConfig,
@@ -715,15 +745,24 @@ pub struct SystemConfig {
 }
 impl_standard_builder! { SystemConfig }
 
-impl tor_circmgr::CircMgrConfig for TorClientConfig {
-    #[cfg(all(
-        feature = "vanguards",
-        any(feature = "onion-service-client", feature = "onion-service-service")
-    ))]
-    fn vanguard_config(&self) -> &tor_guardmgr::VanguardConfig {
-        &self.vanguards
+impl AsRef<tor_guardmgr::VanguardConfig> for TorClientConfig {
+    fn as_ref(&self) -> &tor_guardmgr::VanguardConfig {
+        cfg_if::cfg_if! {
+            if #[cfg(all(
+                feature = "vanguards",
+                any(feature = "onion-service-client", feature = "onion-service-service"),
+            ))]
+            {
+                &self.vanguards
+            } else {
+                &DISABLED_VANGUARDS
+            }
+        }
     }
 }
+
+impl tor_circmgr::CircMgrConfig for TorClientConfig {}
+
 #[cfg(feature = "onion-service-client")]
 impl tor_hsclient::HsClientConnectorConfig for TorClientConfig {}
 
@@ -738,8 +777,8 @@ impl tor_circmgr::hspool::HsCircPoolConfig for TorClientConfig {
     }
 }
 
-impl AsRef<tor_guardmgr::fallback::FallbackList> for TorClientConfig {
-    fn as_ref(&self) -> &tor_guardmgr::fallback::FallbackList {
+impl AsRef<tor_dircommon::fallback::FallbackList> for TorClientConfig {
+    fn as_ref(&self) -> &tor_dircommon::fallback::FallbackList {
         self.tor_network.fallback_caches()
     }
 }
@@ -890,10 +929,12 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
+    use std::net::{Ipv4Addr, Ipv6Addr, SocketAddr, SocketAddrV4, SocketAddrV6};
+
     use super::*;
 
     #[test]
@@ -908,10 +949,14 @@ mod test {
     fn builder() {
         let sec = std::time::Duration::from_secs(1);
 
-        let auth = dir::Authority::builder()
-            .name("Fred")
-            .v3ident([22; 20].into())
-            .clone();
+        let mut authorities = dir::AuthorityContacts::builder();
+        authorities.v3idents().push([22; 20].into());
+        authorities.v3idents().push([44; 20].into());
+        authorities.uploads().push(vec![
+            SocketAddr::V4(SocketAddrV4::new(Ipv4Addr::LOCALHOST, 80)),
+            SocketAddr::V6(SocketAddrV6::new(Ipv6Addr::LOCALHOST, 80, 0, 0)),
+        ]);
+
         let mut fallback = dir::FallbackDir::builder();
         fallback
             .rsa_identity([23; 20].into())
@@ -920,7 +965,7 @@ mod test {
             .push("127.0.0.7:7".parse().unwrap());
 
         let mut bld = TorClientConfig::builder();
-        bld.tor_network().set_authorities(vec![auth]);
+        *bld.tor_network().authorities() = authorities;
         bld.tor_network().set_fallback_caches(vec![fallback]);
         bld.storage()
             .cache_dir(CfgPath::new("/var/tmp/foo".to_owned()))
@@ -1012,6 +1057,17 @@ mod test {
         assert_eq!(dflt.len(), 2);
     }
 
+    #[test]
+    #[cfg(not(all(
+        feature = "vanguards",
+        any(feature = "onion-service-client", feature = "onion-service-service"),
+    )))]
+    fn check_disabled_vanguards_static() {
+        // Force us to evaluate the closure to ensure that it builds correctly.
+        #[allow(clippy::borrowed_box)]
+        let _: &Box<VanguardConfig> = LazyLock::force(&DISABLED_VANGUARDS);
+    }
+
     #[test]
     #[cfg(feature = "pt-client")]
     fn check_bridge_pt() {
