@@ -123,6 +123,8 @@ macro_rules! resolve_listen_port {
 #[allow(clippy::option_option)] // Builder port fields: Some(None) = specified to disable
 pub struct ProxyConfig {
     /// Addresses to listen on for incoming SOCKS connections.
+    //
+    // TODO: Once http-connect is non-experimental, we should rename this option in a backward-compatible way.
     #[builder(field(build = r#"#[allow(deprecated)]
                    // We use this deprecated macro to instantiate the legacy socks_port option.
                    { resolve_listen_port!(self, socks, 9150) }
@@ -380,7 +382,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -567,6 +569,7 @@ mod test {
                 "proxy.socks_listen",
                 "proxy.dns_listen",
                 "use_obsolete_software",
+                "circuit_timing.disused_circuit_timeout",
             ],
         );
 
@@ -581,6 +584,16 @@ mod test {
             ],
         );
 
+        declare_exceptions(
+            None,
+            None,
+            Recognized,
+            &[
+                // Examples exist but are not auto-testable
+                "logging.opentelemetry",
+            ],
+        );
+
         declare_exceptions(
             Some(InOld),
             Some(InNew),
@@ -616,6 +629,17 @@ mod test {
             ],
         );
 
+        declare_exceptions(
+            None,
+            None, // it's there, but not formatted for auto-testing
+            FeatureDependent,
+            &[
+                // Settings only available with tokio-console support
+                "logging.tokio_console",
+                "logging.tokio_console.enabled",
+            ],
+        );
+
         declare_exceptions(
             None,
             None, // it's there, but not formatted for auto-testing
@@ -1632,10 +1656,9 @@ example config file {which:?}, uncommented={uncommented:?}
         use tor_config_path::CfgPath;
         let sec = std::time::Duration::from_secs(1);
 
-        let auth = dir::Authority::builder()
-            .name("Fred")
-            .v3ident([22; 20].into())
-            .clone();
+        let mut authorities = dir::AuthorityContacts::builder();
+        authorities.v3idents().push([22; 20].into());
+
         let mut fallback = dir::FallbackDir::builder();
         fallback
             .rsa_identity([23; 20].into())
@@ -1649,7 +1672,7 @@ example config file {which:?}, uncommented={uncommented:?}
         bld.proxy().socks_listen(Listen::new_localhost(9999));
         bld.logging().console("warn");
 
-        bld_tor.tor_network().set_authorities(vec![auth]);
+        *bld_tor.tor_network().authorities() = authorities;
         bld_tor.tor_network().set_fallback_caches(vec![fallback]);
         bld_tor
             .storage()
