@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // TODO: Stuff to add before this crate is ready....
 //  - Test the absolute heck out of it.
@@ -36,7 +36,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -653,11 +653,12 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
+    use assert_matches::assert_matches;
     use testing::{Dir, MistrustOp, mistrust_build};
 
     #[cfg(target_family = "unix")]
@@ -978,6 +979,15 @@ mod test {
         let _m = Mistrust::default();
     }
 
+    #[test]
+    fn empty_path() {
+        let m = mistrust_build(&[MistrustOp::DangerouslyTrustEveryone()]);
+        assert_matches!(m.check_directory(""), Err(Error::NotFound(_)));
+
+        let m = Mistrust::default();
+        assert_matches!(m.check_directory(""), Err(Error::NotFound(_)));
+    }
+
     // TODO: Write far more tests.
     // * Can there be a test for a failed readlink()?  I can't see an easy way
     //   to provoke that without trying to make a time-of-check/time-of-use race
