@@ -183,6 +183,8 @@ decl_extension_group! {
     }
 }
 
+/// Helper for generating encoding and decoding functions
+/// for [`CircRequestExt`] and [`CircResponseExt`].
 macro_rules! impl_encode_decode {
     ($extgroup:ty, $name:expr) => {
         impl $extgroup {
@@ -224,7 +226,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
