@@ -5,10 +5,10 @@
 
 use super::flow_ctrl::{Xoff, Xon};
 use super::{RelayCellFormat, RelayCmd};
+use crate::chancell::CELL_DATA_LEN;
 use crate::chancell::msg::{
     DestroyReason, HandshakeType, TAP_C_HANDSHAKE_LEN, TAP_S_HANDSHAKE_LEN,
 };
-use crate::chancell::CELL_DATA_LEN;
 use caret::caret_int;
 use derive_deftly::Deftly;
 use std::fmt::Write;
@@ -30,8 +30,8 @@ pub use super::conflux::{ConfluxLink, ConfluxLinked, ConfluxLinkedAck, ConfluxSw
 #[cfg(feature = "hs")]
 #[cfg_attr(docsrs, doc(cfg(feature = "hs")))]
 pub use super::hs::{
-    est_intro::EstablishIntro, EstablishRendezvous, IntroEstablished, Introduce1, Introduce2,
-    IntroduceAck, Rendezvous1, Rendezvous2, RendezvousEstablished,
+    EstablishRendezvous, IntroEstablished, Introduce1, Introduce2, IntroduceAck, Rendezvous1,
+    Rendezvous2, RendezvousEstablished, est_intro::EstablishIntro,
 };
 #[cfg(feature = "experimental-udp")]
 #[cfg_attr(docsrs, doc(cfg(feature = "experimental-udp")))]
@@ -469,8 +469,8 @@ caret_int! {
 
 impl tor_error::HasKind for EndReason {
     fn kind(&self) -> tor_error::ErrorKind {
-        use tor_error::ErrorKind as EK;
         use EndReason as E;
+        use tor_error::ErrorKind as EK;
         match *self {
             E::MISC => EK::RemoteStreamError,
             E::RESOLVEFAILED => EK::RemoteHostResolutionFailed,
@@ -1347,6 +1347,7 @@ macro_rules! empty_body {
        }
    }
 }
+#[allow(unused_imports)] // TODO #1060
 pub(crate) use empty_body;
 
 empty_body! {
@@ -1442,7 +1443,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
