@@ -5,11 +5,13 @@ use std::sync::{Arc, Mutex};
 
 use crate::event::ChanMgrEventSender;
 use async_trait::async_trait;
+#[cfg(feature = "relay")]
+use safelog::Sensitive;
 use tor_error::{HasKind, HasRetryTime, internal};
 use tor_linkspec::{HasChanMethod, OwnedChanTarget, PtTransportName};
 use tor_proto::channel::Channel;
 use tor_proto::memquota::ChannelAccount;
-use tracing::debug;
+use tracing::{debug, instrument};
 
 /// An opaque type that lets a `ChannelFactory` update the `ChanMgr` about bootstrap progress.
 ///
@@ -75,7 +77,7 @@ pub trait IncomingChannelFactory: Send + Sync {
     #[cfg(feature = "relay")]
     async fn accept_from_transport(
         &self,
-        peer: std::net::SocketAddr,
+        peer: Sensitive<std::net::SocketAddr>,
         stream: Self::Stream,
         memquota: ChannelAccount,
     ) -> crate::Result<Arc<Channel>>;
@@ -90,6 +92,7 @@ where
     type BuildSpec = OwnedChanTarget;
     type Stream = CF::Stream;
 
+    #[instrument(skip_all, level = "trace")]
     async fn build_channel(
         &self,
         target: &Self::BuildSpec,
@@ -101,9 +104,10 @@ where
     }
 
     #[cfg(feature = "relay")]
+    #[instrument(skip_all, level = "trace")]
     async fn build_channel_using_incoming(
         &self,
-        peer: std::net::SocketAddr,
+        peer: Sensitive<std::net::SocketAddr>,
         stream: Self::Stream,
         memquota: ChannelAccount,
     ) -> crate::Result<Arc<tor_proto::channel::Channel>> {
@@ -169,6 +173,7 @@ impl<CF> Clone for CompoundFactory<CF> {
 
 #[async_trait]
 impl<CF: ChannelFactory> ChannelFactory for CompoundFactory<CF> {
+    #[instrument(skip_all, level = "trace")]
     async fn connect_via_transport(
         &self,
         target: &OwnedChanTarget,
@@ -208,7 +213,7 @@ impl<CF: IncomingChannelFactory> IncomingChannelFactory for CompoundFactory<CF>
     #[cfg(feature = "relay")]
     async fn accept_from_transport(
         &self,
-        peer: std::net::SocketAddr,
+        peer: Sensitive<std::net::SocketAddr>,
         stream: Self::Stream,
         memquota: ChannelAccount,
     ) -> crate::Result<Arc<Channel>> {
