@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -57,7 +57,8 @@ pub(crate) mod util;
 
 use futures::StreamExt;
 use futures::select_biased;
-use futures::task::SpawnExt;
+#[cfg(feature = "relay")]
+use safelog::Sensitive;
 use std::result::Result as StdResult;
 use std::sync::{Arc, Weak};
 use std::time::Duration;
@@ -70,7 +71,9 @@ use tor_proto::channel::Channel;
 #[cfg(feature = "experimental-api")]
 use tor_proto::memquota::ChannelAccount;
 use tor_proto::memquota::ToplevelAccount;
+use tor_rtcompat::SpawnExt;
 use tracing::debug;
+use tracing::instrument;
 use void::{ResultVoidErrExt, Void};
 
 pub use err::Error;
@@ -250,6 +253,7 @@ impl<R: Runtime> ChanMgr<R> {
     ///
     /// Returns a [`TaskHandle`] that can be used to manage
     /// those daemon tasks that poll periodically.
+    #[instrument(level = "trace", skip_all)]
     pub fn launch_background_tasks(
         self: &Arc<Self>,
         runtime: &R,
@@ -282,7 +286,7 @@ impl<R: Runtime> ChanMgr<R> {
     #[cfg(feature = "relay")]
     pub async fn handle_incoming(
         &self,
-        src: std::net::SocketAddr,
+        src: Sensitive<std::net::SocketAddr>,
         stream: <R as tor_rtcompat::NetStreamProvider>::Stream,
     ) -> Result<Arc<Channel>> {
         self.mgr.handle_incoming(src, stream).await
@@ -294,6 +298,7 @@ impl<R: Runtime> ChanMgr<R> {
     /// If there is already a channel launch attempt in progress, this
     /// function will wait until that launch is complete, and succeed
     /// or fail depending on its outcome.
+    #[instrument(level = "trace", skip_all)]
     pub async fn get_or_launch<T: ChanTarget + ?Sized>(
         &self,
         target: &T,
@@ -381,6 +386,7 @@ impl<R: Runtime> ChanMgr<R> {
     /// make sure that no other code with access to this `ChanMgr` will be able
     /// to use the channel.
     #[cfg(feature = "experimental-api")]
+    #[instrument(level = "trace", skip_all)]
     pub async fn build_unmanaged_channel(
         &self,
         target: impl tor_linkspec::IntoOwnedChanTarget,
@@ -402,6 +408,7 @@ impl<R: Runtime> ChanMgr<R> {
     ///
     /// This is a daemon task that runs indefinitely in the background,
     /// and exits when we find that `chanmgr` is dropped.
+    #[instrument(level = "trace", skip_all)]
     async fn continually_update_channels_config(
         self_: Weak<Self>,
         netdir: Arc<dyn NetDirProvider>,
@@ -443,6 +450,7 @@ impl<R: Runtime> ChanMgr<R> {
     /// Exist when we find that `chanmgr` is dropped
     ///
     /// This is a daemon task that runs indefinitely in the background
+    #[instrument(level = "trace", skip_all)]
     async fn continually_expire_channels(mut sched: TaskSchedule<R>, chanmgr: Weak<Self>) {
         while sched.next().await.is_some() {
             let Some(cm) = Weak::upgrade(&chanmgr) else {
