@@ -9,7 +9,7 @@
 #![allow(clippy::print_stdout)]
 #![allow(clippy::single_char_pattern)]
 #![allow(clippy::unwrap_used)]
-#![allow(clippy::unchecked_duration_subtraction)]
+#![allow(clippy::unchecked_time_subtraction)]
 #![allow(clippy::useless_vec)]
 #![allow(clippy::needless_pass_by_value)]
 //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -22,6 +22,8 @@ use async_trait::async_trait;
 use futures::channel::mpsc;
 use futures_await_test::async_test;
 use itertools::{Itertools, zip_eq};
+#[cfg(feature = "relay")]
+use safelog::Sensitive;
 
 use tor_cell::chancell::msg::PaddingNegotiateCmd;
 use tor_config::PaddingLevel;
@@ -30,6 +32,7 @@ use tor_memquota::ArcMemoryQuotaTrackerExt as _;
 use tor_netdir::NetDir;
 use tor_proto::channel::{Channel, CtrlMsg};
 use tor_proto::memquota::{ChannelAccount, ToplevelAccount};
+use tor_rtcompat::{Runtime, test_with_all_runtimes};
 
 use crate::ChannelUsage;
 use crate::mgr::{AbstractChanMgr, AbstractChannelFactory};
@@ -145,7 +148,7 @@ impl AbstractChannelFactory for FakeChannelFactory {
     #[cfg(feature = "relay")]
     async fn build_channel_using_incoming(
         &self,
-        _peer: std::net::SocketAddr,
+        _peer: Sensitive<std::net::SocketAddr>,
         _stream: Self::Stream,
         _memquota: ChannelAccount,
     ) -> Result<Arc<Self::Channel>> {
@@ -168,14 +171,20 @@ struct Expected {
     nego: Option<(PaddingNegotiateCmd, [u32; 2])>,
 }
 
-async fn case(level: PaddingLevel, dormancy: Dormancy, usage: ChannelUsage) -> CaseContext {
+async fn case(
+    rt: &impl Runtime,
+    level: PaddingLevel,
+    dormancy: Dormancy,
+    usage: ChannelUsage,
+) -> CaseContext {
     let mut cconfig = ChannelConfig::builder();
     cconfig.padding(level);
     let cconfig = cconfig.build().unwrap();
 
     eprintln!("\n---- {:?} {:?} {:?} ----", &cconfig, &dormancy, &usage);
 
-    let (channel, recv) = Channel::new_fake(tor_proto::channel::ChannelType::ClientInitiator);
+    let (channel, recv) =
+        Channel::new_fake(rt.clone(), tor_proto::channel::ChannelType::ClientInitiator);
     let peer_id = channel.target().ed_identity().unwrap().clone();
     let relay_ids = RelayIds::builder()
         .ed_identity(peer_id.clone())
@@ -273,13 +282,24 @@ impl CaseContext {
 /// once (assuming it can be sent before the channel is closed or the next one arrives).
 /// The timing parameters, and enablement, are passed directly to the padding timer.
 #[async_test]
-async fn padding_control_through_layers() {
+async fn padding_control_through_layer() {
+    test_with_all_runtimes!(padding_control_through_layer_impl);
+}
+
+/// Helper for padding_control_through_layer: takes a runtime as an argument.
+async fn padding_control_through_layer_impl(rt: impl tor_rtcompat::Runtime) {
     const STOP_MSG: (PaddingNegotiateCmd, [u32; 2]) = (PaddingNegotiateCmd::STOP, [0, 0]);
     const START_CMD: PaddingNegotiateCmd = PaddingNegotiateCmd::START;
 
     // ---- simple case, active exit, defaults ----
 
-    let mut c = case(PL::default(), Dormancy::Active, ChannelUsage::UserTraffic).await;
+    let mut c = case(
+        &rt,
+        PL::default(),
+        Dormancy::Active,
+        ChannelUsage::UserTraffic,
+    )
+    .await;
     c.expect_1(Expected {
         enabled: Some(true),
         timing: Some(DEF_MS),
@@ -288,7 +308,13 @@ async fn padding_control_through_layers() {
 
     // ---- reduced padding ----
 
-    let mut c = case(PL::Reduced, Dormancy::Active, ChannelUsage::UserTraffic).await;
+    let mut c = case(
+        &rt,
+        PL::Reduced,
+        Dormancy::Active,
+        ChannelUsage::UserTraffic,
+    )
+    .await;
     c.expect_1(Expected {
         enabled: Some(true),
         timing: Some(REDUCED_MS),
@@ -297,7 +323,13 @@ async fn padding_control_through_layers() {
 
     // ---- dormant ----
 
-    let mut c = case(PL::default(), Dormancy::Dormant, ChannelUsage::UserTraffic).await;
+    let mut c = case(
+        &rt,
+        PL::default(),
+        Dormancy::Dormant,
+        ChannelUsage::UserTraffic,
+    )
+    .await;
     c.expect_1(Expected {
         enabled: None,
         timing: None,
@@ -312,7 +344,7 @@ async fn padding_control_through_layers() {
         cconfig.build().unwrap()
     };
 
-    let mut c = case(PL::default(), Dormancy::Active, ChannelUsage::Dir).await;
+    let mut c = case(&rt, PL::default(), Dormancy::Active, ChannelUsage::Dir).await;
     // directory circuits don't get padding (and we don't need to tell the peer to disable)
     c.expect_0();
 
