@@ -5,7 +5,6 @@ use crate::timeouts::{self, Action};
 use crate::{Error, Result};
 use async_trait::async_trait;
 use futures::Future;
-use futures::task::SpawnExt;
 use oneshot_fused_workaround as oneshot;
 use std::sync::{
     Arc,
@@ -17,11 +16,13 @@ use tor_error::into_internal;
 use tor_guardmgr::GuardStatus;
 use tor_linkspec::{IntoOwnedChanTarget, OwnedChanTarget, OwnedCircTarget};
 use tor_netdir::params::NetParameters;
-use tor_proto::ClientTunnel;
 use tor_proto::ccparams::{self, AlgorithmType};
 use tor_proto::client::circuit::{CircParameters, PendingClientTunnel};
+use tor_proto::{CellCount, ClientTunnel, FlowCtrlParameters};
+use tor_rtcompat::SpawnExt;
 use tor_rtcompat::{Runtime, SleepProviderExt};
 use tor_units::Percentage;
+use tracing::instrument;
 
 #[cfg(all(feature = "vanguards", feature = "hs-common"))]
 use tor_guardmgr::vanguards::VanguardMgr;
@@ -61,6 +62,7 @@ pub(crate) trait Buildable: Sized {
         rt: &RT,
         ct: &OwnedChanTarget,
         params: CircParameters,
+        timeouts: Arc<dyn tor_proto::client::circuit::TimeoutEstimator>,
     ) -> Result<Self>;
 
     /// Launch a new circuit through a given relay, given a circuit target
@@ -70,6 +72,7 @@ pub(crate) trait Buildable: Sized {
         rt: &RT,
         ct: &OwnedCircTarget,
         params: CircParameters,
+        timeouts: Arc<dyn tor_proto::client::circuit::TimeoutEstimator>,
     ) -> Result<Self>;
 
     /// Extend this circuit-like object by one hop, to the location described
@@ -87,17 +90,24 @@ pub(crate) trait Buildable: Sized {
 ///
 /// This is common code, shared by all the first-hop functions in the
 /// implementation of `Buildable` for `ClientTunnel`.
+#[instrument(level = "trace", skip_all)]
 async fn create_common<RT: Runtime>(
     chan: Arc<tor_proto::channel::Channel>,
+    timeouts: Arc<dyn tor_proto::client::circuit::TimeoutEstimator>,
     rt: &RT,
 ) -> Result<PendingClientTunnel> {
     // Construct the (zero-hop) circuit.
-    let (pending_tunnel, reactor) = chan.new_tunnel().await.map_err(|error| Error::Protocol {
-        error,
-        peer: None, // we don't blame the peer, because new_circ() does no networking.
-        action: "initializing circuit",
-        unique_id: None,
-    })?;
+    let (pending_tunnel, reactor) =
+        chan.new_tunnel(timeouts)
+            .await
+            .map_err(|error| Error::Protocol {
+                error,
+                peer: None, // we don't blame the peer, because new_circ() does no networking.
+                action: "initializing circuit",
+                unique_id: None,
+            })?;
+
+    tracing::info!("Spawning reactor...");
 
     rt.spawn(async {
         let _ = reactor.run().await;
@@ -111,6 +121,7 @@ async fn create_common<RT: Runtime>(
 impl Buildable for ClientTunnel {
     type Chan = tor_proto::channel::Channel;
 
+    #[instrument(level = "trace", skip_all)]
     async fn open_channel<RT: Runtime>(
         chanmgr: &ChanMgr<RT>,
         target: &OwnedChanTarget,
@@ -142,13 +153,15 @@ impl Buildable for ClientTunnel {
         }
     }
 
+    #[instrument(level = "trace", skip_all)]
     async fn create_chantarget<RT: Runtime>(
         chan: Arc<Self::Chan>,
         rt: &RT,
         ct: &OwnedChanTarget,
         params: CircParameters,
+        timeouts: Arc<dyn tor_proto::client::circuit::TimeoutEstimator>,
     ) -> Result<Self> {
-        let pending_tunnel = create_common(chan, rt).await?;
+        let pending_tunnel = create_common(chan, timeouts, rt).await?;
         let unique_id = Some(pending_tunnel.peek_unique_id());
         pending_tunnel
             .create_firsthop_fast(params)
@@ -160,13 +173,15 @@ impl Buildable for ClientTunnel {
                 unique_id,
             })
     }
+    #[instrument(level = "trace", skip_all)]
     async fn create<RT: Runtime>(
         chan: Arc<Self::Chan>,
         rt: &RT,
         ct: &OwnedCircTarget,
         params: CircParameters,
+        timeouts: Arc<dyn tor_proto::client::circuit::TimeoutEstimator>,
     ) -> Result<Self> {
-        let pending_tunnel = create_common(chan, rt).await?;
+        let pending_tunnel = create_common(chan, timeouts, rt).await?;
         let unique_id = Some(pending_tunnel.peek_unique_id());
 
         let handshake_res = pending_tunnel.create_firsthop(ct, params).await;
@@ -216,7 +231,7 @@ struct Builder<R: Runtime, C: Buildable + Sync + Send + 'static> {
     /// A channel manager that this circuit builder uses to make channels.
     chanmgr: Arc<ChanMgr<R>>,
     /// An estimator to determine the correct timeouts for circuit building.
-    timeouts: timeouts::Estimator,
+    timeouts: Arc<timeouts::Estimator>,
     /// We don't actually hold any clientcircs, so we need to put this
     /// type here so the compiler won't freak out.
     _phantom: std::marker::PhantomData<C>,
@@ -228,7 +243,7 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
         Builder {
             runtime,
             chanmgr,
-            timeouts,
+            timeouts: Arc::new(timeouts),
             _phantom: std::marker::PhantomData,
         }
     }
@@ -243,6 +258,7 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
     ///
     /// (TODO: Find
     /// a better design there.)
+    #[instrument(level = "trace", skip_all)]
     async fn build_notimeout(
         self: Arc<Self>,
         path: OwnedPath,
@@ -254,7 +270,9 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
     ) -> Result<C> {
         match path {
             OwnedPath::ChannelOnly(target) => {
-                let circ = C::create_chantarget(channel, &self.runtime, &target, params).await?;
+                let timeouts = Arc::clone(&self.timeouts);
+                let circ =
+                    C::create_chantarget(channel, &self.runtime, &target, params, timeouts).await?;
                 self.timeouts
                     .note_hop_completed(0, self.runtime.now() - start_time, true);
                 n_hops_built.fetch_add(1, Ordering::SeqCst);
@@ -263,8 +281,10 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
             OwnedPath::Normal(p) => {
                 assert!(!p.is_empty());
                 let n_hops = p.len() as u8;
+                let timeouts = Arc::clone(&self.timeouts);
                 // Each hop has its own circ parameters. This is for the first hop (CREATE).
-                let circ = C::create(channel, &self.runtime, &p[0], params.clone()).await?;
+                let circ =
+                    C::create(channel, &self.runtime, &p[0], params.clone(), timeouts).await?;
                 self.timeouts
                     .note_hop_completed(0, self.runtime.now() - start_time, n_hops == 0);
                 // If we fail after this point, we can't tell whether it's
@@ -289,6 +309,7 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
     }
 
     /// Build a circuit from an [`OwnedPath`].
+    #[instrument(level = "trace", skip_all)]
     async fn build_owned(
         self: &Arc<Self>,
         path: OwnedPath,
@@ -434,6 +455,7 @@ impl<R: Runtime> TunnelBuilder<R> {
     }
 
     /// Reload persistent state from disk, if we don't have storage permission.
+    #[instrument(level = "trace", skip_all)]
     pub(crate) fn reload_state(&self) -> Result<()> {
         if !self.storage.can_store() {
             self.builder
@@ -452,6 +474,7 @@ impl<R: Runtime> TunnelBuilder<R> {
     }
 
     /// Like `build`, but construct a new circuit from an [`OwnedPath`].
+    #[instrument(level = "trace", skip_all)]
     pub(crate) async fn build_owned(
         &self,
         path: OwnedPath,
@@ -470,6 +493,7 @@ impl<R: Runtime> TunnelBuilder<R> {
     /// This circuit is _not_ automatically registered with any
     /// circuit manager; if you don't hang on it it, it will
     /// automatically go away when the last reference is dropped.
+    #[instrument(level = "trace", skip_all)]
     pub async fn build(
         &self,
         path: &TorPath<'_>,
@@ -583,9 +607,17 @@ fn circparameters_from_netparameters(
         .map_err(into_internal!(
             "Unable to build CongestionControl params from NetParams"
         ))?;
+    let flow_ctrl_params = FlowCtrlParameters {
+        cc_xoff_client: CellCount::new(inp.cc_xoff_client.get_u32()),
+        cc_xoff_exit: CellCount::new(inp.cc_xoff_exit.get_u32()),
+        cc_xon_rate: CellCount::new(inp.cc_xon_rate.get_u32()),
+        cc_xon_change_pct: inp.cc_xon_change_pct.get_u32(),
+        cc_xon_ewma_cnt: inp.cc_xon_ewma_cnt.get_u32(),
+    };
     Ok(CircParameters::new(
         inp.extend_by_ed25519_id.into(),
         ccontrol,
+        flow_ctrl_params,
     ))
 }
 
@@ -698,7 +730,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -899,6 +931,7 @@ mod test {
             rt: &RT,
             ct: &OwnedChanTarget,
             _: CircParameters,
+            _timeouts: Arc<dyn tor_proto::client::circuit::TimeoutEstimator>,
         ) -> Result<Self> {
             let (d1, d2) = timeouts_from_chantarget(ct);
             rt.sleep(d1).await;
@@ -917,6 +950,7 @@ mod test {
             rt: &RT,
             ct: &OwnedCircTarget,
             _: CircParameters,
+            _timeouts: Arc<dyn tor_proto::client::circuit::TimeoutEstimator>,
         ) -> Result<Self> {
             let (d1, d2) = timeouts_from_chantarget(ct);
             rt.sleep(d1).await;
