@@ -7,7 +7,7 @@
 use tor_basic_utils::define_accessor_trait;
 use tor_config::impl_standard_builder;
 use tor_config::{ConfigBuildError, define_list_builder_accessors, define_list_builder_helper};
-use tor_guardmgr::{GuardFilter, GuardMgrConfig};
+use tor_guardmgr::{GuardFilter, GuardMgrConfig, VanguardConfig};
 
 use derive_builder::Builder;
 use serde::{Deserialize, Serialize};
@@ -218,11 +218,24 @@ impl_standard_builder! { PreemptiveCircuitConfig }
 pub struct CircuitTiming {
     /// How long after a circuit has first been used should we give
     /// it out for new requests?
+    ///
+    /// This setting applies to circuits without strong isolation.
+    /// See also [`disused_circuit_timeout`](Self::disused_circuit_timeout)
     #[builder(default = "default_max_dirtiness()")]
     #[builder_field_attr(serde(default, with = "humantime_serde::option"))]
     #[getter(skip)]
     pub(crate) max_dirtiness: Duration,
 
+    /// How long after a circuit has become disused should we discard it?
+    ///
+    /// This setting applies to circuits _with_ strong isolation.
+    /// See also [`max_dirtiness`](Self::max_dirtiness)
+    // TODO: Impose a maximum or minimum?
+    #[builder(default = "default_disused_timeout()")]
+    #[builder_field_attr(serde(default, with = "humantime_serde::option"))]
+    #[getter(skip)]
+    pub(crate) disused_circuit_timeout: Duration,
+
     /// When a circuit is requested, we stop retrying new circuits
     /// after this much time.
     // TODO: Impose a maximum or minimum?
@@ -310,6 +323,11 @@ fn default_max_dirtiness() -> Duration {
     Duration::from_secs(60 * 10)
 }
 
+/// Return the default value for `disused_circuit_timeout`.
+fn default_disused_timeout() -> Duration {
+    Duration::from_secs(60 * 60)
+}
+
 /// Return the default value for `request_timeout`.
 fn default_request_timeout() -> Duration {
     Duration::from_secs(60)
@@ -364,17 +382,9 @@ define_accessor_trait! {
     // cloning all the fields an extra time.
     pub trait CircMgrConfig: GuardMgrConfig {
         path_rules: PathConfig,
+        vanguard_config: VanguardConfig,
         circuit_timing: CircuitTiming,
         preemptive_circuits: PreemptiveCircuitConfig,
-        +
-        // Note: ideally this would be defined in the same way as `path_rules`,
-        // `circuit_timing`, etc., but define_accessor_trait unconditionally adds
-        // AsRef<VanguardsConfig> as a supertrait, which can't be cfg'd behind
-        // the vanguards feature.
-
-        /// Access the field
-        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
-        fn vanguard_config(&self) -> &tor_guardmgr::VanguardConfig;
     }
 }
 
@@ -383,7 +393,6 @@ define_accessor_trait! {
 pub(crate) mod test_config {
     use super::*;
     use crate::*;
-    #[cfg(all(feature = "vanguards", feature = "hs-common"))]
     use tor_guardmgr::VanguardConfig;
     use tor_guardmgr::bridge::BridgeConfig;
 
@@ -397,7 +406,6 @@ pub(crate) mod test_config {
         pub circuit_timing: CircuitTiming,
         pub preemptive_circuits: PreemptiveCircuitConfig,
         pub guardmgr: tor_guardmgr::TestConfig,
-        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
         pub vanguard_config: VanguardConfig,
     }
     impl AsRef<[BridgeConfig]> for TestConfig {
@@ -425,7 +433,6 @@ pub(crate) mod test_config {
         fn preemptive_circuits(&self) -> &PreemptiveCircuitConfig {
             &self.preemptive_circuits
         }
-        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
         fn vanguard_config(&self) -> &tor_guardmgr::VanguardConfig {
             &self.vanguard_config
         }
@@ -443,7 +450,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
