@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -51,6 +51,7 @@ use build::TunnelBuilder;
 use mgr::{AbstractTunnel, AbstractTunnelBuilder};
 use tor_basic_utils::retry::RetryDelay;
 use tor_chanmgr::ChanMgr;
+use tor_dircommon::fallback::FallbackList;
 use tor_error::{error_report, warn_report};
 use tor_guardmgr::RetireCircuits;
 use tor_linkspec::ChanTarget;
@@ -63,10 +64,10 @@ use tor_rtcompat::Runtime;
 use tor_linkspec::IntoOwnedChanTarget;
 
 use futures::StreamExt;
-use futures::task::SpawnExt;
 use std::sync::{Arc, Mutex, Weak};
 use std::time::{Duration, Instant};
-use tracing::{debug, info, trace, warn};
+use tor_rtcompat::SpawnExt;
+use tracing::{debug, info, instrument, trace, warn};
 
 #[cfg(feature = "testing")]
 pub use config::test_config::TestConfig;
@@ -97,7 +98,6 @@ cfg_if::cfg_if! {
 
 pub use err::Error;
 pub use isolation::IsolationToken;
-use tor_guardmgr::fallback::FallbackList;
 pub use tor_guardmgr::{ClockSkewEvents, GuardMgrConfig, SkewEstimate};
 pub use tunnel::{
     ClientDataTunnel, ClientDirTunnel, ClientOnionServiceDataTunnel, ClientOnionServiceDirTunnel,
@@ -227,6 +227,7 @@ impl<R: Runtime> CircMgr<R> {
 
     /// Return a circuit suitable for sending one-hop BEGINDIR streams,
     /// launching it if necessary.
+    #[instrument(level = "trace", skip_all)]
     pub async fn get_or_launch_dir(&self, netdir: DirInfo<'_>) -> Result<ClientDirTunnel> {
         let tunnel = self.0.get_or_launch_dir(netdir).await?;
         Ok(tunnel.into())
@@ -237,6 +238,7 @@ impl<R: Runtime> CircMgr<R> {
     ///
     /// If the list of ports is empty, then the chosen circuit will
     /// still end at _some_ exit.
+    #[instrument(level = "trace", skip_all)]
     pub async fn get_or_launch_exit(
         &self,
         netdir: DirInfo<'_>, // TODO: This has to be a NetDir.
@@ -265,6 +267,7 @@ impl<R: Runtime> CircMgr<R> {
     /// This could be used, for example, to download a descriptor for a bridge.
     #[cfg_attr(docsrs, doc(cfg(feature = "specific-relay")))]
     #[cfg(feature = "specific-relay")]
+    #[instrument(level = "trace", skip_all)]
     pub async fn get_or_launch_dir_specific<T: IntoOwnedChanTarget>(
         &self,
         target: T,
@@ -278,6 +281,7 @@ impl<R: Runtime> CircMgr<R> {
     /// Returns a set of [`TaskHandle`]s that can be used to manage the daemon tasks.
     //
     // NOTE(eta): The ?Sized on D is so we can pass a trait object in.
+    #[instrument(level = "trace", skip_all)]
     pub fn launch_background_tasks<D, S>(
         self: &Arc<Self>,
         runtime: &R,
@@ -343,6 +347,7 @@ impl<R: Runtime> CircMgr<R> {
     /// The actual behavior here will depend on the value of `how`.
     ///
     /// Returns whether any of the circuit pools should be cleared.
+    #[instrument(level = "trace", skip_all)]
     pub fn reconfigure<CFG: CircMgrConfig>(
         &self,
         new_config: &CFG,
@@ -473,6 +478,7 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// Returns a set of [`TaskHandle`]s that can be used to manage the daemon tasks.
     //
     // NOTE(eta): The ?Sized on D is so we can pass a trait object in.
+    #[instrument(level = "trace", skip_all)]
     pub(crate) fn launch_background_tasks<D, S>(
         self: &Arc<Self>,
         runtime: &R,
@@ -545,8 +551,9 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
 
     /// Return a circuit suitable for sending one-hop BEGINDIR streams,
     /// launching it if necessary.
+    #[instrument(level = "trace", skip_all)]
     pub(crate) async fn get_or_launch_dir(&self, netdir: DirInfo<'_>) -> Result<Arc<B::Tunnel>> {
-        self.expire_circuits();
+        self.expire_circuits().await;
         let usage = TargetTunnelUsage::Dir;
         self.mgr.get_or_launch(&usage, netdir).await.map(|(c, _)| c)
     }
@@ -556,6 +563,7 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// If the list of ports is empty, then the chosen circuit will
     /// still end at _some_ exit.
+    #[instrument(level = "trace", skip_all)]
     pub(crate) async fn get_or_launch_exit(
         &self,
         netdir: DirInfo<'_>, // TODO: This has to be a NetDir.
@@ -565,7 +573,7 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
         //             additive. The function should be refactored to be builder-like.
         #[cfg(feature = "geoip")] country_code: Option<CountryCode>,
     ) -> Result<Arc<B::Tunnel>> {
-        self.expire_circuits();
+        self.expire_circuits().await;
         let time = Instant::now();
         {
             let mut predictive = self.predictor.lock().expect("preemptive lock poisoned");
@@ -602,11 +610,12 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// This could be used, for example, to download a descriptor for a bridge.
     #[cfg_attr(docsrs, doc(cfg(feature = "specific-relay")))]
     #[cfg(feature = "specific-relay")]
+    #[instrument(level = "trace", skip_all)]
     pub(crate) async fn get_or_launch_dir_specific<T: IntoOwnedChanTarget>(
         &self,
         target: T,
     ) -> Result<Arc<B::Tunnel>> {
-        self.expire_circuits();
+        self.expire_circuits().await;
         let usage = TargetTunnelUsage::DirSpecificTarget(target.to_owned());
         self.mgr
             .get_or_launch(&usage, DirInfo::Nothing)
@@ -619,6 +628,7 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// The actual behavior here will depend on the value of `how`.
     ///
     /// Returns whether any of the circuit pools should be cleared.
+    #[instrument(level = "trace", skip_all)]
     pub(crate) fn reconfigure<CFG: CircMgrConfig>(
         &self,
         new_config: &CFG,
@@ -688,6 +698,7 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// dangling.
     ///
     /// This is a daemon task: it runs indefinitely in the background.
+    #[instrument(level = "trace", skip_all)]
     async fn keep_circmgr_params_updated<D>(
         mut events: impl futures::Stream<Item = DirEvent> + Unpin,
         circmgr: Weak<Self>,
@@ -723,6 +734,7 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// Exit when we notice that `circmgr` or `dirmgr` has been dropped.
     ///
     /// This is a daemon task: it runs indefinitely in the background.
+    #[instrument(level = "trace", skip_all)]
     async fn continually_launch_timeout_testing_circuits<D>(
         mut sched: TaskSchedule<R>,
         circmgr: Weak<Self>,
@@ -733,7 +745,10 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
         while sched.next().await.is_some() {
             if let (Some(cm), Some(dm)) = (Weak::upgrade(&circmgr), Weak::upgrade(&dirmgr)) {
                 if let Ok(netdir) = dm.netdir(Timeliness::Unchecked) {
-                    if let Err(e) = cm.launch_timeout_testing_circuit_if_appropriate(&netdir) {
+                    if let Err(e) = cm
+                        .launch_timeout_testing_circuit_if_appropriate(&netdir)
+                        .await
+                    {
                         warn_report!(e, "Problem launching a timeout testing circuit");
                     }
                     let delay = netdir
@@ -763,13 +778,13 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// This function is invoked periodically from
     /// `continually_launch_timeout_testing_circuits`.
-    fn launch_timeout_testing_circuit_if_appropriate(&self, netdir: &NetDir) -> Result<()> {
+    async fn launch_timeout_testing_circuit_if_appropriate(&self, netdir: &NetDir) -> Result<()> {
         if !self.mgr.peek_builder().learning_timeouts() {
             return Ok(());
         }
         // We expire any too-old circuits here, so they don't get
         // counted towards max_circs.
-        self.expire_circuits();
+        self.expire_circuits().await;
         let max_circs: u64 = netdir
             .params()
             .cbt_max_open_circuits_for_testing
@@ -797,6 +812,7 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// This is a daemon task: it runs indefinitely in the background.
     #[allow(clippy::cognitive_complexity)] // because of tracing
+    #[instrument(level = "trace", skip_all)]
     async fn update_persistent_state<S>(
         mut sched: TaskSchedule<R>,
         circmgr: Weak<Self>,
@@ -877,6 +893,7 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// This would be better handled entirely within `tor-circmgr`, like
     /// other daemon tasks.
+    #[instrument(level = "trace", skip_all)]
     async fn continually_preemptively_build_circuits<D>(
         mut sched: TaskSchedule<R>,
         circmgr: Weak<Self>,
@@ -923,6 +940,7 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// This function is invoked periodically from
     /// `continually_preemptively_build_circuits()`.
     #[allow(clippy::cognitive_complexity)]
+    #[instrument(level = "trace", skip_all)]
     async fn launch_circuits_preemptively(
         &self,
         netdir: DirInfo<'_>,
@@ -990,6 +1008,7 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// Used to implement onion service clients and services.
     #[cfg(feature = "hs-common")]
+    #[instrument(level = "trace", skip_all)]
     pub(crate) async fn launch_hs_unmanaged<T>(
         &self,
         planned_target: Option<T>,
@@ -1047,14 +1066,16 @@ impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// Expired circuits are not closed while they still have users,
     /// but they are no longer given out for new requests.
-    fn expire_circuits(&self) {
+    async fn expire_circuits(&self) {
         // TODO: I would prefer not to call this at every request, but
         // it should be fine for now.  (At some point we may no longer
         // need this, or might not need to call it so often, now that
         // our circuit expiration runs on scheduled timers via
         // spawn_expiration_task.)
         let now = self.mgr.peek_runtime().now();
-        self.mgr.expire_tunnels(now);
+
+        // TODO: Use return value here to implement the above TODO.
+        let _next_expiration = self.mgr.expire_tunnels(now).await;
     }
 
     /// Mark every circuit that we have launched so far as unsuitable for
@@ -1134,7 +1155,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
