@@ -3,6 +3,7 @@
 use std::time::SystemTime;
 
 use rand::Rng;
+use tracing::instrument;
 
 use super::{AnonymousPathBuilder, TorPath};
 use crate::path::pick_path;
@@ -104,6 +105,7 @@ impl ExitPathBuilder {
 
     /// Try to create and return a path corresponding to the requirements of
     /// this builder.
+    #[instrument(skip_all, level = "trace")]
     pub(crate) fn pick_path<'a, R: Rng, RT: Runtime>(
         &self,
         rng: &mut R,
@@ -202,19 +204,19 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
     use crate::path::{
-        assert_same_path_when_owned, MaybeOwnedRelay, OwnedPath, TorPath, TorPathInner,
+        MaybeOwnedRelay, OwnedPath, TorPath, TorPathInner, assert_same_path_when_owned,
     };
     use std::collections::HashSet;
     use tor_basic_utils::test_rng::testing_rng;
     use tor_guardmgr::TestConfig;
     use tor_linkspec::{HasRelayIds, RelayIds};
-    use tor_netdir::{testnet, FamilyRules, SubnetConfig};
+    use tor_netdir::{FamilyRules, SubnetConfig, testnet};
     use tor_persist::TestingStateMgr;
     use tor_relay_selection::LowLevelRelayPredicate;
     use tor_rtcompat::SleepProvider;
