@@ -182,8 +182,6 @@ impl ConfigurationSources {
     // The other inputs are always used and therefore
     // don't need to be lifted into FnOnce() -> Result.
     ///
-    /// `mistrust` is used to check whether the configuration files have appropriate permissions.
-    ///
     /// `ConfigurationSource::Dir`s
     /// will be scanned for files whose name ends in `.toml`.
     /// All those files (if any) will be read (in lexical order by filename).
@@ -238,6 +236,13 @@ impl ConfigurationSources {
         self.options.push(option.into());
     }
 
+    /// All given override options.
+    ///
+    /// See [`push_option`](Self::push_option).
+    pub fn options(&self) -> impl Iterator<Item = &String> + Clone {
+        self.options.iter()
+    }
+
     /// Sets the filesystem permission mistrust
     ///
     /// This value only indicates whether and how to check permissions
@@ -456,7 +461,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
