@@ -26,7 +26,7 @@ use tor_dirclient::DirResponse;
 use tor_error::{info_report, warn_report};
 use tor_rtcompat::Runtime;
 use tor_rtcompat::scheduler::TaskSchedule;
-use tracing::{debug, info, trace, warn};
+use tracing::{debug, info, instrument, trace, warn};
 
 use crate::storage::Store;
 #[cfg(test)]
@@ -227,10 +227,10 @@ pub(crate) fn make_consensus_request(
     request.set_skew_limit(
         // If we are _fast_ by at least this much, then any valid directory will
         // seem to be at least this far in the past.
-        config.tolerance.post_valid_tolerance,
+        config.tolerance.post_valid_tolerance(),
         // If we are _slow_ by this much, then any valid directory will seem to
         // be at least this far in the future.
-        config.tolerance.pre_valid_tolerance,
+        config.tolerance.pre_valid_tolerance(),
     );
 
     Ok(ClientRequest::Consensus(request))
@@ -273,6 +273,7 @@ pub(crate) fn make_requests_for_documents<R: Runtime>(
 }
 
 /// Launch a single client request and get an associated response.
+#[instrument(level = "trace", skip_all)]
 async fn fetch_single<R: Runtime>(
     rt: &R,
     request: ClientRequest,
@@ -305,6 +306,7 @@ static CANNED_RESPONSE: LazyLock<Mutex<Vec<String>>> = LazyLock::new(|| Mutex::n
 ///
 /// Don't launch more than `parallelism` requests at once.
 #[allow(clippy::cognitive_complexity)] // TODO: maybe refactor?
+#[instrument(level = "trace", skip_all)]
 async fn fetch_multiple<R: Runtime>(
     dirmgr: Arc<DirMgr<R>>,
     attempt_id: AttemptId,
@@ -458,6 +460,7 @@ pub(crate) async fn load<R: Runtime>(
 /// This can launch one or more download requests, but will not launch more
 /// than `parallelism` requests at a time.
 #[allow(clippy::cognitive_complexity)] // TODO: Refactor?
+#[instrument(level = "trace", skip_all)]
 async fn download_attempt<R: Runtime>(
     dirmgr: &Arc<DirMgr<R>>,
     state: &mut Box<dyn DirState>,
@@ -542,6 +545,7 @@ async fn download_attempt<R: Runtime>(
 /// The first time that the state becomes ["usable"](Readiness::Usable), notify
 /// the sender in `on_usable`.
 #[allow(clippy::cognitive_complexity)] // TODO: Refactor!
+#[instrument(level = "trace", skip_all)]
 pub(crate) async fn download<R: Runtime>(
     dirmgr: Weak<DirMgr<R>>,
     state: &mut Box<dyn DirState>,
@@ -733,15 +737,15 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
-    use crate::DownloadSchedule;
     use crate::storage::DynStore;
     use crate::test::new_mgr;
     use std::sync::Mutex;
+    use tor_dircommon::retry::DownloadSchedule;
     use tor_netdoc::doc::microdesc::MdDigest;
     use tor_rtcompat::SleepProvider;
 
