@@ -1025,13 +1025,14 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use tor_linkspec::{HasRelayIds, RelayIdType};
     use tor_netdir::NetDir;
-    use tor_netdoc::doc::netstatus::{RelayFlags, RelayWeight};
+    use tor_netdoc::doc::netstatus::RelayWeight;
+    use tor_netdoc::types::relay_flags::RelayFlag;
 
     use super::*;
     use crate::FirstHopId;
@@ -1054,7 +1055,7 @@ mod test {
             // The default network has 40 relays, and the first 10 are
             // not Guard by default.
             if idx >= 10 {
-                builder.rs.add_flags(RelayFlags::GUARD);
+                builder.rs.add_flags(RelayFlag::Guard);
                 if idx >= 20 {
                     builder.rs.protos("DirCache=2".parse().unwrap());
                 } else {
