@@ -12,12 +12,12 @@ use std::sync::{Arc, RwLock, Weak};
 use std::time::{Duration, SystemTime};
 
 use futures::stream::BoxStream;
-use futures::task::SpawnExt as _;
 use futures::{FutureExt as _, future};
 use futures::{StreamExt as _, select_biased};
 use postage::stream::Stream as _;
 use postage::watch;
 use rand::RngCore;
+use tor_rtcompat::SpawnExt as _;
 
 use tor_async_utils::PostageWatchSenderExt as _;
 use tor_config::ReconfigureError;
@@ -26,7 +26,7 @@ use tor_netdir::{DirEvent, NetDir, NetDirProvider, Timeliness};
 use tor_persist::{DynStorageHandle, StateMgr};
 use tor_relay_selection::RelaySelector;
 use tor_rtcompat::Runtime;
-use tracing::{debug, info};
+use tracing::{debug, info, instrument};
 
 use crate::{RetireCircuits, VanguardMode};
 
@@ -173,6 +173,7 @@ impl<R: Runtime> VanguardMgr<R> {
     /// These run until the `VanguardMgr` is dropped.
     //
     // This spawns [`VanguardMgr::maintain_vanguard_sets`].
+    #[instrument(level = "trace", skip_all)]
     pub fn launch_background_tasks(
         self: &Arc<Self>,
         netdir_provider: &Arc<dyn NetDirProvider>,
@@ -600,7 +601,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
