@@ -8,13 +8,13 @@ use std::sync::{Arc, Mutex, MutexGuard};
 use std::time::{Duration, Instant};
 
 use futures::FutureExt as _;
-use futures::task::{SpawnError, SpawnExt as _};
+use futures::task::SpawnError;
 
 use async_trait::async_trait;
 use educe::Educe;
 use either::Either::{self, *};
 use postage::stream::Stream as _;
-use tracing::{debug, error, trace};
+use tracing::{debug, error, instrument, trace};
 
 use safelog::DisplayRedacted as _;
 use tor_basic_utils::define_accessor_trait;
@@ -22,7 +22,7 @@ use tor_circmgr::isolation::Isolation;
 use tor_error::{Bug, ErrorReport as _, debug_report, error_report, internal};
 use tor_hscrypto::pk::HsId;
 use tor_netdir::NetDir;
-use tor_rtcompat::Runtime;
+use tor_rtcompat::{Runtime, SpawnExt as _};
 
 use crate::isol_map;
 use crate::{ConnError, HsClientConnector, HsClientSecretKeys};
@@ -522,6 +522,7 @@ impl<D: MockableConnectorData> Services<D> {
 
     /// Connect to a hidden service
     // We *do* drop guard.  There is *one* await point, just after drop(guard).
+    #[instrument(skip_all, level = "trace")]
     pub(crate) async fn get_or_launch_connection(
         connector: &HsClientConnector<impl Runtime, D>,
         netdir: &Arc<NetDir>,
@@ -744,7 +745,7 @@ pub(crate) mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
