@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -100,6 +100,7 @@ pub use anon_level::Anonymity;
 pub use config::OnionServiceConfig;
 pub use err::{ClientError, EstablishSessionError, FatalError, IntroRequestError, StartupError};
 pub use ipt_mgr::IptError;
+use keys::HsTimePeriodKeySpecifier;
 pub use keys::{
     BlindIdKeypairSpecifier, BlindIdPublicKeySpecifier, DescSigningKeypairSpecifier,
     HsIdKeypairSpecifier, HsIdPublicKeySpecifier,
@@ -108,10 +109,14 @@ use pow::{NewPowManager, PowManager};
 pub use publish::UploadError as DescUploadError;
 pub use req::{RendRequest, StreamRequest};
 pub use tor_hscrypto::pk::HsId;
+use tor_keymgr::KeystoreEntry;
 pub use tor_persist::hsnickname::{HsNickname, InvalidNickname};
 
 pub use helpers::handle_rend_requests;
 
+#[cfg(feature = "onion-service-cli-extra")]
+use tor_netdir::NetDir;
+
 //---------- top-level service implementation (types and methods) ----------
 
 /// Convenience alias for link specifiers of an intro point
@@ -125,8 +130,9 @@ pub(crate) type NtorPublicKey = curve25519::PublicKey;
 
 /// A handle to a running instance of an onion service.
 //
-// TODO (#1228): Write more.
-// TODO (#1247): Choose a better name for this struct
+/// To construct a `RunningOnionService`, use [`OnionServiceBuilder`]
+/// to build an [`OnionService`], and then call its
+/// [``.launch()``](OnionService::launch) method.
 //
 // (APIs should return Arc<OnionService>)
 #[must_use = "a hidden service object will terminate the service when dropped"]
@@ -219,7 +225,7 @@ impl From<oneshot::Canceled> for ShutdownStatus {
     }
 }
 
-/// A handle to an instance of an onion service, which may or may not be running.
+/// A handle to an instance of an onion service.
 ///
 /// To construct an `OnionService`, use [`OnionServiceBuilder`].
 /// It will not start handling requests until you call its
@@ -247,6 +253,8 @@ impl OnionService {
     /// Tell this onion service to begin running, and return a
     /// [`RunningOnionService`] and its stream of rendezvous requests.
     ///
+    /// Returns `Ok(None)` if the service specified is disabled in the config.
+    ///
     /// You can turn the resulting stream into a stream of [`StreamRequest`]
     /// using the [`handle_rend_requests`] helper function.
     ///
@@ -259,7 +267,7 @@ impl OnionService {
         netdir_provider: Arc<dyn NetDirProvider>,
         circ_pool: Arc<HsCircPool<R>>,
         path_resolver: Arc<tor_config_path::CfgPathResolver>,
-    ) -> Result<(Arc<RunningOnionService>, impl Stream<Item = RendRequest>), StartupError>
+    ) -> Result<Option<(Arc<RunningOnionService>, impl Stream<Item = RendRequest>)>, StartupError>
     where
         R: Runtime,
     {
@@ -280,6 +288,10 @@ impl OnionService {
         let selector = KeystoreSelector::Primary;
         maybe_generate_hsid(&keymgr, &config.nickname, offline_hsid, selector)?;
 
+        if !config.enabled() {
+            return Ok(None);
+        }
+
         if config.restricted_discovery.enabled {
             info!(
                 nickname=%nickname,
@@ -381,7 +393,7 @@ impl OnionService {
         });
 
         let stream = svc.launch()?;
-        Ok((svc, stream))
+        Ok(Some((svc, stream)))
     }
 
     /// Return the onion address of this service.
@@ -429,6 +441,22 @@ impl OnionService {
 
         maybe_generate_hsid(&self.keymgr, &self.config.nickname, offline_hsid, selector)
     }
+
+    /// List the no-longer-relevant keys of this service.
+    ///
+    /// Returns the [`KeystoreEntry`]s associated with time periods that are not
+    /// "relevant" according to the specified [`NetDir`],
+    /// (i.e. the keys associated with time periods
+    /// the service is not publishing descriptors for).
+    // TODO: unittest
+    #[cfg(feature = "onion-service-cli-extra")]
+    pub fn list_expired_keys(&self, netdir: &NetDir) -> tor_keymgr::Result<Vec<KeystoreEntry>> {
+        list_expired_keys_for_service(
+            &netdir.hs_all_time_periods(),
+            self.config.nickname(),
+            &self.keymgr,
+        )
+    }
 }
 
 impl OnionServiceBuilder {
@@ -644,6 +672,54 @@ pub fn supported_hsservice_protocols() -> tor_protover::Protocols {
     .collect()
 }
 
+/// Returns all the keys (as [`KeystoreEntry`]) of the service
+/// identified by `nickname` that are expired according to the
+/// provided [`HsDirParams`].
+fn list_expired_keys_for_service<'a>(
+    relevant_periods: &[HsDirParams],
+    nickname: &HsNickname,
+    keymgr: &'a KeyMgr,
+) -> tor_keymgr::Result<Vec<KeystoreEntry<'a>>> {
+    let arti_pat = tor_keymgr::KeyPathPattern::Arti(format!("hss/{}/*", nickname));
+    let possibly_relevant_keys = keymgr.list_matching(&arti_pat)?;
+    let mut expired_keys = Vec::new();
+
+    for entry in possibly_relevant_keys {
+        let key_path = entry.key_path();
+        let mut append_if_expired = |spec: &dyn HsTimePeriodKeySpecifier| {
+            if spec.nickname() != nickname {
+                return Err(internal!(
+                    "keymgr gave us key {spec:?} that doesn't match our pattern {arti_pat:?}"
+                )
+                .into());
+            }
+            let is_expired = relevant_periods
+                .iter()
+                .all(|p| &p.time_period() != spec.period());
+
+            if is_expired {
+                expired_keys.push(entry.clone());
+            }
+
+            tor_keymgr::Result::Ok(())
+        };
+
+        macro_rules! append_if_expired {
+            ($K:ty) => {{
+                if let Ok(spec) = <$K>::try_from(key_path) {
+                    append_if_expired(&spec)?;
+                }
+            }};
+        }
+
+        append_if_expired!(BlindIdPublicKeySpecifier);
+        append_if_expired!(BlindIdKeypairSpecifier);
+        append_if_expired!(DescSigningKeypairSpecifier);
+    }
+
+    Ok(expired_keys)
+}
+
 #[cfg(test)]
 pub(crate) mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -655,7 +731,7 @@ pub(crate) mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
