@@ -63,8 +63,8 @@ pub trait KeystoreError:
 
 impl HasKind for Error {
     fn kind(&self) -> tor_error::ErrorKind {
-        use tor_error::ErrorKind as EK;
         use Error as E;
+        use tor_error::ErrorKind as EK;
 
         match self {
             E::Keystore(e) => e.kind(),
@@ -181,7 +181,7 @@ mod tests {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
