@@ -8,9 +8,9 @@ use std::result::Result as StdResult;
 use std::str::FromStr as _;
 use std::sync::Arc;
 
-use crate::keystore::ctor::err::{CTorKeystoreError, MalformedClientKeyError};
 use crate::keystore::ctor::CTorKeystore;
-use crate::keystore::fs_utils::{checked_op, FilesystemAction, FilesystemError, RelKeyPath};
+use crate::keystore::ctor::err::{CTorKeystoreError, MalformedClientKeyError};
+use crate::keystore::fs_utils::{FilesystemAction, FilesystemError, RelKeyPath, checked_op};
 use crate::keystore::{EncodableItem, ErasedKey, KeySpecifier, Keystore};
 use crate::raw::{RawEntryId, RawKeystoreEntry};
 use crate::{
@@ -175,7 +175,7 @@ impl CTorClientKeystore {
             let content = match self.read_key(path) {
                 Ok(c) => c,
                 Err(e) => {
-                    debug_report!(e.clone(), "failed to read {}", path.display_lossy());
+                    debug_report!(&e, "failed to read {}", path.display_lossy());
                     return Some(Err(e));
                 }
             }?;
@@ -337,15 +337,15 @@ mod tests {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
     use std::fs;
-    use tempfile::{tempdir, TempDir};
+    use tempfile::{TempDir, tempdir};
 
-    use crate::test_utils::{assert_found, DummyKey, TestCTorSpecifier};
+    use crate::test_utils::{DummyKey, TestCTorSpecifier, assert_found};
 
     #[cfg(unix)]
     use std::os::unix::fs::PermissionsExt;
