@@ -4,7 +4,7 @@
 
 use crate::raw::{RawEntryId, RawKeystoreEntry};
 use crate::{
-    ArtiPath, BoxedKeystore, KeyCertificateSpecifier, KeyPath, KeyPathError, KeyPathInfo,
+    ArtiPath, BoxedKeystore, Error, KeyCertificateSpecifier, KeyPath, KeyPathError, KeyPathInfo,
     KeyPathInfoExtractor, KeyPathPattern, KeySpecifier, KeystoreCorruptionError,
     KeystoreEntryResult, KeystoreId, KeystoreSelector, Result,
 };
@@ -235,6 +235,10 @@ impl KeyMgr {
     /// Returns `Ok(None)` if none of the key stores have the requested key.
     ///
     /// Returns an error if the specified keystore does not exist.
+    // TODO: The function takes `&KeystoreId`, but it would be better to accept a
+    // `KeystoreSelector`.
+    // This way, the caller can pass `KeystoreSelector::Primary` directly without
+    // needing to know the specific `KeystoreId` of the primary keystore.
     #[cfg(feature = "onion-service-cli-extra")]
     pub fn get_from<K: ToEncodableKey>(
         &self,
@@ -245,6 +249,38 @@ impl KeyMgr {
         self.get_from_store(key_spec, &K::Key::item_type(), store)
     }
 
+    /// Validates the integrity of a [`KeystoreEntry`].
+    ///
+    /// This retrieves the key corresponding to the provided [`KeystoreEntry`],
+    /// and checks if its contents are valid (i.e. that the key can be parsed).
+    /// The [`KeyPath`] of the entry is further validated using [`describe`](KeyMgr::describe).
+    ///
+    /// NOTE: Currently, ctor entries cannot be validated using [`describe`](KeyMgr::describe), so they
+    /// are considered valid if the manager successfully retrieves the corresponding keys,
+    /// but are otherwise not validated by this procedure.
+    ///
+    /// Returns `Ok(())` if the specified keystore entry is valid, and `Err` otherwise.
+    ///
+    /// NOTE: If the specified entry does not exist, this will only validate its [`KeyPath`].
+    #[cfg(feature = "onion-service-cli-extra")]
+    pub fn validate_entry_integrity(&self, entry: &KeystoreEntry) -> Result<()> {
+        let selector = entry.keystore_id().into();
+        let store = self.select_keystore(&selector)?;
+        // Ignore the parsed key, only checking if it parses correctly
+        let _ = store.get(entry.key_path(), entry.key_type())?;
+
+        // TODO: Implement `describe()` support for CTor keystore entries
+        if !matches!(entry.key_path(), KeyPath::CTor(_)) {
+            // Ignore the result, just checking if the path is recognized
+            let _ = self
+                .describe(entry.key_path())
+                // TODO: `Error::Corruption` might not be the best fit for this situation.
+                .map_err(|e| Error::Corruption(e.into()))?;
+        }
+
+        Ok(())
+    }
+
     /// Generate a new key of type `K`, and insert it into the key store specified by `selector`.
     ///
     /// If the key already exists in the specified key store, the `overwrite` flag is used to
@@ -252,8 +288,8 @@ impl KeyMgr {
     ///
     /// On success, this function returns the newly generated key.
     ///
-    /// Returns [`Error::KeyAlreadyExists`](crate::Error::KeyAlreadyExists)
-    /// if the key already exists in the specified key store and `overwrite` is `false`.
+    /// Returns [`Error::KeyAlreadyExists`] if the key already exists in the specified
+    /// key store and `overwrite` is `false`.
     ///
     /// **IMPORTANT**: using this function concurrently with any other `KeyMgr` operation that
     /// mutates the key store state is **not** recommended, as it can yield surprising results! The
@@ -414,7 +450,7 @@ impl KeyMgr {
             .collect::<Result<Vec<_>>>()
     }
 
-    /// List keys and certificates of a specific keystore.
+    /// List all the configured keystore.
     #[cfg(feature = "onion-service-cli-extra")]
     pub fn list_keystores(&self) -> Vec<KeystoreId> {
         self.all_stores()
@@ -744,7 +780,7 @@ mod tests {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -1443,9 +1479,11 @@ mod tests {
 
         let mgr = builder.build().unwrap();
 
-        assert!(!mgr.secondary_stores[0]
-            .contains(&TestKeySpecifier1, &TestItem::item_type())
-            .unwrap());
+        assert!(
+            !mgr.secondary_stores[0]
+                .contains(&TestKeySpecifier1, &TestItem::item_type())
+                .unwrap()
+        );
 
         // Insert a key into Keystore2
         mgr.insert(
@@ -1464,27 +1502,33 @@ mod tests {
         assert_eq!(key.meta.is_generated(), false);
 
         // Try to remove the key from a non-existent key store
-        assert!(mgr
-            .remove::<TestItem>(
+        assert!(
+            mgr.remove::<TestItem>(
                 &TestKeySpecifier1,
                 KeystoreSelector::Id(&KeystoreId::from_str("not_an_id_we_know_of").unwrap())
             )
-            .is_err());
+            .is_err()
+        );
         // The key still exists in Keystore2
-        assert!(mgr.secondary_stores[0]
-            .contains(&TestKeySpecifier1, &TestItem::item_type())
-            .unwrap());
+        assert!(
+            mgr.secondary_stores[0]
+                .contains(&TestKeySpecifier1, &TestItem::item_type())
+                .unwrap()
+        );
 
         // Try to remove the key from the primary key store
-        assert!(mgr
-            .remove::<TestItem>(&TestKeySpecifier1, KeystoreSelector::Primary)
-            .unwrap()
-            .is_none());
+        assert!(
+            mgr.remove::<TestItem>(&TestKeySpecifier1, KeystoreSelector::Primary)
+                .unwrap()
+                .is_none()
+        );
 
         // The key still exists in Keystore2
-        assert!(mgr.secondary_stores[0]
-            .contains(&TestKeySpecifier1, &TestItem::item_type())
-            .unwrap());
+        assert!(
+            mgr.secondary_stores[0]
+                .contains(&TestKeySpecifier1, &TestItem::item_type())
+                .unwrap()
+        );
 
         // Removing from Keystore2 should succeed.
         let removed_key = mgr
@@ -1502,9 +1546,11 @@ mod tests {
         assert_eq!(removed_key.meta.is_generated(), false);
 
         // The key doesn't exist in Keystore2 anymore
-        assert!(!mgr.secondary_stores[0]
-            .contains(&TestKeySpecifier1, &TestItem::item_type())
-            .unwrap());
+        assert!(
+            !mgr.secondary_stores[0]
+                .contains(&TestKeySpecifier1, &TestItem::item_type())
+                .unwrap()
+        );
     }
 
     #[test]
@@ -1524,10 +1570,11 @@ mod tests {
         .unwrap();
 
         // There is no corresponding public key entry.
-        assert!(mgr
-            .get::<TestPublicKey>(&TestPublicKeySpecifier1)
-            .unwrap()
-            .is_none());
+        assert!(
+            mgr.get::<TestPublicKey>(&TestPublicKeySpecifier1)
+                .unwrap()
+                .is_none()
+        );
 
         // Try to generate a new key (overwrite = false)
         let err = mgr
@@ -1551,10 +1598,11 @@ mod tests {
         assert_eq!(key.meta.is_generated(), false);
 
         // We don't store public keys in the keystore
-        assert!(mgr
-            .get::<TestPublicKey>(&TestPublicKeySpecifier1)
-            .unwrap()
-            .is_none());
+        assert!(
+            mgr.get::<TestPublicKey>(&TestPublicKeySpecifier1)
+                .unwrap()
+                .is_none()
+        );
 
         // Try to generate a new key (overwrite = true)
         let generated_key = mgr
@@ -1582,10 +1630,11 @@ mod tests {
         assert_eq!(retrieved_key.meta.is_generated(), true);
 
         // We don't store public keys in the keystore
-        assert!(mgr
-            .get::<TestPublicKey>(&TestPublicKeySpecifier1)
-            .unwrap()
-            .is_none());
+        assert!(
+            mgr.get::<TestPublicKey>(&TestPublicKeySpecifier1)
+                .unwrap()
+                .is_none()
+        );
     }
 
     #[test]
