@@ -194,11 +194,9 @@ impl<H: HasRelayIds> ListByRelayIds<H> {
     ///
     /// Returns an empty iterator if `key` has no relay IDs.
     pub fn by_all_ids<'a>(&'a self, key: &'a impl HasRelayIds) -> impl Iterator<Item = &'a H> + 'a {
-        // TODO: see comments on `empty_iterator`
-        #[allow(deprecated)]
         key.identities()
             .next()
-            .map_or_else(|| self.empty_iterator(), |id| self.by_id(id))
+            .map_or_else(Default::default, |id| self.by_id(id))
             .filter(|val| val.has_all_relay_ids_from(key))
     }
 
@@ -303,7 +301,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
