@@ -25,7 +25,7 @@
 /// or other aspects of the HSDir ring structure.)
 use std::time::{Duration, SystemTime};
 
-use crate::{params::NetParameters, Error, HsDirs, Result};
+use crate::{Error, HsDirs, Result, params::NetParameters};
 use time::{OffsetDateTime, UtcOffset};
 use tor_hscrypto::time::TimePeriod;
 use tor_netdoc::doc::netstatus::{MdConsensus, SharedRandVal};
@@ -291,13 +291,13 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
     use hex_literal::hex;
-    use tor_netdoc::doc::netstatus::{ConsensusBuilder, Lifetime, MdConsensusRouterStatus};
+    use tor_netdoc::doc::netstatus::{Lifetime, MdConsensusBuilder};
 
     /// Helper: parse an rfc3339 time.
     ///
@@ -328,7 +328,7 @@ mod test {
     const SRV1: [u8; 32] = *b"next saturday night were sending";
     const SRV2: [u8; 32] = *b"you......... back to the future!";
 
-    fn example_consensus_builder() -> ConsensusBuilder<MdConsensusRouterStatus> {
+    fn example_consensus_builder() -> MdConsensusBuilder {
         let mut bld = MdConsensus::builder();
 
         bld.consensus_method(34)
