@@ -204,6 +204,11 @@ impl HsDescInner {
     ///
     /// On success, return the signing key that was used for every certificate in the
     /// inner document, and the inner document itself.
+    //
+    // TODO: replace Itertools::exactly_one() with a stdlib equivalent when there is one.
+    //
+    // See issue #48919 <https://github.com/rust-lang/rust/issues/48919>
+    #[allow(unstable_name_collisions)]
     fn take_from_reader(
         input: &mut NetDocReader<'_, HsInnerKwd>,
     ) -> Result<(Option<Ed25519Identity>, UncheckedHsDescInner)> {
@@ -449,7 +454,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
