@@ -21,40 +21,31 @@ ns_use_this_variety! {
     pub use [crate::doc::netstatus::rs::build]::?::{RouterStatusBuilder};
 }
 
-/// A single microdescriptor consensus netstatus
-///
-/// TODO: This should possibly turn into a parameterized type, to represent
-/// votes and ns consensuses.
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
+/// A single consensus netstatus, as produced by the old parser.
 #[derive(Debug, Clone)]
+#[non_exhaustive]
 pub struct Consensus {
-    /// Part of the header shared by all consensus types.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    pub(crate) header: Header,
+    /// What kind of consensus document is this?  Absent in votes and
+    /// in ns-flavored consensuses.
+    pub flavor: ConsensusFlavor,
+    /// The preamble, except for the intro item.
+    pub preamble: Preamble,
     /// List of voters whose votes contributed to this consensus.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    pub(crate) voters: Vec<ConsensusVoterInfo>,
+    pub voters: Vec<ConsensusVoterInfo>,
     /// A list of routerstatus entries for the relays on the network,
     /// with one entry per relay.
     ///
     /// These are currently ordered by the router's RSA identity, but this is not
     /// to be relied on, since we may want to even abolish RSA at some point!
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    pub(crate) relays: Vec<RouterStatus>,
+    pub relays: Vec<RouterStatus>,
     /// Footer for the consensus object.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    pub(crate) footer: Footer,
+    pub footer: Footer,
 }
 
 impl Consensus {
     /// Return the Lifetime for this consensus.
     pub fn lifetime(&self) -> &Lifetime {
-        &self.header.lifetime
+        &self.preamble.lifetime
     }
 
     /// Return a slice of all the routerstatus entries in this consensus.
@@ -70,36 +61,36 @@ impl Consensus {
 
     /// Return the map of network parameters that this consensus advertises.
     pub fn params(&self) -> &NetParams<i32> {
-        &self.header.params
+        &self.preamble.params
     }
 
     /// Return the latest shared random value, if the consensus
     /// contains one.
     pub fn shared_rand_cur(&self) -> Option<&SharedRandStatus> {
-        self.header.shared_rand_cur.as_ref()
+        self.preamble.shared_rand_current_value.as_ref()
     }
 
     /// Return the previous shared random value, if the consensus
     /// contains one.
     pub fn shared_rand_prev(&self) -> Option<&SharedRandStatus> {
-        self.header.shared_rand_prev.as_ref()
+        self.preamble.shared_rand_previous_value.as_ref()
     }
 
     /// Return a [`ProtoStatus`] that lists the network's current requirements and
     /// recommendations for the list of protocols that every relay must implement.  
     pub fn relay_protocol_status(&self) -> &ProtoStatus {
-        &self.header.proto_statuses.relay
+        &self.preamble.proto_statuses.relay
     }
 
     /// Return a [`ProtoStatus`] that lists the network's current requirements and
     /// recommendations for the list of protocols that every client must implement.
     pub fn client_protocol_status(&self) -> &ProtoStatus {
-        &self.header.proto_statuses.client
+        &self.preamble.proto_statuses.client
     }
 
     /// Return a set of all known [`ProtoStatus`] values.
     pub fn protocol_statuses(&self) -> &Arc<ProtoStatuses> {
-        &self.header.proto_statuses
+        &self.preamble.proto_statuses
     }
 }
 
@@ -208,20 +199,20 @@ impl Consensus {
         r: &mut NetDocReader<'a, NetstatusKwd>,
     ) -> Result<(&'a str, &'a str, UncheckedConsensus)> {
         use NetstatusKwd::*;
-        let (header, start_pos) = {
+        let ((flavor, preamble), start_pos) = {
             let mut h = r.pause_at(|i| i.is_ok_with_kwd_in(&[DIR_SOURCE]));
-            let header_sec = NS_HEADER_RULES_CONSENSUS.parse(&mut h)?;
+            let preamble_sec = NS_HEADER_RULES_CONSENSUS.parse(&mut h)?;
             // Unwrapping should be safe because above `.parse` would have
             // returned an Error
             #[allow(clippy::unwrap_used)]
-            let pos = header_sec.first_item().unwrap().offset_in(r.str()).unwrap();
-            (Header::from_section(&header_sec)?, pos)
+            let pos = preamble_sec.first_item().unwrap().offset_in(r.str()).unwrap();
+            (Preamble::from_section(&preamble_sec)?, pos)
         };
-        if RouterStatus::flavor() != header.flavor {
+        if RouterStatus::flavor() != flavor {
             return Err(EK::BadDocumentType.with_msg(format!(
                 "Expected {:?}, got {:?}",
                 RouterStatus::flavor(),
-                header.flavor
+                flavor
             )));
         }
 
@@ -245,7 +236,8 @@ impl Consensus {
         let footer = Self::take_footer(r)?;
 
         let consensus = Consensus {
-            header,
+            flavor,
+            preamble,
             voters,
             relays,
             footer,
@@ -300,18 +292,18 @@ impl Consensus {
             siggroup,
             n_authorities: None,
         };
-        let lifetime = unval.consensus.header.lifetime.clone();
-        let delay = unval.consensus.header.voting_delay.unwrap_or((0, 0));
+        let lifetime = unval.consensus.preamble.lifetime.clone();
+        let delay = unval.consensus.preamble.voting_delay.unwrap_or((0, 0));
         let dist_interval = time::Duration::from_secs(delay.1.into());
-        let starting_time = lifetime.valid_after - dist_interval;
-        let timebound = TimerangeBound::new(unval, starting_time..lifetime.valid_until);
+        let starting_time = *lifetime.valid_after - dist_interval;
+        let timebound = TimerangeBound::new(unval, starting_time..*lifetime.valid_until);
         Ok((signed_str, remainder, timebound))
     }
 }
 
-impl Header {
-    /// Extract the CommonHeader members from a single header section.
-    fn from_section(sec: &Section<'_, NetstatusKwd>) -> Result<Header> {
+impl Preamble {
+    /// Extract the CommonPreamble members from a single preamble section.
+    fn from_section(sec: &Section<'_, NetstatusKwd>) -> Result<(ConsensusFlavor, Preamble)> {
         use NetstatusKwd::*;
 
         {
@@ -358,7 +350,7 @@ impl Header {
             .split(',')
             .map(str::to_string)
             .collect();
-        let relay_versions = sec
+        let server_versions = sec
             .maybe(SERVER_VERSIONS)
             .args_as_str()
             .unwrap_or("")
@@ -391,12 +383,12 @@ impl Header {
 
         let consensus_method: u32 = sec.required(CONSENSUS_METHOD)?.parse_arg(0)?;
 
-        let shared_rand_prev = sec
+        let shared_rand_previous_value = sec
             .get(SHARED_RAND_PREVIOUS_VALUE)
             .map(SharedRandStatus::from_item)
             .transpose()?;
 
-        let shared_rand_cur = sec
+        let shared_rand_current_value = sec
             .get(SHARED_RAND_CURRENT_VALUE)
             .map(SharedRandStatus::from_item)
             .transpose()?;
@@ -409,18 +401,19 @@ impl Header {
             None
         };
 
-        Ok(Header {
-            flavor,
+        let preamble = Preamble {
             lifetime,
             client_versions,
-            relay_versions,
+            server_versions,
             proto_statuses,
             params,
             voting_delay,
             consensus_method,
-            shared_rand_prev,
-            shared_rand_cur,
-        })
+            shared_rand_previous_value,
+            shared_rand_current_value,
+        };
+
+        Ok((flavor, preamble))
     }
 }
 
@@ -430,26 +423,19 @@ impl Header {
 /// check_signature() on that result with the set of certs that you
 /// have.  Make sure only to provide authority certificates representing
 /// real authorities!
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
 #[derive(Debug, Clone)]
+#[non_exhaustive]
 pub struct UnvalidatedConsensus {
     /// The consensus object. We don't want to expose this until it's
     /// validated.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    consensus: Consensus,
+    pub consensus: Consensus,
     /// The signatures that need to be validated before we can call
     /// this consensus valid.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    siggroup: SignatureGroup,
+    pub siggroup: SignatureGroup,
     /// The total number of authorities that we believe in.  We need
     /// this information in order to validate the signatures, since it
     /// determines how many signatures we need to find valid in `siggroup`.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    n_authorities: Option<u16>,
+    pub n_authorities: Option<u16>,
 }
 
 impl UnvalidatedConsensus {
