@@ -40,7 +40,7 @@ use crate::types::misc::*;
 use crate::types::policy::*;
 use crate::types::version::TorVersion;
 use crate::util::PeekableIterator;
-use crate::{AllowAnnotations, Error, NetdocErrorKind as EK, Result, doc};
+use crate::{AllowAnnotations, Error, KeywordEncodable, NetdocErrorKind as EK, Result, doc};
 
 use ll::pk::ed25519::Ed25519Identity;
 use std::sync::Arc;
@@ -61,39 +61,24 @@ pub const DOC_DIGEST_LEN: usize = 20;
 pub type RdDigest = [u8; DOC_DIGEST_LEN];
 
 /// A router descriptor, with possible annotations.
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
+#[non_exhaustive]
 pub struct AnnotatedRouterDesc {
     /// Annotation for this router descriptor; possibly empty.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ann: RouterAnnotation,
+    pub ann: RouterAnnotation,
     /// Underlying router descriptor; signatures not checked yet.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    router: UncheckedRouterDesc,
+    pub router: UncheckedRouterDesc,
 }
 
 /// Annotations about a router descriptor, as stored on disc.
-#[allow(dead_code)] // don't warn about fields not getting read.
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
 #[derive(Default)]
+#[non_exhaustive]
 pub struct RouterAnnotation {
     /// Description of where we got this router descriptor
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    source: Option<String>,
+    pub source: Option<String>,
     /// When this descriptor was first downloaded.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    downloaded: Option<time::SystemTime>,
+    pub downloaded: Option<time::SystemTime>,
     /// Description of what we're willing to use this descriptor for.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    purpose: Option<String>,
+    pub purpose: Option<String>,
 }
 
 /// Information about a relay, parsed from a router descriptor.
@@ -109,89 +94,65 @@ pub struct RouterAnnotation {
 ///
 /// Before using this type to connect to a relay, you MUST check that
 /// it is valid, using is_expired_at().
-#[allow(dead_code)] // don't warn about fields not getting read.
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
 #[derive(Clone, Debug)]
+#[non_exhaustive]
 pub struct RouterDesc {
     /// Human-readable nickname for this relay.
     ///
     /// This is not secure, and not guaranteed to be unique.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    nickname: Nickname,
+    pub nickname: Nickname,
     /// IPv4 address for this relay.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ipv4addr: Option<net::Ipv4Addr>,
+    pub ipv4addr: Option<net::Ipv4Addr>,
     /// IPv4 ORPort for this relay.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    orport: u16,
+    pub orport: u16,
     /// IPv6 address and port for this relay.
     // TODO: we don't use a socketaddrv6 because we don't care about
     // the flow and scope fields.  We should decide whether that's a
     // good idea.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ipv6addr: Option<(net::Ipv6Addr, u16)>,
+    pub ipv6addr: Option<(net::Ipv6Addr, u16)>,
     /// Directory port for contacting this relay for direct HTTP
     /// directory downloads.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    dirport: u16,
+    pub dirport: u16,
     /// Declared uptime for this relay, in seconds.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    uptime: Option<u64>,
+    pub uptime: Option<u64>,
     /// Time when this router descriptor was published.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    published: time::SystemTime,
+    pub published: time::SystemTime,
     /// Ed25519 identity certificate (identity key authenticating a
     /// signing key)
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    identity_cert: tor_cert::Ed25519Cert,
+    pub identity_cert: tor_cert::Ed25519Cert,
     /// RSA identity key for this relay. (Deprecated; never use this without
     /// the ed25519 identity as well).
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    rsa_identity_key: ll::pk::rsa::PublicKey,
+    pub rsa_identity_key: ll::pk::rsa::PublicKey,
     /// RSA identity key for this relay. (Deprecated; never use this without
     /// the ed25519 identity as well).
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    rsa_identity: ll::pk::rsa::RsaIdentity,
+    pub rsa_identity: ll::pk::rsa::RsaIdentity,
     /// Key for extending a circuit to this relay using the ntor protocol.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ntor_onion_key: ll::pk::curve25519::PublicKey,
+    pub ntor_onion_key: ll::pk::curve25519::PublicKey,
     /// Key for extending a circuit to this relay using the
     /// (deprecated) TAP protocol.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    tap_onion_key: Option<ll::pk::rsa::PublicKey>,
+    pub tap_onion_key: Option<ll::pk::rsa::PublicKey>,
     /// List of subprotocol versions supported by this relay.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    proto: Arc<tor_protover::Protocols>,
+    pub proto: Arc<tor_protover::Protocols>,
     /// True if this relay says it's a directory cache.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    is_dircache: bool,
+    pub is_dircache: bool,
     /// True if this relay says that it caches extrainfo documents.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    is_extrainfo_cache: bool,
+    pub is_extrainfo_cache: bool,
     /// Declared family members for this relay.  If two relays are in the
     /// same family, they shouldn't be used in the same circuit.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    family: Arc<RelayFamily>,
+    pub family: Arc<RelayFamily>,
     /// Declared (and proven) family IDs for this relay. If two relays
     /// share a family ID, they shouldn't be used in the same circuit.
     family_ids: Vec<RelayFamilyId>,
     /// Software and version that this relay says it's running.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    platform: Option<RelayPlatform>,
+    pub platform: Option<RelayPlatform>,
     /// A complete address-level policy for which IPv4 addresses this relay
     /// says it supports.
     // TODO: these polices can get bulky too. Perhaps we should
     // de-duplicate them too.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ipv4_policy: AddrPolicy,
+    pub ipv4_policy: AddrPolicy,
     /// A summary of which ports this relay is willing to connect to
     /// on IPv6.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ipv6_policy: Arc<PortPolicy>,
+    pub ipv6_policy: Arc<PortPolicy>,
 }
 
 /// Description of the software a relay is running.
@@ -972,7 +933,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
