@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -47,8 +47,11 @@
 // TODO #1645 (either remove this, or decide to have it everywhere)
 #![cfg_attr(not(all(feature = "full", feature = "experimental")), allow(unused))]
 
-#[cfg(feature = "hs-service")]
-pub(crate) mod build;
+#[cfg(feature = "parse2")]
+#[macro_use]
+pub mod parse2;
+#[cfg(feature = "encode")]
+pub mod encode;
 #[macro_use]
 pub(crate) mod parse;
 pub mod doc;
@@ -56,8 +59,9 @@ mod err;
 pub mod types;
 mod util;
 
-#[cfg(feature = "parse2")]
-pub mod parse2;
+#[cfg(all(test, feature = "parse2"))]
+mod test2;
+
 #[doc(hidden)]
 pub use derive_deftly;
 
@@ -68,9 +72,9 @@ pub use util::batching_split_before;
 
 pub use err::{BuildError, Error, NetdocErrorKind, Pos};
 
-#[cfg(feature = "hs-service")]
-#[cfg_attr(docsrs, doc(cfg(feature = "hs-service")))]
-pub use build::NetdocBuilder;
+#[cfg(feature = "encode")]
+#[cfg_attr(docsrs, doc(cfg(feature = "encode")))]
+pub use encode::NetdocBuilder;
 
 /// Alias for the Result type returned by most objects in this module.
 pub type Result<T> = std::result::Result<T, Error>;
@@ -79,6 +83,20 @@ pub type Result<T> = std::result::Result<T, Error>;
 /// module.
 pub type BuildResult<T> = std::result::Result<T, BuildError>;
 
+/// Keywords that can be encoded (written) into a (being-built) network document
+pub trait KeywordEncodable {
+    /// Encoding of the keyword.
+    ///
+    /// Used for error reporting, and also by `NetdocEncoder::item`.
+    fn to_str(self) -> &'static str;
+}
+
+impl KeywordEncodable for &'static str {
+    fn to_str(self) -> &'static str {
+        self
+    }
+}
+
 /// Indicates whether we should parse an annotated list of objects or a
 /// non-annotated list.
 #[derive(PartialEq, Debug, Eq)]
@@ -94,6 +112,35 @@ pub enum AllowAnnotations {
     AnnotationsNotAllowed,
 }
 
+/// A "normally formatted" argument to a netdoc item
+///
+/// A type that is represented as a single argument
+/// whose representation is as for the type's `FromStr` and `Display`.
+///
+/// Implementing this trait enables a blanket impl of `parse2::ItemArgumentParseable`
+/// and `build::ItemArgument`.
+pub trait NormalItemArgument: std::str::FromStr + std::fmt::Display {}
+// TODO: should we implement ItemArgument for, say, tor_llcrypto::pk::rsa::RsaIdentity ?
+// It's not clear whether it's always formatted the same way in all parts of the spec.
+// The Display impl of RsaIdentity adds a `$` which is not supposed to be present
+// in (for example) an authority certificate (authcert)'s "fingerprint" line.
+
+impl NormalItemArgument for usize {}
+impl NormalItemArgument for u8 {}
+impl NormalItemArgument for u16 {}
+impl NormalItemArgument for u32 {}
+impl NormalItemArgument for u64 {}
+impl NormalItemArgument for u128 {}
+
+impl NormalItemArgument for isize {}
+impl NormalItemArgument for i8 {}
+impl NormalItemArgument for i16 {}
+impl NormalItemArgument for i32 {}
+impl NormalItemArgument for i64 {}
+impl NormalItemArgument for i128 {}
+
+impl NormalItemArgument for String {}
+
 /// Return a list of the protocols [supported](tor_protover::doc_supported)
 /// by this crate.
 pub fn supported_protocols() -> tor_protover::Protocols {
@@ -123,7 +170,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
