@@ -24,7 +24,7 @@ pub(crate) mod object {
     /// indicates the end of a begin or end tag.
     pub(crate) const TAG_END: &str = "-----";
     /// Maximum PEM base64 line length (not enforced during parsing)
-    #[cfg(feature = "hs-service")]
+    #[cfg(feature = "encode")]
     pub(crate) const BASE64_PEM_MAX_LINE: usize = 64;
 }
 
@@ -747,7 +747,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
