@@ -29,7 +29,7 @@ macro_rules! netdoc_ordering_check {
     { <=? subdoc    subdoc     $f1:ident } => {};
     // Not in the allowed list, must be an error:
     { <=? $k0:ident $k1:ident  $f1:ident } => {
-        compile_error(concat!(
+        compile_error!(concat!(
             "in netdoc, ", stringify!($k1)," field ", stringify!($f1),
             " may not come after ", stringify!($k0),
         ));
@@ -50,7 +50,242 @@ pub fn netdoc_parseable_derive_debug(ttype: &str, msg: &str, vals: &[&dyn Debug]
     .expect("write to String failed");
 }
 
+define_derive_deftly_module! {
+    /// Common definitions for `NetdocParseable` and `NetdocParseableFields`
+    ///
+    /// Importing template must define these:
+    ///
+    ///  * **`F_INTRO`**, **`F_SUBDOC`**, **`F_SIGNATURE`**
+    ///    conditions for the fundamental field kinds which aren't supported everywhere.
+    ///
+    ///    The `F_FLATTEN` and `F_NORMAL` field type conditions are defined here.
+    ///
+    ///  * **`THIS_ITEM`**: consumes the next item and evaluates to it as an `UnparsedItem`.
+    ///    See the definition in `NetdocParseable`.
+    ///
+    ///  * **`F_ACCUMULATE_VAR`** the variable or field into which to accumulate
+    ///    normal items for this field.  Must be of type `&mut $F_ACCUMULATE_TYPE`.
+    NetdocParseableCommon beta_deftly:
+
+    // Convenience alias for our prelude
+    ${define P { $crate::parse2::internal_prelude }}
+
+    // Defines the `dtrace` macro.
+    ${define DEFINE_DTRACE {
+        #[allow(unused_macros)]
+        macro_rules! dtrace { { $$msg:literal $$(, $$val:expr )* $$(,)? } => {
+          ${if tmeta(netdoc(debug)) {
+              netdoc_parseable_derive_debug(
+                  ${concat $ttype},
+                  $$msg,
+                  &[ $$( &&$$val as _, )* ],
+              )
+          }}
+        }}
+    }}
+
+    // Is this field `flatten`?
+    ${defcond F_FLATTEN fmeta(netdoc(flatten))}
+    // Is this field normal (non-structural)?
+    ${defcond F_NORMAL not(any(F_SIGNATURE, F_INTRO, F_FLATTEN, F_SUBDOC))}
+
+    // Field keyword as `&str`
+    ${define F_KEYWORD_STR { ${concat
+        ${if any(F_FLATTEN, F_SUBDOC) {
+          ${if F_INTRO {
+            ${error "#[deftly(netdoc(subdoc))] and (flatten) not supported for intro items"}
+          } else {
+            // Sub-documents and flattened fields have their keywords inside;
+            // if we ask for the field-based keyword name for one of those then that's a bug.
+            ${error "internal error, subdoc KeywordRef"}
+          }}
+        }}
+        ${fmeta(netdoc(keyword)) as str,
+          default ${concat ${kebab_case $fname}}}
+    }}}
+    // Field keyword as `&str` for debugging and error reporting
+    ${define F_KEYWORD_REPORT {
+        ${if F_SUBDOC { ${concat $fname} }
+             else { $F_KEYWORD_STR }}
+    }}
+    // Field keyword as `KeywordRef`
+    ${define F_KEYWORD { (KeywordRef::new_const($F_KEYWORD_STR)) }}
+
+    // The effective field type for parsing.
+    //
+    // Handles #[deftly(netdoc(default))], in which case we parse as if the field was Option,
+    // and substitute in the default at the end.
+    //
+    ${define F_EFFECTIVE_TYPE {
+        ${if all(fmeta(netdoc(default))) {
+            Option::<$ftype>
+        } else {
+            $ftype
+        }}
+    }}
+
+    // Provide `$<selector_ $fname>` for every (suitable) field.
+    ${define ITEM_SET_SELECTORS {
+        $(
+          ${when not(any(F_FLATTEN))}
+
+          // See `mod multiplicity`.
+        ${if not(all(F_INTRO, fmeta(netdoc(with)))) {
+          // If the intro it has `with`, we don't check its trait impl, and this ends up unused
+          let $<selector_ $fname> = MultiplicitySelector::<$F_EFFECTIVE_TYPE>::default();
+        }}
+        )
+    }}
+    // The item set selector for this field.
+    // We must provide this, rather than expanding $<selector_ $fname> at the use sites,
+    // because the identifier `selector_` has different macro_rules hygiene here vs there!
+    // TODO derive-deftly#130
+    ${define F_SELECTOR $<selector_ $fname>}
+
+    // Check that every field type implements the necessary trait.
+    ${define CHECK_FIELD_TYPES_PARSEABLE {
+        $(
+          ${when not(any(F_FLATTEN))}
+
+          // Expands to `selector_FIELD.check_SOMETHING();`
+          //
+          // If the relevant trait isn't implemented, rustc reports the error by
+          // pointing at the `check-something` call.  We re-span that identifier
+          // to point to the field name, so that's where the error is reported.
+          //
+          // Without this, we just get a report that `item` doesn't implement the required
+          // trait - but `item` is a local variable here, so the error points into the macro
+        ${if not(all(any(F_INTRO, F_NORMAL), fmeta(netdoc(with)))) {
+          $<selector_ $fname> . ${paste_spanned $fname ${select1
+                  any(F_INTRO, F_NORMAL){
+                      // For the intro item, this is not completely precise, because the
+                      // it will allow Option<> and Vec<> which aren't allowed there.
+                      ${if
+                        fmeta(netdoc(single_arg)) { check_item_argument_parseable }
+                        else { check_item_value_parseable }
+                      }
+                  }
+                  F_SIGNATURE { check_signature_item_parseable }
+                  F_SUBDOC    { check_subdoc_parseable         }
+          }} ();
+        }}
+        )
+    }}
+
+    // Convert the UnparsedItem (in `item` to the value (to accumulate).
+    // Expands to an expression.
+    ${define ITEM_VALUE_FROM_UNPARSED {
+        ${if fmeta(netdoc(with)) {
+          ${fmeta(netdoc(with)) as path}
+              ::${paste_spanned $fname from_unparsed}
+              (item)?
+        } else if fmeta(netdoc(single_arg)) { {
+          let item = ItemValueParseable::from_unparsed(item)?;
+          let (item,) = item;
+          item
+        } } else {
+          ItemValueParseable::from_unparsed(item)?
+        }}
+    }}
+
+    // Type into which we accumulate value(s) of this field
+    ${define F_ACCUMULATE_TYPE {
+        ${if F_FLATTEN {
+            <$ftype as NetdocParseableFields>::Accumulator
+        } else {
+            Option::<$F_EFFECTIVE_TYPE>
+        }
+    }}}
+
+    // Accumulates `item` (which must be `ItemSetMethods::Each`) into `$F_ACCUMULATE_VAR`
+    ${define ACCUMULATE_ITEM_VALUE { {
+        $F_SELECTOR.${paste_spanned $fname accumulate}($F_ACCUMULATE_VAR, item)?;
+    } }}
+
+    // Handle a nonstructural field, parsing and accumulating its value
+    //
+    // Looks at `kw` for the keyword.
+    //
+    // Expands to a series of `if ... { ... } else`.
+    // The use site must provide (maybe further arms) and a fallback block!
+    //
+    // If the item is the intro item for this document, evaluates `break` -
+    // so if `f_INTRO` is not trivially false, must be expanded within a field loop.
+    ${define NONSTRUCTURAL_ACCUMULATE_ELSE {
+        ${for fields {
+          ${when not(any(F_FLATTEN, F_SUBDOC))}
+
+          if kw == $F_KEYWORD {
+            ${select1
+              F_NORMAL {
+                let item = $THIS_ITEM;
+                dtrace!("is normal", item);
+                let item = $ITEM_VALUE_FROM_UNPARSED;
+                $ACCUMULATE_ITEM_VALUE
+              }
+              F_SIGNATURE {
+                let hash_inputs = input
+                      .peek_signature_hash_inputs(signed_doc_body)?
+                      .expect("not eof, we peeked kw");
+
+                let item = $THIS_ITEM;
+                dtrace!("is signature", item);
+                let item =
+                    SignatureItemParseable::from_unparsed_and_body(item, &hash_inputs)?;
+                $ACCUMULATE_ITEM_VALUE
+              }
+              F_INTRO {
+                dtrace!("is intro", kw);
+                break;
+              } // start of next similar document
+            }
+          } else
+        }}
+        ${for fields {
+          ${when F_FLATTEN}
+
+          if $ftype::is_item_keyword(kw) {
+              dtrace!(${concat "is flatten in " $fname}, kw);
+              let item = $THIS_ITEM;
+              <$ftype as NetdocParseableFields>::accumulate_item($F_ACCUMULATE_VAR, item)?;
+          } else
+        }}
+    }}
+
+    // Completes a document
+    //
+    // The fields accumulated so far must be in `$fpatname` (as a value, not a ref,
+    // and therefore not in $F_ACCUMULATE_VAR).
+    //
+    // Expands to code which resolves the fields, and ends with `Ok(document value)`.
+    ${define FINISH_RESOLVE {
+        ${for fields {
+            ${select1
+              F_INTRO {}
+              any(F_NORMAL, F_SIGNATURE) {
+                  let $fpatname = $F_SELECTOR.finish($fpatname, $F_KEYWORD_REPORT)?;
+              }
+              F_FLATTEN {
+                  let $fpatname = <$ftype as NetdocParseableFields>::finish($fpatname)?;
+              }
+              F_SUBDOC {
+                  let $fpatname = $F_SELECTOR.finish_subdoc($fpatname)?;
+              }
+            }
+        }}
+        $(
+            ${when not(F_INTRO)}
+          ${if fmeta(netdoc(default)) {
+            let $fpatname = Option::unwrap_or_default($fpatname);
+          }}
+        )
+        Ok($vpat)
+    }}
+}
+
 define_derive_deftly! {
+    use NetdocParseableCommon;
+
     /// Derive [`NetdocParseable`] for a document (or sub-document)
     ///
     /// ### Expected input structure
@@ -126,6 +361,19 @@ define_derive_deftly! {
     ///
     ///   Use `STR` as the Keyword for this Item.
     ///
+    /// * **`#[deftly(netdoc(single_arg))]`**:
+    ///
+    ///   The field type implements `ItemArgumentParseable`,
+    ///   instead of `ItemValueParseable`,
+    ///   and is parsed as if `(FIELD_TYPE,)` had been written.
+    ///
+    /// * **`#[deftly(netdoc(with = "MODULE"))]`**:
+    ///
+    ///   Instead of `ItemValueParseable`, the item is parsed with `MODULE::from_unparsed`,
+    ///   which must have the same signature as [`ItemValueParseable::from_unparsed`].
+    ///
+    ///   (Not supported for sub-documents, signature items, or field collections.)
+    ///
     /// * **`#[deftly(netdoc(default))]`**:
     ///
     ///   This field is optional ("at most once");
@@ -135,6 +383,14 @@ define_derive_deftly! {
     ///   With `netdoc(default)`, the field value doesn't need unwrapping.
     ///   With `Option` it is possible to see if the field was provided.
     ///
+    /// * **`#[deftly(netdoc(flatten))]`**:
+    ///
+    ///   This field is a struct containing further individual normal fields.
+    ///   The Items for those individual fields can appear in *this*
+    ///   outer document in any order, interspersed with other normal fields.
+    ///
+    ///   The field type must implement [`NetdocParseableFields`].
+    ///
     /// * **`#[deftly(netdoc(subdoc))]`**:
     ///
     ///   This field is a sub-document.
@@ -154,7 +410,7 @@ define_derive_deftly! {
     /// use tor_netdoc::derive_deftly_template_NetdocParseable;
     /// use tor_netdoc::derive_deftly_template_NetdocSigned;
     /// use tor_netdoc::derive_deftly_template_ItemValueParseable;
-    /// use tor_netdoc::parse2::{parse_netdoc, VerifyFailed};
+    /// use tor_netdoc::parse2::{parse_netdoc, ParseInput, VerifyFailed};
     /// use tor_netdoc::parse2::{SignatureItemParseable, SignatureHashInputs};
     ///
     /// #[derive(Deftly, Debug, Clone)]
@@ -200,15 +456,13 @@ define_derive_deftly! {
     ///     }
     /// }
     ///
-    /// let doc: NdThingSigned = parse_netdoc(&doc_text, "<input>").unwrap();
+    /// let input = ParseInput::new(&doc_text, "<input>");
+    /// let doc: NdThingSigned = parse_netdoc(&input).unwrap();
     /// let doc = doc.verify_foolish_timeless().unwrap();
     /// assert_eq!(doc.value.0, "something");
     /// ```
     export NetdocParseable for struct, expect items, beta_deftly:
 
-    // Convenience alias for our prelude
-    ${define P { $crate::parse2::internal_prelude }}
-
     // Predicate for the toplevel
     ${defcond T_SIGNATURES tmeta(netdoc(signatures))}
 
@@ -216,37 +470,8 @@ define_derive_deftly! {
     ${defcond F_INTRO all(not(T_SIGNATURES), approx_equal($findex, 0))}
     ${defcond F_SUBDOC fmeta(netdoc(subdoc))}
     ${defcond F_SIGNATURE T_SIGNATURES} // signatures section documents have only signature fields
-    ${defcond F_NORMAL not(any(F_SIGNATURE, F_INTRO, F_SUBDOC))}
 
-    // Field keyword as `&str`
-    ${define F_KEYWORD_STR { ${concat
-        ${if F_SUBDOC {
-            // Sub-documents have their own keywords; if we ask for the field-based
-            // keyword name of a sub-document, then that's a bug.
-            ${error "internal error, subdoc KeywordRef"}
-        }}
-        ${fmeta(netdoc(keyword)) as str,
-          default ${concat ${kebab_case $fname}}}
-    }}}
-    // Field keyword as `&str` for debugging and error reporting
-    ${define F_KEYWORD_REPORT {
-        ${if F_SUBDOC { ${concat $fname} }
-             else { $F_KEYWORD_STR }}
-    }}
-    // Field keyword as `KeywordRef`
-    ${define F_KEYWORD { (KeywordRef::new_const($F_KEYWORD_STR)) }}
-
-    // The effective field type for parsing.
-    //
-    // Handles #[deftly(netdoc(default))], in which case we parse as if the field was Option,
-    // and substitute in the default at the end.
-    ${define F_EFFECTIVE_TYPE {
-        ${if all(fmeta(netdoc(default)), not(F_INTRO)) {
-            Option::<$ftype>
-        } else {
-            $ftype
-        }}
-    }}
+    ${define F_ACCUMULATE_VAR { (&mut $fpatname) }}
 
     impl<$tgens> $P::NetdocParseable for $ttype {
         fn doctype_for_error() -> &'static str {
@@ -268,8 +493,9 @@ define_derive_deftly! {
         fn from_items<'s>(
             input: &mut $P::ItemStream<'s>,
             outer_stop: $P::stop_at!(),
-        ) -> Result<$ttype, $P::ErrorProblem> {
+        ) -> $P::Result<$ttype, $P::ErrorProblem> {
             use $P::*;
+            $DEFINE_DTRACE
 
             //----- compile-time check that fields are in the right order in the struct -----
 
@@ -279,6 +505,7 @@ define_derive_deftly! {
                     ${select1
                       F_INTRO     { intro     }
                       F_NORMAL    { normal    }
+                      F_FLATTEN   { normal    }
                       F_SUBDOC    { subdoc    }
                     }
                     $fname
@@ -286,26 +513,9 @@ define_derive_deftly! {
               }
             }}
 
-            //----- Debugging -----
-
-            macro_rules! dtrace { { $$msg:literal $$(, $$val:expr )* $$(,)? } => {
-              ${if tmeta(netdoc(debug)) {
-                  netdoc_parseable_derive_debug(
-                      ${concat $ttype},
-                      $$msg,
-                      &[ $$( &&$$val as _, )* ],
-                  )
-              }}
-            }}
-
             //----- prepare item set selectors for every field -----
-
-          $(
-            ${when not(F_INTRO)}
-
-            // See `mod multiplicity`.
-            let $<selector_ $fname> = ItemSetSelector::<$F_EFFECTIVE_TYPE>::default();
-          )
+            $ITEM_SET_SELECTORS
+            $CHECK_FIELD_TYPES_PARSEABLE
 
             // Is this an intro item keyword ?
             //
@@ -314,7 +524,7 @@ define_derive_deftly! {
             //    fn(KeywordRef) -> bool
             ${define F_SUBDOC_IS_INTRO_ITEM_KEYWORD {
                 ${if not(F_SUBDOC) { ${error "internal-error: subdoc kw, but not subdoc field"} }}
-                $<selector_ $fname>.is_intro_item_keyword
+                $F_SELECTOR.is_intro_item_keyword
             }}
 
             //----- Helper fragments for parsing individual pieces of the document -----
@@ -339,11 +549,6 @@ define_derive_deftly! {
                 input.next_item()?.expect("peeked")
             }}
 
-            // Accumulates `item` (which must be DataSet::Value) into `Putnam`
-            ${define ACCUMULATE_ITEM_VALUE { {
-                $<selector_ $fname>.accumulate(&mut $fpatname, item)?;
-            } }}
-
             //----- keyword classification closures -----
 
             // Is this a keyword for one of our sub-documents?
@@ -382,11 +587,11 @@ define_derive_deftly! {
             if !Self::is_intro_item_keyword(item.keyword()) {
                 Err(EP::WrongDocumentType)?;
             }
-            let $fpatname: $ftype = <$ftype as ItemValueParseable>::from_unparsed(item)?;
+            let $fpatname: $ftype = $ITEM_VALUE_FROM_UNPARSED;
 
           } else {
 
-            let mut $fpatname: Option<$F_EFFECTIVE_TYPE> = None;
+            let mut $fpatname = $F_ACCUMULATE_TYPE::default();
 
           }})
 
@@ -399,35 +604,8 @@ define_derive_deftly! {
                     dtrace!("is inner stop", kw);
                     break;
                 };
-              ${for fields {
-                ${when not(F_SUBDOC)}
 
-                if kw == $F_KEYWORD {
-                  ${select1
-                    F_NORMAL {
-                      let item = $THIS_ITEM;
-                      dtrace!("is normal", item);
-                      let item = ItemValueParseable::from_unparsed(item)?;
-                      $ACCUMULATE_ITEM_VALUE
-                    }
-                    F_SIGNATURE {
-                      let hash_inputs = input
-                            .peek_signature_hash_inputs(signed_doc_body)?
-                            .expect("not eof, we peeked kw");
-
-                      let item = $THIS_ITEM;
-                      dtrace!("is signature", item);
-                      let item =
-                          SignatureItemParseable::from_unparsed_and_body(item, &hash_inputs)?;
-                      $ACCUMULATE_ITEM_VALUE
-                    }
-                    F_INTRO {
-                      dtrace!("is intro", kw);
-                      break;
-                    } // start of next similar document
-                  }
-                } else
-              }}
+                $NONSTRUCTURAL_ACCUMULATE_ELSE
                 {
                     dtrace!("is unknown (in normal)");
                     let _: UnparsedItem = $THIS_ITEM;
@@ -450,6 +628,8 @@ define_derive_deftly! {
                     break;
                 };
 
+                $F_SELECTOR.can_accumulate(&mut $fpatname)?;
+
                 dtrace!("is this subdoc", kw);
                 let item = NetdocParseable::from_items(input, inner_stop);
                 dtrace!("parsed this subdoc", item.as_ref().map(|_| ()));
@@ -462,24 +642,109 @@ define_derive_deftly! {
             // Resolve all the fields
             dtrace!("reached end, resolving");
 
-          $(
-            ${when not(any(F_INTRO, F_SUBDOC))}
-            let $fpatname = $<selector_ $fname>.finish($fpatname, $F_KEYWORD_REPORT)?;
-          )
-          $(
-            ${when F_SUBDOC}
-            let $fpatname = $<selector_ $fname>.finish_subdoc($fpatname)?;
-          )
-          $(
-            ${when not(F_INTRO)}
-          ${if fmeta(netdoc(default)) {
-            let $fpatname = Option::unwrap_or_default($fpatname);
+            $FINISH_RESOLVE
+        }
+    }
+}
+
+define_derive_deftly! {
+    use NetdocParseableCommon;
+
+    /// Derive [`NetdocParseableFields`] for a struct with individual items
+    ///
+    /// Defines a struct `FooNetdocParseAccumulator` to be the
+    /// `NetdocParseableFields::Accumulator`.
+    ///
+    /// Similar to
+    /// [`#[derive_deftly(NetdocParseable)]`](derive_deftly_template_NetdocParseable),
+    /// but:
+    ///
+    ///  * Derives [`NetdocParseableFields`]
+    ///  * The input struct can contain only normal non-structural items
+    ///    (so it's not a sub-document with an intro item).
+    ///  * The only attributes supported are the field attributes
+    ///    `#[deftly(netdoc(keyword = STR))]`
+    ///    `#[deftly(netdoc(default))]`
+    ///    `#[deftly(netdoc(single_arg))]`
+    ///    `#[deftly(netdoc(with = "MODULE"))]`
+    ///    `#[deftly(netdoc(flatten))]`
+    export NetdocParseableFields for struct , expect items, beta_deftly:
+
+    ${defcond F_INTRO false}
+    ${defcond F_SUBDOC false}
+    ${defcond F_SIGNATURE false}
+
+    ${define THIS_ITEM item}
+    ${define F_ACCUMULATE_VAR { (&mut acc.$fname) }}
+
+    #[doc = ${concat "Partially parsed `" $tname "`"}]
+    ///
+    /// Used for [`${concat $P::NetdocParseableFields::Accumulator}`].
+    #[derive(Default, Debug)]
+    $tvis struct $<$tname NetdocParseAccumulator><$tdefgens> { $(
+        $fname: $F_ACCUMULATE_TYPE,
+    ) }
+
+    impl<$tgens> $P::NetdocParseableFields for $ttype {
+        type Accumulator = $<$ttype NetdocParseAccumulator>;
+
+        fn is_item_keyword(
+            #[allow(unused_variables)] // If there are no fields, this is unused
+            kw: $P::KeywordRef<'_>,
+        ) -> bool {
+            #[allow(unused_imports)] // false positives in some situations
+            use $P::*;
+
+          ${for fields {
+            ${when not(F_FLATTEN)}
+            kw == $F_KEYWORD ||
           }}
-          )
+          ${for fields {
+            ${when F_FLATTEN}
+            <$ftype as NetdocParseableFields>::is_item_keyword(kw) ||
+          }}
+            false
+        }
 
-            let r = $vpat;
+        fn accumulate_item(
+            #[allow(unused_variables)] // If there are no fields, this is unused
+            acc: &mut Self::Accumulator,
+            #[allow(unused_variables)] // If there are no fields, this is unused
+            item: $P::UnparsedItem<'_>,
+        ) -> $P::Result<(), $P::ErrorProblem> {
+            #[allow(unused_imports)] // false positives in some situations
+            use $P::*;
+            $DEFINE_DTRACE
 
-            Ok(r)
+            $ITEM_SET_SELECTORS
+            $CHECK_FIELD_TYPES_PARSEABLE
+
+            #[allow(unused_variables)] // If there are no fields, this is unused
+            let kw = item.keyword();
+
+            $NONSTRUCTURAL_ACCUMULATE_ELSE
+            {
+                panic!("accumulate_item called though is_intro_item_keyword returns false");
+            }
+
+            #[allow(unreachable_code)] // If there are no fields!
+            Ok(())
+        }
+
+        fn finish(
+            #[allow(unused_variables)] // If there are no fields, this is unused
+            acc: Self::Accumulator
+        ) -> $P::Result<Self, $P::ErrorProblem> {
+            #[allow(unused_imports)] // false positives in some situations
+            use $P::*;
+            $DEFINE_DTRACE
+
+            dtrace!("finish, resolving");
+
+            $ITEM_SET_SELECTORS
+
+         $( let $fpatname = acc.$fname; )
+            $FINISH_RESOLVE
         }
     }
 }
@@ -520,7 +785,7 @@ define_derive_deftly! {
     //
     // We don't make this a generic struct because the defining module (crate)
     // will want to add verification methods, which means they must define the struct.
-    export NetdocSigned expect items, beta_deftly:
+    export NetdocSigned for struct, expect items, beta_deftly:
 
     // Convenience alias for our prelude
     ${define P { $crate::parse2::internal_prelude }}
@@ -563,7 +828,7 @@ define_derive_deftly! {
         fn from_items<'s>(
             input: &mut $P::ItemStream<'s>,
             outer_stop: $P::stop_at!(),
-        ) -> Result<$<$ttype Signed>, $P::ErrorProblem> {
+        ) -> $P::Result<$<$ttype Signed>, $P::ErrorProblem> {
             input.parse_signed(outer_stop)
         }
     }
@@ -594,12 +859,12 @@ define_derive_deftly! {
     /// Each field should be:
     ///
     ///  * `impl `[`ItemArgumentParseable`] (one argument),
-    ///  * `Option<impl ItemArgumentParseable>` (one optional argument), or
-    ///  * `Vec<impl ItemArgumentParseable>` (zero or more arguments).
+    ///  * `Option<impl ItemArgumentParseable>` (one optional argument),
+    ///  * `Vec<impl ItemArgumentParseable>` (zero or more arguments), or
+    ///  * `BTreeSet<impl ItemArgumentParseable + Ord>` (zero or more arguments).
     ///
-    /// `ItemArgumentParseable` is implemented for every `impl FromStr`,
-    /// so `impl FromStr`, `Option<impl FromStr>` and `Vec<impl FromStr>`
-    /// are supported.
+    /// `ItemArgumentParseable` can be implemented via `impl FromStr`,
+    /// by writing `impl NormalItemArgument`.
     ///
     /// For `Option` or `Vec`, we expect that *if* there are any further arguments,
     /// they are for this field.
@@ -622,16 +887,45 @@ define_derive_deftly! {
     ///
     /// ### Field-level attributes:
     ///
+    ///  * **`#[deftly(netdoc(rest))]**:
+    ///
+    ///    The field is the whole rest of the line.
+    ///    Must come after any other normal argument fields.
+    ///    Only allowed once.
+    ///
+    ///    The field type must implement `FromStr`.
+    ///    (I.e. `Vec` , `Option` etc., are not allowed.)
+    ///
     ///  * **`#[deftly(netdoc(object))]**:
     ///
     ///    The field is the Object.
     ///    It must implement [`ItemObjectParseable`]
-    ///    (so it can be be `Option<impl ItemObjectParseable>`
-    ///    for an optional item.)
+    ///    (or be `Option<impl ItemObjectParseable>`).
     ///
     ///    Only allowed once.
     ///    If omittted, any object is rejected.
     ///
+    ///  * **`#[deftly(netdoc(object(label = "LABEL")))]**:
+    ///
+    ///    Sets the expected label for an Object.
+    ///    If not supplied, uses [`ItemObjectParseable::check_label`].
+    ///
+    ///  * **`#[deftly(netdoc(with = "MODULE")]**:
+    ///
+    ///    Instead of `ItemArgumentParseable`, the argument is parsed with `MODULE::from_args`,
+    ///    which must have the same signature as [`ItemArgumentParseable::from_args`].
+    ///
+    ///    With `#[deftly(netdoc(rest))]`, FUNCTION replaces
+    ///    `<FIELD AS FromStr>::from_str`.
+    ///
+    ///    With `#[deftly(netdoc(objecte))]`, uses `MODULE::try_from`
+    ///    which must have the signature `fn(Vec<u8>) -> Result<OBJECT, _>;
+    ///    like `TryFrom::<Vec<u8>>>::try_from`.
+    ///    LABEL must also be specified
+    ///    unless the object also implements `ItemObjectParseable`.
+    ///    Errors from parsing will be discarded and replaced with
+    ///    [`ErrorProblem::ObjectInvalidData`].
+    ///
     ///  * **`#[deftly(netdoc(sig_hash = "HASH_METHOD"))]**:
     ///
     ///    This item is a signature item.
@@ -642,7 +936,7 @@ define_derive_deftly! {
     ///    which will be resolved with `sig_hash_methods::*` in scope.
     ///
     ///    `fn HASH_METHOD(body: &SignatureHashInputs) -> HASH_FIELD_VALUE`.
-    export ItemValueParseable for struct, expect items:
+    export ItemValueParseable for struct, expect items, beta_deftly:
 
     ${define P { $crate::parse2::internal_prelude }}
 
@@ -661,7 +955,7 @@ define_derive_deftly! {
           ${if T_IS_SIGNATURE {
             document_body: &SignatureHashInputs<'_>,
           }}
-        ) -> Result<Self, $P::EP>
+        ) -> $P::Result<Self, $P::EP>
         {
             #[allow(unused_imports)] // false positive when macro is used with prelude in scope
             use $P::*;
@@ -671,28 +965,50 @@ define_derive_deftly! {
             let mut args = input.args_mut();
           $(
             let $fpatname = ${select1
-              F_NORMAL {
-                  <$ftype as ItemArgumentParseable>::from_args(&mut args, stringify!($fname))?
-              }
-              all(F_OBJECT, not(fmeta(netdoc(object(label))))) {
-                  <$ftype as ItemObjectParseable>::from_bytes_option(
-                      object
-                          .map(|object| object.decode_data()).transpose()?
-                          .as_deref()
-                  )?
-              }
-              all(F_OBJECT, fmeta(netdoc(object(label)))) { {
-                  let object = object.ok_or_else(|| EP::MissingObject)?;
-                  if object.label() != ${fmeta(netdoc(object(label))) as str} {
-                      return Err(EP::ObjectIncorrectLabel)
-                  }
-                  object.decode_data()?
+              F_NORMAL { {
+                  let selector = MultiplicitySelector::<$ftype>::default();
+                ${if not(fmeta(netdoc(with))) {
+                  selector.${paste_spanned $fname check_argument_value_parseable}();
+                }}
+                  selector.parse_with(
+                      &mut args,
+                      ${fmeta(netdoc(with))
+                        as path,
+                        default { ItemArgumentParseable }}::${paste_spanned $fname from_args},
+                  ).map_err(args.error_handler(stringify!($fname)))?
               } }
-              F_REST {
+              F_OBJECT { {
+                  let selector = MultiplicitySelector::<$ftype>::default();
+                  let object = object.map(|object| {
+                      let data = object.decode_data()?;
+                      ${if fmeta(netdoc(object(label))) {
+                          if object.label() != ${fmeta(netdoc(object(label))) as str} {
+                              return Err(EP::ObjectIncorrectLabel)
+                          }
+                      } else {
+                          selector.check_label(object.label())?;
+                      }}
+                      ${if fmeta(netdoc(with)) {
+                          ${fmeta(netdoc(with)) as path}::${paste_spanned $fname try_from}
+                              (data)
+                              .map_err(|_| EP::ObjectInvalidData)
+                      } else {
+                          selector.${paste_spanned $fname check_object_parseable}();
+                          ItemObjectParseable::from_bytes(&data)
+                      }}
+                  }).transpose()?;
+                  selector.resolve_option(object)?
+              } }
+              F_REST { {
                   // consumes `args`, leading to compile error if the rest field
                   // isn't last (or is combined with no_extra_args).
-                  <$ftype as FromStr>::parse(args.into_rest())?
-              }
+                  let args_consume = args;
+                  ${fmeta(netdoc(with))
+                    as path,
+                    default { <$ftype as FromStr>::from_str }}(args_consume.into_remaining())
+                      .map_err(|_| AE::Invalid)
+                      .map_err(args_consume.error_handler(stringify!($fname)))?
+              } }
               F_SIG_HASH { {
                   #[allow(unused_imports)]
                   use $P::sig_hash_methods::*;
