@@ -15,15 +15,15 @@
 //!
 //!  * Create a TLS connection as an object that implements AsyncRead +
 //!    AsyncWrite + StreamOps, and pass it to a [ChannelBuilder].  This will
-//!    yield an [handshake::ClientInitiatorHandshake] that represents
+//!    yield an [crate::client::channel::handshake::ClientInitiatorHandshake] that represents
 //!    the state of the handshake.
-//!  * Call [handshake::ClientInitiatorHandshake::connect] on the result
+//!  * Call [crate::client::channel::handshake::ClientInitiatorHandshake::connect] on the result
 //!    to negotiate the rest of the handshake.  This will verify
 //!    syntactic correctness of the handshake, but not its cryptographic
 //!    integrity.
-//!  * Call [handshake::UnverifiedChannel::check] on the result.  This
+//!  * Call handshake::UnverifiedChannel::check on the result.  This
 //!    finishes the cryptographic checks.
-//!  * Call [handshake::VerifiedChannel::finish] on the result. This
+//!  * Call handshake::VerifiedChannel::finish on the result. This
 //!    completes the handshake and produces an open channel and Reactor.
 //!  * Launch an asynchronous task to call the reactor's run() method.
 //!
@@ -45,19 +45,14 @@
 //!
 //! # Limitations
 //!
-//! This is client-only, and only supports link protocol version 4.
-//!
-//! TODO: There is no channel padding.
-//!
-//! TODO: There is no flow control, rate limiting, queueing, or
-//! fairness.
+//! TODO: There is no rate limiting or fairness.
 
 /// The size of the channel buffer for communication between `Channel` and its reactor.
 pub const CHANNEL_BUFFER_SIZE: usize = 128;
 
 mod circmap;
 mod handler;
-mod handshake;
+pub(crate) mod handshake;
 pub mod kist;
 mod msg;
 pub mod padding;
@@ -66,15 +61,19 @@ mod reactor;
 mod unique_id;
 
 pub use crate::channel::params::*;
-use crate::channel::reactor::{BoxedChannelSink, BoxedChannelStream, Reactor};
+pub(crate) use crate::channel::reactor::Reactor;
+use crate::channel::reactor::{BoxedChannelSink, BoxedChannelStream};
 pub use crate::channel::unique_id::UniqId;
-use crate::client::circuit::PendingClientTunnel;
+use crate::client::channel::ClientChannelBuilder;
+use crate::client::circuit::padding::{PaddingController, QueuedCellPaddingInfo};
+use crate::client::circuit::{PendingClientTunnel, TimeoutEstimator};
 use crate::memquota::{ChannelAccount, CircuitAccount, SpecificAccount as _};
 use crate::util::err::ChannelClosed;
 use crate::util::oneshot_broadcast;
 use crate::util::ts::AtomicOptTimestamp;
 use crate::{ClockSkew, client};
 use crate::{Error, Result};
+use cfg_if::cfg_if;
 use reactor::BoxedChannelStreamOps;
 use safelog::sensitive as sv;
 use std::future::{Future, IntoFuture};
@@ -90,6 +89,9 @@ use tor_linkspec::{HasRelayIds, OwnedChanTarget};
 use tor_memquota::mq_queue::{self, ChannelSpec as _, MpscSpec};
 use tor_rtcompat::{CoarseTimeProvider, DynTimeProvider, SleepProvider, StreamOps};
 
+#[cfg(feature = "circ-padding")]
+use tor_async_utils::counting_streams::{self, CountingSink, CountingStream};
+
 /// Imports that are re-exported pub if feature `testing` is enabled
 ///
 /// Putting them together in a little module like this allows us to select the
@@ -97,7 +99,7 @@ use tor_rtcompat::{CoarseTimeProvider, DynTimeProvider, SleepProvider, StreamOps
 mod testing_exports {
     #![allow(unreachable_pub)]
     pub use super::reactor::CtrlMsg;
-    pub use crate::client::circuit::celltypes::CreateResponse;
+    pub use crate::circuit::celltypes::CreateResponse;
 }
 #[cfg(feature = "testing")]
 pub use testing_exports::*;
@@ -115,11 +117,13 @@ use std::result::Result as StdResult;
 use std::sync::Arc;
 use std::task::{Context, Poll};
 
-use tracing::trace;
+use tracing::{instrument, trace};
 
 // reexport
+pub use super::client::channel::handshake::ClientInitiatorHandshake;
+#[cfg(feature = "relay")]
+pub use super::relay::channel::handshake::RelayInitiatorHandshake;
 use crate::channel::unique_id::CircUniqIdContext;
-pub use handshake::{ClientInitiatorHandshake, UnverifiedChannel, VerifiedChannel};
 
 use kist::KistParams;
 
@@ -174,14 +178,6 @@ pub enum ChannelType {
     },
 }
 
-impl ChannelType {
-    /// Return true if this channel type is an initiator.
-    #[expect(unused)] // TODO: Remove once used.
-    pub(crate) fn is_initiator(&self) -> bool {
-        matches!(self, Self::ClientInitiator | Self::RelayInitiator)
-    }
-}
-
 /// A channel cell frame used for sending and receiving cells on a channel. The handler takes care
 /// of the cell codec transition depending in which state the channel is.
 ///
@@ -189,6 +185,9 @@ impl ChannelType {
 /// lifetime.
 pub(crate) type ChannelFrame<T> = asynchronous_codec::Framed<T, handler::ChannelCellHandler>;
 
+/// An entry in a channel's queue of cells to be flushed.
+pub(crate) type ChanCellQueueEntry = (AnyChanCell, Option<QueuedCellPaddingInfo>);
+
 /// Helper: Return a new channel frame [ChannelFrame] from an object implementing AsyncRead + AsyncWrite. In the
 /// tor context, it is always a TLS stream.
 ///
@@ -199,7 +198,9 @@ where
     T: AsyncRead + AsyncWrite,
     I: Into<handler::ChannelCellHandler>,
 {
-    asynchronous_codec::Framed::new(tls, ty.into())
+    let mut framed = asynchronous_codec::Framed::new(tls, ty.into());
+    framed.set_send_high_water_mark(32 * 1024);
+    framed
 }
 
 /// An open client channel, ready to send and receive Tor cells.
@@ -239,7 +240,7 @@ pub struct Channel {
     /// A channel used to send control messages to the Reactor.
     control: mpsc::UnboundedSender<CtrlMsg>,
     /// A channel used to send cells to the Reactor.
-    cell_tx: mq_queue::Sender<AnyChanCell, mq_queue::MpscSpec>,
+    cell_tx: CellTx,
 
     /// A receiver that indicates whether the channel is closed.
     ///
@@ -247,6 +248,9 @@ pub struct Channel {
     /// Read to decide if operations may succeed, and is returned by `wait_for_close`.
     reactor_closed_rx: oneshot_broadcast::Receiver<Result<CloseInfo>>,
 
+    /// Padding controller, used to report when data is queued for this channel.
+    padding_ctrl: PaddingController,
+
     /// A unique identifier for this channel.
     unique_id: UniqId,
     /// Validated identity and address information for this peer.
@@ -342,15 +346,34 @@ enum PaddingControlState {
 
 use PaddingControlState as PCS;
 
+cfg_if! {
+    if #[cfg(feature="circ-padding")] {
+        /// Implementation type for a ChannelSender.
+        type CellTx = CountingSink<mq_queue::Sender<ChanCellQueueEntry, mq_queue::MpscSpec>>;
+
+        /// Implementation type for a cell queue held by a reactor.
+        type CellRx = CountingStream<mq_queue::Receiver<ChanCellQueueEntry, mq_queue::MpscSpec>>;
+    } else {
+        /// Implementation type for a ChannelSender.
+        type CellTx = mq_queue::Sender<ChanCellQueueEntry, mq_queue::MpscSpec>;
+
+        /// Implementation type for a cell queue held by a reactor.
+        type CellRx = mq_queue::Receiver<ChanCellQueueEntry, mq_queue::MpscSpec>;
+    }
+}
+
 /// A handle to a [`Channel`]` that can be used, by circuits, to send channel cells.
 #[derive(Debug)]
 pub(crate) struct ChannelSender {
     /// MPSC sender to send cells.
-    cell_tx: mq_queue::Sender<AnyChanCell, mq_queue::MpscSpec>,
+    cell_tx: CellTx,
     /// A receiver used to check if the channel is closed.
     reactor_closed_rx: oneshot_broadcast::Receiver<Result<CloseInfo>>,
     /// Unique ID for this channel. For logging.
     unique_id: UniqId,
+    /// Padding controller for this channel:
+    /// used to report when we queue data that will eventually wind up on the channel.
+    padding_ctrl: PaddingController,
 }
 
 impl ChannelSender {
@@ -379,11 +402,39 @@ impl ChannelSender {
     /// (This can sometimes be used to avoid having to keep
     /// a separate clone of the time provider.)
     pub(crate) fn time_provider(&self) -> &DynTimeProvider {
-        self.cell_tx.time_provider()
+        cfg_if! {
+            if #[cfg(feature="circ-padding")] {
+                self.cell_tx.inner().time_provider()
+            } else {
+                self.cell_tx.time_provider()
+            }
+        }
+    }
+
+    /// Return an approximate count of the number of outbound cells queued for this channel.
+    ///
+    /// This count is necessarily approximate,
+    /// because the underlying count can be modified by other senders and receivers
+    /// between when this method is called and when its return value is used.
+    ///
+    /// Does not include cells that have already been passed to the TLS connection.
+    ///
+    /// Circuit padding uses this count to determine
+    /// when messages are already outbound for the first hop of a circuit.
+    #[cfg(feature = "circ-padding")]
+    pub(crate) fn approx_count(&self) -> usize {
+        self.cell_tx.approx_count()
+    }
+
+    /// Note that a cell has been queued that will eventually be placed onto this sender.
+    ///
+    /// We use this as an input for padding machines.
+    pub(crate) fn note_cell_queued(&self) {
+        self.padding_ctrl.queued_data(crate::HopNum::from(0));
     }
 }
 
-impl Sink<AnyChanCell> for ChannelSender {
+impl Sink<ChanCellQueueEntry> for ChannelSender {
     type Error = Error;
 
     fn poll_ready(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Result<()>> {
@@ -393,21 +444,21 @@ impl Sink<AnyChanCell> for ChannelSender {
             .map_err(|_| ChannelClosed.into())
     }
 
-    fn start_send(self: Pin<&mut Self>, cell: AnyChanCell) -> Result<()> {
+    fn start_send(self: Pin<&mut Self>, cell: ChanCellQueueEntry) -> Result<()> {
         let this = self.get_mut();
         if this.reactor_closed_rx.is_ready() {
             return Err(ChannelClosed.into());
         }
-        this.check_cell(&cell)?;
+        this.check_cell(&cell.0)?;
         {
             use tor_cell::chancell::msg::AnyChanMsg::*;
-            match cell.msg() {
+            match cell.0.msg() {
                 Relay(_) | Padding(_) | Vpadding(_) => {} // too frequent to log.
                 _ => trace!(
                     channel_id = %this.unique_id,
                     "Sending {} for {}",
-                    cell.msg().cmd(),
-                    CircId::get_or_zero(cell.circid())
+                    cell.0.msg().cmd(),
+                    CircId::get_or_zero(cell.0.circid())
                 ),
             }
         }
@@ -433,6 +484,8 @@ impl Sink<AnyChanCell> for ChannelSender {
 }
 
 /// Structure for building and launching a Tor channel.
+//
+// TODO(relay): Remove this as we now have ClientChannelBuilder and soon RelayChannelBuilder.
 #[derive(Default)]
 pub struct ChannelBuilder {
     /// If present, a description of the address we're trying to connect to,
@@ -482,7 +535,13 @@ impl ChannelBuilder {
         T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
         S: CoarseTimeProvider + SleepProvider,
     {
-        handshake::ClientInitiatorHandshake::new(tls, self.target, sleep_prov, memquota)
+        // TODO(relay): We could just make the target be taken as a parameter instead of using a
+        // setter that is also replicated on the client builder? Food for thought on refactor here.
+        let mut builder = ClientChannelBuilder::new();
+        if let Some(target) = self.target {
+            builder.set_declared_method(target);
+        }
+        builder.launch(tls, sleep_prov, memquota)
     }
 }
 
@@ -518,6 +577,8 @@ impl Channel {
         let (control_tx, control_rx) = mpsc::unbounded();
         let (cell_tx, cell_rx) = mq_queue::MpscSpec::new(CHANNEL_BUFFER_SIZE)
             .new_mq(dyn_time.clone(), memquota.as_raw_account())?;
+        #[cfg(feature = "circ-padding")]
+        let (cell_tx, cell_rx) = counting_streams::channel(cell_tx, cell_rx);
         let unused_since = AtomicOptTimestamp::new();
         unused_since.update();
 
@@ -530,11 +591,20 @@ impl Channel {
         };
         let details = Arc::new(details);
 
+        // We might be using experimental maybenot padding; this creates the padding framework for that.
+        //
+        // TODO: This backend is currently optimized for circuit padding,
+        // so it might allocate a bit more than necessary to account for multiple hops.
+        // We should tune it when we deploy padding in production.
+        let (padding_ctrl, padding_event_stream) =
+            client::circuit::padding::new_padding(DynTimeProvider::new(sleep_prov.clone()));
+
         let channel = Arc::new(Channel {
             channel_type,
             control: control_tx,
             cell_tx,
             reactor_closed_rx,
+            padding_ctrl: padding_ctrl.clone(),
             unique_id,
             peer_id,
             clock_skew,
@@ -544,9 +614,17 @@ impl Channel {
         });
 
         // We start disabled; the channel manager will `reconfigure` us soon after creation.
-        let padding_timer = Box::pin(padding::Timer::new_disabled(sleep_prov, None)?);
+        let padding_timer = Box::pin(padding::Timer::new_disabled(sleep_prov.clone(), None)?);
+
+        cfg_if! {
+            if #[cfg(feature = "circ-padding")] {
+                use crate::util::sink_blocker::{SinkBlocker,CountingPolicy};
+                let sink = SinkBlocker::new(sink, CountingPolicy::new_unlimited());
+            }
+        }
 
         let reactor = Reactor {
+            runtime: sleep_prov,
             control: control_rx,
             cells: cell_rx,
             reactor_closed_tx,
@@ -559,6 +637,9 @@ impl Channel {
             unique_id,
             details,
             padding_timer,
+            padding_ctrl,
+            padding_event_stream,
+            padding_blocker: None,
             special_outgoing: Default::default(),
         };
 
@@ -580,7 +661,13 @@ impl Channel {
     /// (This can sometimes be used to avoid having to keep
     /// a separate clone of the time provider.)
     pub fn time_provider(&self) -> &DynTimeProvider {
-        self.cell_tx.time_provider()
+        cfg_if! {
+            if #[cfg(feature="circ-padding")] {
+                self.cell_tx.inner().time_provider()
+            } else {
+                self.cell_tx.time_provider()
+            }
+        }
     }
 
     /// Return an OwnedChanTarget representing the actual handshake used to
@@ -601,6 +688,7 @@ impl Channel {
     }
 
     /// Send a control message
+    #[instrument(level = "trace", skip_all)]
     fn send_control(&self, msg: CtrlMsg) -> StdResult<(), ChannelClosed> {
         self.control
             .unbounded_send(msg)
@@ -626,6 +714,7 @@ impl Channel {
     /// Idempotent.
     ///
     /// There is no way to undo the effect of this call.
+    #[instrument(level = "trace", skip_all)]
     pub fn engage_padding_activities(&self) {
         let mut mutable = self.mutable();
 
@@ -663,6 +752,7 @@ impl Channel {
     /// Reparameterise (update parameters; reconfigure)
     ///
     /// Returns `Err` if the channel was closed earlier
+    #[instrument(level = "trace", skip_all)]
     pub fn reparameterize(&self, params: Arc<ChannelPaddingInstructionsUpdates>) -> Result<()> {
         let mut mutable = self
             .mutable
@@ -685,6 +775,7 @@ impl Channel {
     /// Update the KIST parameters.
     ///
     /// Returns `Err` if the channel is closed.
+    #[instrument(level = "trace", skip_all)]
     pub fn reparameterize_kist(&self, kist_params: KistParams) -> Result<()> {
         Ok(self.send_control(CtrlMsg::KistConfigUpdate(kist_params))?)
     }
@@ -717,6 +808,7 @@ impl Channel {
             cell_tx: self.cell_tx.clone(),
             reactor_closed_rx: self.reactor_closed_rx.clone(),
             unique_id: self.unique_id,
+            padding_ctrl: self.padding_ctrl.clone(),
         }
     }
 
@@ -727,14 +819,16 @@ impl Channel {
     /// To use the results of this method, call Reactor::run() in a
     /// new task, then use the methods of
     /// [crate::client::circuit::PendingClientTunnel] to build the circuit.
+    #[instrument(level = "trace", skip_all)]
     pub async fn new_tunnel(
         self: &Arc<Self>,
+        timeouts: Arc<dyn TimeoutEstimator>,
     ) -> Result<(PendingClientTunnel, client::reactor::Reactor)> {
         if self.is_closing() {
             return Err(ChannelClosed.into());
         }
 
-        let time_prov = self.cell_tx.time_provider().clone();
+        let time_prov = self.time_provider().clone();
         let memquota = CircuitAccount::new(&self.details.memquota)?;
 
         // TODO: blocking is risky, but so is unbounded.
@@ -748,7 +842,8 @@ impl Channel {
             sender,
             tx,
         })?;
-        let (id, circ_unique_id) = rx.await.map_err(|_| ChannelClosed)??;
+        let (id, circ_unique_id, padding_ctrl, padding_stream) =
+            rx.await.map_err(|_| ChannelClosed)??;
 
         trace!("{}: Allocated CircId {}", circ_unique_id, id);
 
@@ -760,6 +855,9 @@ impl Channel {
             circ_unique_id,
             time_prov,
             memquota,
+            padding_ctrl,
+            padding_stream,
+            timeouts,
         ))
     }
 
@@ -772,11 +870,13 @@ impl Channel {
     /// It's not necessary to call this method if you're just done
     /// with a channel: the channel should close on its own once nothing
     /// is using it any more.
+    #[instrument(level = "trace", skip_all)]
     pub fn terminate(&self) {
         let _ = self.send_control(CtrlMsg::Shutdown);
     }
 
     /// Tell the reactor that the circuit with the given ID has gone away.
+    #[instrument(level = "trace", skip_all)]
     pub fn close_circuit(&self, circid: CircId) -> Result<()> {
         self.send_control(CtrlMsg::CloseCircuit(circid))?;
         Ok(())
@@ -799,6 +899,36 @@ impl Channel {
             })
     }
 
+    /// Install a [`CircuitPadder`](client::CircuitPadder) for this channel.
+    ///
+    /// Replaces any previous padder installed.
+    #[cfg(feature = "circ-padding-manual")]
+    pub async fn start_padding(self: &Arc<Self>, padder: client::CircuitPadder) -> Result<()> {
+        self.set_padder_impl(Some(padder)).await
+    }
+
+    /// Remove any [`CircuitPadder`](client::CircuitPadder) installed for this channel.
+    ///
+    /// Does nothing if there was not a padder installed there.
+    #[cfg(feature = "circ-padding-manual")]
+    pub async fn stop_padding(self: &Arc<Self>) -> Result<()> {
+        self.set_padder_impl(None).await
+    }
+
+    /// Replace the [`CircuitPadder`](client::CircuitPadder) installed for this channel with `padder`.
+    #[cfg(feature = "circ-padding-manual")]
+    async fn set_padder_impl(
+        self: &Arc<Self>,
+        padder: Option<client::CircuitPadder>,
+    ) -> Result<()> {
+        let (tx, rx) = oneshot::channel();
+        let msg = CtrlMsg::SetChannelPadder { padder, sender: tx };
+        self.control
+            .unbounded_send(msg)
+            .map_err(|_| Error::ChannelClosed(ChannelClosed))?;
+        rx.await.map_err(|_| Error::ChannelClosed(ChannelClosed))?
+    }
+
     /// Make a new fake reactor-less channel.  For testing only, obviously.
     ///
     /// Returns the receiver end of the control message mpsc.
@@ -811,7 +941,10 @@ impl Channel {
     //  * It returns the mpsc Receiver
     //  * It does not require explicit specification of details
     #[cfg(feature = "testing")]
-    pub fn new_fake(channel_type: ChannelType) -> (Channel, mpsc::UnboundedReceiver<CtrlMsg>) {
+    pub fn new_fake(
+        rt: impl SleepProvider + CoarseTimeProvider,
+        channel_type: ChannelType,
+    ) -> (Channel, mpsc::UnboundedReceiver<CtrlMsg>) {
         let (control, control_recv) = mpsc::unbounded();
         let details = fake_channel_details();
 
@@ -824,12 +957,14 @@ impl Channel {
 
         // This will make rx trigger immediately.
         let (_tx, rx) = oneshot_broadcast::channel();
+        let (padding_ctrl, _) = client::circuit::padding::new_padding(DynTimeProvider::new(rt));
 
         let channel = Channel {
             channel_type,
             control,
             cell_tx: fake_mpsc().0,
             reactor_closed_rx: rx,
+            padding_ctrl,
             unique_id,
             peer_id,
             clock_skew: ClockSkew::None,
@@ -845,7 +980,7 @@ impl Channel {
 /// `my_ident`, return a ChanMismatch error.
 ///
 /// This is a helper for [`Channel::check_match`] and
-/// [`UnverifiedChannel::check_internal`].
+/// UnverifiedChannel::check_internal.
 fn check_id_match_helper<T, U>(my_ident: &T, wanted_ident: &U) -> Result<()>
 where
     T: HasRelayIds + ?Sized,
@@ -917,11 +1052,11 @@ fn fake_channel_details() -> Arc<ChannelDetails> {
 
 /// Make an MPSC queue, of the type we use in Channels, but a fake one for testing
 #[cfg(any(test, feature = "testing"))] // Used by Channel::new_fake which is also feature=testing
-pub(crate) fn fake_mpsc() -> (
-    mq_queue::Sender<AnyChanCell, mq_queue::MpscSpec>,
-    mq_queue::Receiver<AnyChanCell, mq_queue::MpscSpec>,
-) {
-    crate::fake_mpsc(CHANNEL_BUFFER_SIZE)
+pub(crate) fn fake_mpsc() -> (CellTx, CellRx) {
+    let (tx, rx) = crate::fake_mpsc(CHANNEL_BUFFER_SIZE);
+    #[cfg(feature = "circ-padding")]
+    let (tx, rx) = counting_streams::channel(tx, rx);
+    (tx, rx)
 }
 
 #[cfg(test)]
@@ -935,10 +1070,13 @@ pub(crate) mod test {
     use crate::util::fake_mq;
     use tor_cell::chancell::msg::HandshakeType;
     use tor_cell::chancell::{AnyChanCell, msg};
-    use tor_rtcompat::PreferredRuntime;
+    use tor_rtcompat::{PreferredRuntime, test_with_one_runtime};
 
     /// Make a new fake reactor-less channel.  For testing only, obviously.
-    pub(crate) fn fake_channel(channel_type: ChannelType) -> Channel {
+    pub(crate) fn fake_channel(
+        rt: impl SleepProvider + CoarseTimeProvider,
+        channel_type: ChannelType,
+    ) -> Channel {
         let unique_id = UniqId::new();
         let peer_id = OwnedChanTarget::builder()
             .ed_identity([6_u8; 32].into())
@@ -947,11 +1085,13 @@ pub(crate) mod test {
             .expect("Couldn't construct peer id");
         // This will make rx trigger immediately.
         let (_tx, rx) = oneshot_broadcast::channel();
+        let (padding_ctrl, _) = client::circuit::padding::new_padding(DynTimeProvider::new(rt));
         Channel {
             channel_type,
             control: mpsc::unbounded().0,
             cell_tx: fake_mpsc().0,
             reactor_closed_rx: rx,
+            padding_ctrl,
             unique_id,
             peer_id,
             clock_skew: ClockSkew::None,
@@ -963,9 +1103,9 @@ pub(crate) mod test {
 
     #[test]
     fn send_bad() {
-        tor_rtcompat::test_with_all_runtimes!(|_rt| async move {
+        tor_rtcompat::test_with_all_runtimes!(|rt| async move {
             use std::error::Error;
-            let chan = fake_channel(ChannelType::ClientInitiator);
+            let chan = fake_channel(rt, ChannelType::ClientInitiator);
 
             let cell = AnyChanCell::new(CircId::new(7), msg::Created2::new(&b"hihi"[..]).into());
             let e = chan.sender().check_cell(&cell);
@@ -1007,42 +1147,48 @@ pub(crate) mod test {
 
     #[test]
     fn check_match() {
-        let chan = fake_channel(ChannelType::ClientInitiator);
+        test_with_one_runtime!(|rt| async move {
+            let chan = fake_channel(rt, ChannelType::ClientInitiator);
 
-        let t1 = OwnedChanTarget::builder()
-            .ed_identity([6; 32].into())
-            .rsa_identity([10; 20].into())
-            .build()
-            .unwrap();
-        let t2 = OwnedChanTarget::builder()
-            .ed_identity([1; 32].into())
-            .rsa_identity([3; 20].into())
-            .build()
-            .unwrap();
-        let t3 = OwnedChanTarget::builder()
-            .ed_identity([3; 32].into())
-            .rsa_identity([2; 20].into())
-            .build()
-            .unwrap();
+            let t1 = OwnedChanTarget::builder()
+                .ed_identity([6; 32].into())
+                .rsa_identity([10; 20].into())
+                .build()
+                .unwrap();
+            let t2 = OwnedChanTarget::builder()
+                .ed_identity([1; 32].into())
+                .rsa_identity([3; 20].into())
+                .build()
+                .unwrap();
+            let t3 = OwnedChanTarget::builder()
+                .ed_identity([3; 32].into())
+                .rsa_identity([2; 20].into())
+                .build()
+                .unwrap();
 
-        assert!(chan.check_match(&t1).is_ok());
-        assert!(chan.check_match(&t2).is_err());
-        assert!(chan.check_match(&t3).is_err());
+            assert!(chan.check_match(&t1).is_ok());
+            assert!(chan.check_match(&t2).is_err());
+            assert!(chan.check_match(&t3).is_err());
+        });
     }
 
     #[test]
     fn unique_id() {
-        let ch1 = fake_channel(ChannelType::ClientInitiator);
-        let ch2 = fake_channel(ChannelType::ClientInitiator);
-        assert_ne!(ch1.unique_id(), ch2.unique_id());
+        test_with_one_runtime!(|rt| async move {
+            let ch1 = fake_channel(rt.clone(), ChannelType::ClientInitiator);
+            let ch2 = fake_channel(rt, ChannelType::ClientInitiator);
+            assert_ne!(ch1.unique_id(), ch2.unique_id());
+        });
     }
 
     #[test]
     fn duration_unused_at() {
-        let details = fake_channel_details();
-        let mut ch = fake_channel(ChannelType::ClientInitiator);
-        ch.details = details.clone();
-        details.unused_since.update();
-        assert!(ch.duration_unused().is_some());
+        test_with_one_runtime!(|rt| async move {
+            let details = fake_channel_details();
+            let mut ch = fake_channel(rt, ChannelType::ClientInitiator);
+            ch.details = details.clone();
+            details.unused_since.update();
+            assert!(ch.duration_unused().is_some());
+        });
     }
 }
