@@ -6,14 +6,13 @@ use futures::stream::StreamExt;
 use tor_cell::chancell::msg::AnyChanMsg;
 use tor_error::internal;
 
-use crate::channel::{ChannelType, UniqId, new_frame};
+use crate::channel::{ChannelFrame, ChannelType, UniqId};
 use crate::memquota::ChannelAccount;
 use crate::util::skew::ClockSkew;
 use crate::{Error, Result};
 use tor_cell::chancell::{AnyChanCell, ChanMsg, msg};
 use tor_rtcompat::{CoarseTimeProvider, SleepProvider, StreamOps};
 
-use std::net::SocketAddr;
 use std::sync::Arc;
 use std::time::SystemTime;
 
@@ -24,145 +23,38 @@ use tor_llcrypto::pk::rsa::RsaIdentity;
 
 use digest::Digest;
 
-use super::ChannelFrame;
-
-use tracing::{debug, trace};
+use tracing::{debug, instrument, trace};
 
 /// A list of the link protocols that we support.
-static LINK_PROTOCOLS: &[u16] = &[4, 5];
+pub(crate) static LINK_PROTOCOLS: &[u16] = &[4, 5];
 
-/// A raw client channel on which nothing has been done.
-pub struct ClientInitiatorHandshake<
-    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
-    S: CoarseTimeProvider + SleepProvider,
-> {
-    /// Runtime handle (insofar as we need it)
-    sleep_prov: S,
-
-    /// Memory quota account
-    memquota: ChannelAccount,
-
-    /// Underlying TLS stream.
-    ///
-    /// (We don't enforce that this is actually TLS, but if it isn't, the
-    /// connection won't be secure.)
-    tls: T,
-
-    /// Declared target method for this channel, if any.
-    target_method: Option<ChannelMethod>,
-
-    /// Logging identifier for this stream.  (Used for logging only.)
-    unique_id: UniqId,
-}
-
-/// A client channel on which versions have been negotiated and the
-/// relay's handshake has been read, but where the certs have not
-/// been checked.
-pub struct UnverifiedChannel<
-    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
-    S: CoarseTimeProvider + SleepProvider,
-> {
-    /// Indicate what type of channel this is.
-    channel_type: ChannelType,
-    /// Runtime handle (insofar as we need it)
-    sleep_prov: S,
-    /// Memory quota account
-    memquota: ChannelAccount,
-    /// The negotiated link protocol.  Must be a member of LINK_PROTOCOLS
-    link_protocol: u16,
-    /// The Source+Sink on which we're reading and writing cells.
-    framed_tls: ChannelFrame<T>,
-    /// The certs cell that we got from the relay.
-    certs_cell: msg::Certs,
-    /// Declared target method for this channel, if any.
-    target_method: Option<ChannelMethod>,
-    /// The netinfo cell that we got from the relay.
-    #[allow(dead_code)] // Relays will need this.
-    netinfo_cell: msg::Netinfo,
-    /// How much clock skew did we detect in this handshake?
-    ///
-    /// This value is _unauthenticated_, since we have not yet checked whether
-    /// the keys in the handshake are the ones we expected.
-    clock_skew: ClockSkew,
-    /// Logging identifier for this stream.  (Used for logging only.)
-    unique_id: UniqId,
-}
-
-/// A client channel on which versions have been negotiated,
-/// relay's handshake has been read, but the client has not yet
-/// finished the handshake.
+/// Base trait that all handshake type must implement.
 ///
-/// This type is separate from UnverifiedChannel, since finishing the
-/// handshake requires a bunch of CPU, and you might want to do it as
-/// a separate task or after a yield.
-pub struct VerifiedChannel<
+/// It has common code that all handshake share including getters for the channel frame for cell
+/// decoding/encoding and the unique ID used for logging.
+///
+/// It has both a recv() and send() function for the VERSIONS cell since every handshake must start
+/// with this cell to negotiate the link protocol version.
+pub(crate) trait ChannelBaseHandshake<T>
+where
     T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
-    S: CoarseTimeProvider + SleepProvider,
-> {
-    /// Indicate what type of channel this is.
-    channel_type: ChannelType,
-    /// Runtime handle (insofar as we need it)
-    sleep_prov: S,
-    /// Memory quota account
-    memquota: ChannelAccount,
-    /// The negotiated link protocol.
-    link_protocol: u16,
-    /// The Source+Sink on which we're reading and writing cells.
-    framed_tls: ChannelFrame<T>,
-    /// Declared target method for this stream, if any.
-    target_method: Option<ChannelMethod>,
-    /// Logging identifier for this stream.  (Used for logging only.)
-    unique_id: UniqId,
-    /// Validated Ed25519 identity for this peer.
-    ed25519_id: Ed25519Identity,
-    /// Validated RSA identity for this peer.
-    rsa_id: RsaIdentity,
-    /// Authenticated clock skew for this peer.
-    clock_skew: ClockSkew,
-}
-
-impl<
-    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
-    S: CoarseTimeProvider + SleepProvider,
-> ClientInitiatorHandshake<T, S>
 {
-    /// Construct a new OutboundClientHandshake.
-    pub(crate) fn new(
-        tls: T,
-        target_method: Option<ChannelMethod>,
-        sleep_prov: S,
-        memquota: ChannelAccount,
-    ) -> Self {
-        Self {
-            tls,
-            target_method,
-            unique_id: UniqId::new(),
-            sleep_prov,
-            memquota,
-        }
-    }
+    /// Return a mutable reference to the channel frame.
+    fn framed_tls(&mut self) -> &mut ChannelFrame<T>;
+    /// Return a reference to the unique ID of this handshake.
+    fn unique_id(&self) -> &UniqId;
 
-    /// Negotiate a link protocol version with the relay, and read
-    /// the relay's handshake information.
+    /// Send a [msg::Versions] cell.
     ///
-    /// Takes a function that reports the current time.  In theory, this can just be
-    /// `SystemTime::now()`.
-    pub async fn connect<F>(mut self, now_fn: F) -> Result<UnverifiedChannel<T, S>>
+    /// A tuple is returned that is respectively the instant and wallclock of the send.
+    async fn send_versions_cell<F>(
+        &mut self,
+        now_fn: F,
+    ) -> Result<(coarsetime::Instant, SystemTime)>
     where
         F: FnOnce() -> SystemTime,
     {
-        // New channel frame as a Client Initiator.
-        let mut framed_tls = new_frame(self.tls, ChannelType::ClientInitiator);
-
-        match &self.target_method {
-            Some(method) => debug!(
-                stream_id = %self.unique_id,
-                "starting Tor handshake with {:?}",
-                method
-            ),
-            None => debug!(stream_id = %self.unique_id, "starting Tor handshake"),
-        }
-        trace!(stream_id = %self.unique_id, "sending versions");
+        trace!(stream_id = %self.unique_id(), "sending versions");
         // Send versions cell
         let version_cell = AnyChanCell::new(
             None,
@@ -170,15 +62,26 @@ impl<
                 .map_err(|e| Error::from_cell_enc(e, "versions message"))?
                 .into(),
         );
-        framed_tls.send(version_cell).await?;
-        let versions_flushed_at = coarsetime::Instant::now();
-        let versions_flushed_wallclock = now_fn();
+        self.framed_tls().send(version_cell).await?;
+        Ok((
+            coarsetime::Instant::now(), // Flushed at instant
+            now_fn(),                   // Flushed at wallclock
+        ))
+    }
 
+    /// Receive a [msg::Versions] cell.
+    ///
+    /// The negotiated link protocol is returned, and also recorded in the underlying channel
+    /// frame. This automatically transitions the frame into the "Handshake" state of the
+    /// underlying cell handler. In other words, once the link protocol version is negotiated, the
+    /// handler can encode and decode cells for that version in order to continue the handshake.
+    async fn recv_versions_cell(&mut self) -> Result<u16> {
         // Get versions cell.
-        trace!(stream_id = %self.unique_id, "waiting for versions");
+        // Get versions cell.
+        trace!(stream_id = %self.unique_id(), "waiting for versions");
         // This can be None if we've reached EOF or any type of I/O error on the underlying TCP or
         // TLS stream. Either case, it is unexpected.
-        let Some(cell) = framed_tls.next().await.transpose()? else {
+        let Some(cell) = self.framed_tls().next().await.transpose()? else {
             return Err(Error::ChanIoErr(Arc::new(std::io::Error::from(
                 std::io::ErrorKind::UnexpectedEof,
             ))));
@@ -188,113 +91,190 @@ impl<
                 "Unexpected cell, expecting a VERSIONS cell",
             )));
         };
-        trace!(stream_id = %self.unique_id, "received their VERSIONS {:?}", their_versions);
+        trace!(stream_id = %self.unique_id(), "received their VERSIONS {:?}", their_versions);
 
         // Determine which link protocol we negotiated.
         let link_protocol = their_versions
             .best_shared_link_protocol(LINK_PROTOCOLS)
             .ok_or_else(|| Error::HandshakeProto("No shared link protocols".into()))?;
-        trace!(stream_id = %self.unique_id, "negotiated version {}", link_protocol);
+        trace!(stream_id = %self.unique_id(), "negotiated version {}", link_protocol);
 
         // Set the link protocol into our channel frame.
-        framed_tls.codec_mut().set_link_version(link_protocol)?;
+        self.framed_tls()
+            .codec_mut()
+            .set_link_version(link_protocol)?;
+        Ok(link_protocol)
+    }
+}
 
-        // Read until we have the netinfo cells.
-        let mut certs: Option<msg::Certs> = None;
-        let mut netinfo: Option<(msg::Netinfo, coarsetime::Instant)> = None;
-        let mut seen_authchallenge = false;
+/// Handshake initiator base trait. All initiator handshake should implement this trait in order to
+/// enjoy the helper functions.
+///
+/// It requires the base handshake trait to be implement for access to the base getters.
+pub(crate) trait ChannelInitiatorHandshake<T>: ChannelBaseHandshake<T>
+where
+    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
+{
+    /// Return true iff this handshake is expecting to receive an AUTH_CHALLENGE from the
+    /// responder. As a handshake initiator, we always know if we expect one or not. A client or
+    /// bridge do not authenticate with the responder while relays will always do.
+    fn is_expecting_auth_challenge(&self) -> bool;
 
-        // Loop: reject duplicate and unexpected cells
-        trace!(stream_id = %self.unique_id, "waiting for rest of handshake.");
-        while let Some(cell) = framed_tls.next().await.transpose()? {
+    /// As an initiator, we are expecting the responder's cells which are (not in that order):
+    ///     - [msg::AuthChallenge], [msg::Certs], [msg::Netinfo]
+    ///
+    /// Any duplicate, missing cell or unexpected results in a protocol level error.
+    ///
+    /// This returns the [msg::AuthChallenge], [msg::Certs] and [msg::Netinfo] cells along the
+    /// instant when the netinfo cell was received. This is needed for the clock skew calculation.
+    async fn recv_cells_from_responder(
+        &mut self,
+    ) -> Result<(
+        Option<msg::AuthChallenge>,
+        msg::Certs,
+        (msg::Netinfo, coarsetime::Instant),
+    )> {
+        let mut auth_challenge_cell: Option<msg::AuthChallenge> = None;
+        let mut certs_cell: Option<msg::Certs> = None;
+        let mut netinfo_cell: Option<(msg::Netinfo, coarsetime::Instant)> = None;
+
+        // IMPORTANT: Protocol wise, we MUST only allow one single cell of each type for a valid
+        // handshake. Any duplicates lead to a failure. They can arrive in any order unfortunately.
+
+        // Read until we have the netinfo cell.
+        while let Some(cell) = self.framed_tls().next().await.transpose()? {
             use super::AnyChanMsg::*;
             let (_, m) = cell.into_circid_and_msg();
-            trace!("{}: received a {} cell.", self.unique_id, m.cmd());
+            trace!(stream_id = %self.unique_id(), "received a {} cell.", m.cmd());
             match m {
-                // Are these technically allowed?
-                Padding(_) | Vpadding(_) => (),
+                // Ignore the padding. Only VPADDING cell can be sent during handshaking.
+                Vpadding(_) => (),
                 // Clients don't care about AuthChallenge
-                AuthChallenge(_) => {
-                    if seen_authchallenge {
-                        return Err(Error::HandshakeProto("Duplicate authchallenge cell".into()));
+                AuthChallenge(ac) => {
+                    if auth_challenge_cell.replace(ac).is_some() {
+                        return Err(Error::HandshakeProto(
+                            "Duplicate AUTH_CHALLENGE cell".into(),
+                        ));
                     }
-                    seen_authchallenge = true;
                 }
                 Certs(c) => {
-                    if certs.is_some() {
-                        return Err(Error::HandshakeProto("Duplicate certs cell".into()));
+                    if certs_cell.replace(c).is_some() {
+                        return Err(Error::HandshakeProto("Duplicate CERTS cell".into()));
                     }
-                    certs = Some(c);
                 }
                 Netinfo(n) => {
-                    if netinfo.is_some() {
+                    if netinfo_cell.is_some() {
                         // This should be impossible, since we would
                         // exit this loop on the first netinfo cell.
                         return Err(Error::from(internal!(
                             "Somehow tried to record a duplicate NETINFO cell"
                         )));
                     }
-                    netinfo = Some((n, coarsetime::Instant::now()));
+                    netinfo_cell = Some((n, coarsetime::Instant::now()));
                     break;
                 }
-                _ => return Err(Error::from(internal!("Unexpected cell {m:?}"))),
+                // This should not happen because the ChannelFrame makes sure that only allowed cell on
+                // the channel are decoded. However, Rust wants us to consider all AnyChanMsg.
+                _ => {
+                    return Err(Error::from(internal!(
+                        "Unexpected cell during initiator handshake: {m:?}"
+                    )));
+                }
             }
         }
 
-        // If we have certs and netinfo, we can finish authenticating.
-        match (certs, netinfo) {
-            (Some(_), None) => Err(Error::HandshakeProto(
-                "Missing netinfo or closed stream".into(),
-            )),
-            (None, _) => Err(Error::HandshakeProto("Missing certs cell".into())),
-            (Some(certs_cell), Some((netinfo_cell, netinfo_rcvd_at))) => {
-                trace!(stream_id = %self.unique_id, "received handshake, ready to verify.");
-                // Try to compute our clock skew.  It won't be authenticated
-                // yet, since we haven't checked the certificates.
-                let clock_skew = if let Some(netinfo_timestamp) = netinfo_cell.timestamp() {
-                    let delay = netinfo_rcvd_at - versions_flushed_at;
-                    ClockSkew::from_handshake_timestamps(
-                        versions_flushed_wallclock,
-                        netinfo_timestamp,
-                        delay.into(),
-                    )
-                } else {
-                    ClockSkew::None
-                };
-                Ok(UnverifiedChannel {
-                    channel_type: ChannelType::ClientInitiator,
-                    link_protocol,
-                    framed_tls,
-                    certs_cell,
-                    netinfo_cell,
-                    clock_skew,
-                    target_method: self.target_method.take(),
-                    unique_id: self.unique_id,
-                    sleep_prov: self.sleep_prov.clone(),
-                    memquota: self.memquota.clone(),
-                })
-            }
-        }
+        // Missing any of the above means we are not connected to a Relay and so we abort the
+        // handshake protocol.
+        let Some((netinfo, netinfo_rcvd_at)) = netinfo_cell else {
+            return Err(Error::HandshakeProto("Missing NETINFO cell".into()));
+        };
+        let Some(certs) = certs_cell else {
+            return Err(Error::HandshakeProto("Missing CERTS cell".into()));
+        };
+        // If we plan to authenticate, we require an AUTH_CHALLENGE cell from the responder.
+        if self.is_expecting_auth_challenge() && auth_challenge_cell.is_none() {
+            return Err(Error::HandshakeProto("Missing AUTH_CHALLENGE cell".into()));
+        };
+
+        Ok((auth_challenge_cell, certs, (netinfo, netinfo_rcvd_at)))
     }
 }
 
+/// A base channel on which versions have been negotiated and the relay's handshake has been read,
+/// but where the certs have not been checked.
+///
+/// Both relay and client have specialized objects for an unverified channel which include this one
+/// as the base in order to share functionnalities.
+pub(crate) struct UnverifiedChannel<
+    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
+    S: CoarseTimeProvider + SleepProvider,
+> {
+    /// Indicate what type of channel this is.
+    pub(crate) channel_type: ChannelType,
+    /// Runtime handle (insofar as we need it)
+    pub(crate) sleep_prov: S,
+    /// Memory quota account
+    pub(crate) memquota: ChannelAccount,
+    /// The negotiated link protocol.  Must be a member of LINK_PROTOCOLS
+    pub(crate) link_protocol: u16,
+    /// The Source+Sink on which we're reading and writing cells.
+    pub(crate) framed_tls: ChannelFrame<T>,
+    /// The certs cell that we got from the relay.
+    pub(crate) certs_cell: msg::Certs,
+    /// Declared target method for this channel, if any.
+    pub(crate) target_method: Option<ChannelMethod>,
+    /// How much clock skew did we detect in this handshake?
+    ///
+    /// This value is _unauthenticated_, since we have not yet checked whether
+    /// the keys in the handshake are the ones we expected.
+    pub(crate) clock_skew: ClockSkew,
+    /// Logging identifier for this stream.  (Used for logging only.)
+    pub(crate) unique_id: UniqId,
+}
+
+/// A base channel on which versions have been negotiated, relay's handshake has been read, but the
+/// client has not yet finished the handshake.
+///
+/// This type is separate from UnverifiedChannel, since finishing the handshake requires a bunch of
+/// CPU, and you might want to do it as a separate task or after a yield.
+///
+/// Both relay and client have specialized objects for an unverified channel which include this one
+/// as the base in order to share functionnalities.
+pub(crate) struct VerifiedChannel<
+    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
+    S: CoarseTimeProvider + SleepProvider,
+> {
+    /// Indicate what type of channel this is.
+    pub(crate) channel_type: ChannelType,
+    /// Runtime handle (insofar as we need it)
+    pub(crate) sleep_prov: S,
+    /// Memory quota account
+    pub(crate) memquota: ChannelAccount,
+    /// The negotiated link protocol.
+    pub(crate) link_protocol: u16,
+    /// The Source+Sink on which we're reading and writing cells.
+    pub(crate) framed_tls: ChannelFrame<T>,
+    /// Declared target method for this stream, if any.
+    pub(crate) target_method: Option<ChannelMethod>,
+    /// Logging identifier for this stream.  (Used for logging only.)
+    pub(crate) unique_id: UniqId,
+    /// Validated Ed25519 identity for this peer.
+    pub(crate) ed25519_id: Ed25519Identity,
+    /// Validated RSA identity for this peer.
+    pub(crate) rsa_id: RsaIdentity,
+    /// Peer verified RSA cert digest.
+    pub(crate) rsa_cert_digest: [u8; 32],
+    /// Peer TLS certificate digest
+    pub(crate) peer_cert_digest: [u8; 32],
+    /// Authenticated clock skew for this peer.
+    pub(crate) clock_skew: ClockSkew,
+}
+
 impl<
     T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
     S: CoarseTimeProvider + SleepProvider,
 > UnverifiedChannel<T, S>
 {
-    /// Return the reported clock skew from this handshake.
-    ///
-    /// Note that the skew reported by this function might not be "true": the
-    /// relay might have its clock set wrong, or it might be lying to us.
-    ///
-    /// The clock skew reported here is not yet authenticated; if you need to
-    /// make sure that the skew is authenticated, use
-    /// [`Channel::clock_skew`](super::Channel::clock_skew) instead.
-    pub fn clock_skew(&self) -> ClockSkew {
-        self.clock_skew
-    }
-
     /// Validate the certificates and keys in the relay's handshake.
     ///
     /// 'peer' is the peer that we want to make sure we're connecting to.
@@ -308,22 +288,23 @@ impl<
     ///
     /// This is a separate function because it's likely to be somewhat
     /// CPU-intensive.
-    pub fn check<U: ChanTarget + ?Sized>(
+    #[instrument(skip_all, level = "trace")]
+    pub(crate) fn check<U: ChanTarget + ?Sized>(
         self,
         peer: &U,
         peer_cert: &[u8],
         now: Option<std::time::SystemTime>,
     ) -> Result<VerifiedChannel<T, S>> {
-        let peer_cert_sha256 = ll::d::Sha256::digest(peer_cert);
-        self.check_internal(peer, &peer_cert_sha256[..], now)
+        let peer_cert_sha256 = ll::d::Sha256::digest(peer_cert).into();
+        self.check_internal(peer, peer_cert_sha256, now)
     }
 
     /// Same as `check`, but takes the SHA256 hash of the peer certificate,
     /// since that is all we use.
-    fn check_internal<U: ChanTarget + ?Sized>(
+    pub(crate) fn check_internal<U: ChanTarget + ?Sized>(
         self,
         peer: &U,
-        peer_cert_sha256: &[u8],
+        peer_cert_digest: [u8; 32],
         now: Option<SystemTime>,
     ) -> Result<VerifiedChannel<T, S>> {
         use tor_cert::CertType;
@@ -424,7 +405,7 @@ impl<
         sigs.push(&sk_tls_sig);
         let (sk_tls_timeliness, sk_tls) = check_timeliness(sk_tls, now, self.clock_skew);
 
-        if peer_cert_sha256 != sk_tls.subject_key().as_bytes() {
+        if peer_cert_digest != sk_tls.subject_key().as_bytes() {
             return Err(Error::HandshakeProto(
                 "Peer cert did not authenticate TLS cert".into(),
             ));
@@ -530,6 +511,8 @@ impl<
             target_method: self.target_method,
             ed25519_id: *identity_key,
             rsa_id,
+            rsa_cert_digest: *rsa_cert.digest(),
+            peer_cert_digest,
             clock_skew: self.clock_skew,
             sleep_prov: self.sleep_prov,
             memquota: self.memquota,
@@ -542,34 +525,23 @@ impl<
     S: CoarseTimeProvider + SleepProvider,
 > VerifiedChannel<T, S>
 {
-    /// Send a 'Netinfo' message to the relay to finish the handshake,
-    /// and create an open channel and reactor.
-    ///
     /// The channel is used to send cells, and to create outgoing circuits.
     /// The reactor is used to route incoming messages to their appropriate
     /// circuit.
-    pub async fn finish(mut self) -> Result<(Arc<super::Channel>, super::reactor::Reactor<S>)> {
+    #[instrument(skip_all, level = "trace")]
+    pub(crate) async fn finish(
+        mut self,
+    ) -> Result<(Arc<super::Channel>, super::reactor::Reactor<S>)> {
         // We treat a completed channel -- that is to say, one where the
         // authentication is finished -- as incoming traffic.
         //
         // TODO: conceivably we should remember the time when we _got_ the
         // final cell on the handshake, and update the channel completion
         // time to be no earlier than _that_ timestamp.
+        //
+        // TODO: This shouldn't be here. This should be called in the trait functions that actually
+        // receives the data (recv_*). We'll move it at a later commit.
         crate::note_incoming_traffic();
-        trace!(stream_id = %self.unique_id, "Sending netinfo cell.");
-
-        // We do indeed want the real IP here, regardless of whether the
-        // ChannelMethod is Direct connection or not.  The role of the IP in a
-        // NETINFO cell is to tell our peer what address we believe they had, so
-        // that they can better notice MITM attacks and such.
-        let peer_ip = self
-            .target_method
-            .as_ref()
-            .and_then(ChannelMethod::socket_addrs)
-            .and_then(|addrs| addrs.first())
-            .map(SocketAddr::ip);
-        let netinfo = msg::Netinfo::from_client(peer_ip);
-        self.framed_tls.send(netinfo.into()).await?;
 
         // We have finalized the handshake, move our codec to Open.
         self.framed_tls.codec_mut().set_open()?;
@@ -617,6 +589,31 @@ impl<
     }
 }
 
+/// Helper: Calculate a clock skew from the [msg::Netinfo] cell data and the time at which we sent
+/// the [msg::Versions] cell.
+///
+/// This is unauthenticated as in not validated with the certificates. Before using it, make sure
+/// that you have authenticated the other party.
+pub(crate) fn unauthenticated_clock_skew(
+    netinfo_cell: &msg::Netinfo,
+    netinfo_rcvd_at: coarsetime::Instant,
+    versions_flushed_at: coarsetime::Instant,
+    versions_flushed_wallclock: SystemTime,
+) -> ClockSkew {
+    // Try to compute our clock skew.  It won't be authenticated yet, since we haven't checked
+    // the certificates.
+    if let Some(netinfo_timestamp) = netinfo_cell.timestamp() {
+        let delay = netinfo_rcvd_at - versions_flushed_at;
+        ClockSkew::from_handshake_timestamps(
+            versions_flushed_wallclock,
+            netinfo_timestamp,
+            delay.into(),
+        )
+    } else {
+        ClockSkew::None
+    }
+}
+
 #[cfg(test)]
 pub(super) mod test {
     #![allow(clippy::unwrap_used)]
@@ -625,9 +622,10 @@ pub(super) mod test {
     use std::time::{Duration, SystemTime};
 
     use super::*;
-    use crate::Result;
     use crate::channel::handler::test::MsgBuf;
+    use crate::channel::new_frame;
     use crate::util::fake_mq;
+    use crate::{Result, channel::ClientInitiatorHandshake};
     use tor_cell::chancell::msg;
     use tor_linkspec::OwnedChanTarget;
     use tor_rtcompat::{PreferredRuntime, Runtime};
@@ -680,11 +678,11 @@ pub(super) mod test {
             let handshake = ClientInitiatorHandshake::new(mb, None, rt.clone(), fake_mq());
             let unverified = handshake.connect(|| now).await?;
 
-            assert_eq!(unverified.link_protocol, 5);
+            assert_eq!(unverified.link_protocol(), 5);
             // No timestamp in the NETINFO, so no skew.
             assert_eq!(unverified.clock_skew(), ClockSkew::None);
 
-            // Try again with an authchallenge cell and some padding.
+            // Try again with some padding.
             let mut buf = Vec::new();
             buf.extend_from_slice(VERSIONS);
             buf.extend_from_slice(NOCERTS);
@@ -764,7 +762,7 @@ pub(super) mod test {
             assert!(matches!(err, Error::HandshakeProto(_)));
             assert_eq!(
                 format!("{}", err),
-                "Handshake protocol violation: Duplicate certs cell"
+                "Handshake protocol violation: Duplicate CERTS cell"
             );
 
             let mut buf = Vec::new();
@@ -777,7 +775,7 @@ pub(super) mod test {
             assert!(matches!(err, Error::HandshakeProto(_)));
             assert_eq!(
                 format!("{}", err),
-                "Handshake protocol violation: Duplicate authchallenge cell"
+                "Handshake protocol violation: Duplicate AUTH_CHALLENGE cell"
             );
         });
     }
@@ -792,7 +790,7 @@ pub(super) mod test {
             assert!(matches!(err, Error::HandshakeProto(_)));
             assert_eq!(
                 format!("{}", err),
-                "Handshake protocol violation: Missing certs cell"
+                "Handshake protocol violation: Missing CERTS cell"
             );
         });
     }
@@ -807,7 +805,7 @@ pub(super) mod test {
             assert!(matches!(err, Error::HandshakeProto(_)));
             assert_eq!(
                 format!("{}", err),
-                "Handshake protocol violation: Missing netinfo or closed stream"
+                "Handshake protocol violation: Missing NETINFO cell"
             );
         });
     }
@@ -832,8 +830,6 @@ pub(super) mod test {
     where
         R: Runtime,
     {
-        let localhost = std::net::IpAddr::V4(std::net::Ipv4Addr::LOCALHOST);
-        let netinfo_cell = msg::Netinfo::from_client(Some(localhost));
         let mut framed_tls = new_frame(MsgBuf::new(&b""[..]), ChannelType::ClientInitiator);
         let _ = framed_tls.codec_mut().set_link_version(4);
         let _ = framed_tls.codec_mut().set_open();
@@ -843,7 +839,6 @@ pub(super) mod test {
             link_protocol: 4,
             framed_tls,
             certs_cell: certs,
-            netinfo_cell,
             clock_skew,
             target_method: None,
             unique_id: UniqId::new(),
@@ -863,7 +858,7 @@ pub(super) mod test {
         when: Option<SystemTime>,
         peer_ed: &[u8],
         peer_rsa: &[u8],
-        peer_cert_sha256: &[u8],
+        peer_cert_sha256: [u8; 32],
         runtime: &R,
     ) -> Result<VerifiedChannel<MsgBuf, R>>
     where
@@ -889,7 +884,7 @@ pub(super) mod test {
             None,
             &[0_u8; 32],
             &[0_u8; 20],
-            &[0_u8; 128],
+            [0_u8; 32],
             &rt,
         )
         .err()
@@ -914,7 +909,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             certs::PEER_ED,
             certs::PEER_RSA,
-            certs::PEER_CERT_DIGEST,
+            *certs::PEER_CERT_DIGEST,
             &rt,
         );
         let _ = res.unwrap();
@@ -947,7 +942,7 @@ pub(super) mod test {
                 Some(cert_timestamp()),
                 certs::PEER_ED,
                 certs::PEER_RSA,
-                certs::PEER_CERT_DIGEST,
+                *certs::PEER_CERT_DIGEST,
                 &rt,
             )
             .err()
@@ -973,7 +968,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             &[0x10; 32],
             certs::PEER_RSA,
-            certs::PEER_CERT_DIGEST,
+            *certs::PEER_CERT_DIGEST,
             &rt,
         )
         .err()
@@ -991,7 +986,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             certs::PEER_ED,
             &[0x99; 20],
-            certs::PEER_CERT_DIGEST,
+            *certs::PEER_CERT_DIGEST,
             &rt,
         )
         .err()
@@ -1009,7 +1004,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             certs::PEER_ED,
             certs::PEER_RSA,
-            &[0; 32],
+            [0; 32],
             &rt,
         )
         .err()
@@ -1039,7 +1034,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             certs::PEER_ED,
             certs::PEER_RSA,
-            certs::PEER_CERT_DIGEST,
+            *certs::PEER_CERT_DIGEST,
             &rt,
         )
         .err()
@@ -1060,7 +1055,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             certs::PEER_ED,
             certs::PEER_RSA,
-            certs::PEER_CERT_DIGEST,
+            *certs::PEER_CERT_DIGEST,
             &rt,
         )
         .err()
@@ -1095,7 +1090,7 @@ pub(super) mod test {
             "DCB604DB2034B00FD16986D4ADB9D16B21CB4E4457A33DEC0F538903683E96E90006DA3A805CF6006F9179066534DE6B45AD47A5C469063EE462762723396DC9F25452A0A52DA3F5087DD239F2A311F6B0D4DFEFF4ABD089DC3D0237A0ABAB19EB2045B91CDCAF04BE0A72D548A27BF2E77BD876ECFE5E1BE622350DA6BF31F6E306ED896488DD5B39409B23FC3EB7B2C9F7328EB18DA36D54D80575899EA6507CCBFCDF1F"
         );
 
-        pub(crate) const PEER_CERT_DIGEST: &[u8] =
+        pub(crate) const PEER_CERT_DIGEST: &[u8; 32] =
             &hex!("b4fd606b64e4cbd466b8d76cb131069bae6f3aa1878857c9f624e31d77a799b8");
 
         pub(crate) const PEER_ED: &[u8] =
@@ -1119,6 +1114,8 @@ pub(super) mod test {
                 target_method: Some(ChannelMethod::Direct(vec![peer_addr])),
                 ed25519_id,
                 rsa_id,
+                rsa_cert_digest: [0; 32],
+                peer_cert_digest: [0; 32],
                 clock_skew: ClockSkew::None,
                 sleep_prov: rt,
                 memquota: fake_mq(),
