@@ -8,16 +8,18 @@
 
 use super::circmap::{CircEnt, CircMap};
 use crate::client::circuit::halfcirc::HalfCirc;
+use crate::client::circuit::padding::{
+    PaddingController, PaddingEvent, PaddingEventStream, SendPadding, StartBlocking,
+};
 use crate::util::err::ReactorError;
 use crate::util::oneshot_broadcast;
-use crate::{Error, Result};
+use crate::{Error, HopNum, Result};
 use tor_async_utils::SinkPrepareExt as _;
 use tor_cell::chancell::ChanMsg;
-use tor_cell::chancell::msg::{Destroy, DestroyReason, PaddingNegotiate};
+use tor_cell::chancell::msg::{Destroy, DestroyReason, Padding, PaddingNegotiate};
 use tor_cell::chancell::{AnyChanCell, CircId, msg::AnyChanMsg};
 use tor_error::debug_report;
-use tor_memquota::mq_queue;
-use tor_rtcompat::SleepProvider;
+use tor_rtcompat::{CoarseTimeProvider, DynTimeProvider, SleepProvider};
 
 #[cfg_attr(not(target_os = "linux"), allow(unused))]
 use tor_error::error_report;
@@ -39,8 +41,9 @@ use std::pin::Pin;
 use std::sync::Arc;
 
 use crate::channel::{ChannelDetails, CloseInfo, kist::KistParams, padding, params::*, unique_id};
-use crate::client::circuit::{CircuitRxSender, celltypes::CreateResponse};
-use tracing::{debug, trace};
+use crate::circuit::celltypes::CreateResponse;
+use crate::client::circuit::CircuitRxSender;
+use tracing::{debug, instrument, trace};
 
 /// A boxed trait object that can provide `ChanCell`s.
 pub(super) type BoxedChannelStream =
@@ -53,11 +56,22 @@ pub(super) type BoxedChannelStreamOps = Box<dyn StreamOps + Send + Unpin + 'stat
 /// The type of a oneshot channel used to inform reactor users of the result of an operation.
 pub(super) type ReactorResultChannel<T> = oneshot::Sender<Result<T>>;
 
+cfg_if::cfg_if! {
+    if #[cfg(feature = "circ-padding")] {
+        use crate::util::sink_blocker::{SinkBlocker, CountingPolicy};
+        /// Type used by a channel reactor to send cells to the network.
+        pub(super) type ChannelOutputSink = SinkBlocker<BoxedChannelSink, CountingPolicy>;
+    } else {
+        /// Type used by a channel reactor to send cells to the network.
+        pub(super) type ChannelOutputSink = BoxedChannelSink;
+    }
+}
+
 /// A message telling the channel reactor to do something.
 #[cfg_attr(docsrs, doc(cfg(feature = "testing")))]
 #[derive(Debug)]
 #[allow(unreachable_pub)] // Only `pub` with feature `testing`; otherwise, visible in crate
-#[allow(clippy::exhaustive_enums)]
+#[allow(clippy::exhaustive_enums, private_interfaces)]
 pub enum CtrlMsg {
     /// Shut down the reactor.
     Shutdown,
@@ -71,7 +85,12 @@ pub enum CtrlMsg {
         /// Channel to send other messages from this circuit down.
         sender: CircuitRxSender,
         /// Oneshot channel to send the new circuit's identifiers down.
-        tx: ReactorResultChannel<(CircId, crate::circuit::UniqId)>,
+        tx: ReactorResultChannel<(
+            CircId,
+            crate::circuit::UniqId,
+            PaddingController,
+            PaddingEventStream,
+        )>,
     },
     /// Enable/disable/reconfigure channel padding
     ///
@@ -88,6 +107,14 @@ pub enum CtrlMsg {
     /// the sender of these messages is responsible for the optimisation of
     /// ensuring that "no-change" messages are elided.
     KistConfigUpdate(KistParams),
+    /// Change the current padding implementation to the one provided.
+    #[cfg(feature = "circ-padding-manual")]
+    SetChannelPadder {
+        /// The padder to install, or None to remove any existing padder.
+        padder: Option<crate::client::CircuitPadder>,
+        /// A oneshot channel to use in reporting the outcome.
+        sender: oneshot::Sender<Result<()>>,
+    },
 }
 
 /// Object to handle incoming cells and background tasks on a channel.
@@ -95,7 +122,9 @@ pub enum CtrlMsg {
 /// This type is returned when you finish a channel; you need to spawn a
 /// new task that calls `run()` on it.
 #[must_use = "If you don't call run() on a reactor, the channel won't work."]
-pub struct Reactor<S: SleepProvider> {
+pub struct Reactor<S: SleepProvider + CoarseTimeProvider> {
+    /// Underlying runtime we use for generating sleep futures and telling time.
+    pub(super) runtime: S,
     /// A receiver for control messages from `Channel` objects.
     pub(super) control: mpsc::UnboundedReceiver<CtrlMsg>,
     /// A oneshot sender that is used to alert other tasks when this reactor is
@@ -104,7 +133,7 @@ pub struct Reactor<S: SleepProvider> {
     /// A receiver for cells to be sent on this reactor's sink.
     ///
     /// `Channel` objects have a sender that can send cells here.
-    pub(super) cells: mq_queue::Receiver<AnyChanCell, mq_queue::MpscSpec>,
+    pub(super) cells: super::CellRx,
     /// A Stream from which we can read `ChanCell`s.
     ///
     /// This should be backed by a TLS connection if you want it to be secure.
@@ -112,11 +141,16 @@ pub struct Reactor<S: SleepProvider> {
     /// A Sink to which we can write `ChanCell`s.
     ///
     /// This should also be backed by a TLS connection if you want it to be secure.
-    pub(super) output: BoxedChannelSink,
+    pub(super) output: ChannelOutputSink,
     /// A handler for setting stream options on the underlying stream.
     #[cfg_attr(not(target_os = "linux"), allow(unused))]
     pub(super) streamops: BoxedChannelStreamOps,
-    /// Timer tracking when to generate channel padding
+    /// Timer tracking when to generate channel padding.
+    ///
+    /// Note that this is _distinct_ from the experimental maybenot-based padding
+    /// implemented with padding_ctrl and padding_stream.
+    /// This is the existing per-channel padding
+    /// in the tor protocol used to resist netflow attacks.
     pub(super) padding_timer: Pin<Box<padding::Timer<S>>>,
     /// Outgoing cells introduced at the channel reactor
     pub(super) special_outgoing: SpecialOutgoing,
@@ -128,6 +162,20 @@ pub struct Reactor<S: SleepProvider> {
     pub(super) details: Arc<ChannelDetails>,
     /// Context for allocating unique circuit log identifiers.
     pub(super) circ_unique_id_ctx: unique_id::CircUniqIdContext,
+    /// A padding controller to which padding-related events should be reported.
+    ///
+    /// (This is used for experimental maybenot-based padding.)
+    //
+    // TODO: It would be good to use S here instead of DynTimeProvider,
+    // but we still need the latter for the clones of padding_ctrl that we hand out
+    // inside ChannelSender.
+    pub(super) padding_ctrl: PaddingController<DynTimeProvider>,
+    /// An event stream telling us about padding-related events.
+    ///
+    /// (This is used for experimental maybenot-based padding.)
+    pub(super) padding_event_stream: PaddingEventStream<DynTimeProvider>,
+    /// If present, the current rules for blocking the output based on the padding framework.
+    pub(super) padding_blocker: Option<StartBlocking>,
     /// What link protocol is the channel using?
     #[allow(dead_code)] // We don't support protocols where this would matter
     pub(super) link_protocol: u16,
@@ -136,8 +184,10 @@ pub struct Reactor<S: SleepProvider> {
 /// Outgoing cells introduced at the channel reactor
 #[derive(Default, Debug, Clone)]
 pub(super) struct SpecialOutgoing {
-    /// If we must send a `PaddingNegotiate`
-    pub(super) padding_negotiate: Option<PaddingNegotiate>,
+    /// If we must send a `PaddingNegotiate`, this is present.
+    padding_negotiate: Option<PaddingNegotiate>,
+    /// A number of pending PADDING cells that we have to send, once there is space.
+    n_padding: u16,
 }
 
 impl SpecialOutgoing {
@@ -146,32 +196,42 @@ impl SpecialOutgoing {
     /// Called by the reactor before looking for cells from the reactor's clients.
     /// The returned message *must* be sent by the caller, not dropped!
     #[must_use = "SpecialOutgoing::next()'s return value must be actually sent"]
-    pub(super) fn next(&mut self) -> Option<AnyChanCell> {
+    fn next(&mut self) -> Option<AnyChanCell> {
         // If this gets more cases, consider making SpecialOutgoing into a #[repr(C)]
         // enum, so that we can fast-path the usual case of "no special message to send".
         if let Some(p) = self.padding_negotiate.take() {
             return Some(p.into());
         }
+        if self.n_padding > 0 {
+            self.n_padding -= 1;
+            return Some(Padding::new().into());
+        }
         None
     }
+
+    /// Try to queue a padding cell to be sent.
+    fn queue_padding_cell(&mut self) {
+        self.n_padding = self.n_padding.saturating_add(1);
+    }
 }
 
 /// Allows us to just say debug!("{}: Reactor did a thing", &self, ...)
 ///
 /// There is no risk of confusion because no-one would try to print a
 /// Reactor for some other reason.
-impl<S: SleepProvider> fmt::Display for Reactor<S> {
+impl<S: SleepProvider + CoarseTimeProvider> fmt::Display for Reactor<S> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         fmt::Debug::fmt(&self.unique_id, f)
     }
 }
 
-impl<S: SleepProvider> Reactor<S> {
+impl<S: SleepProvider + CoarseTimeProvider> Reactor<S> {
     /// Launch the reactor, and run until the channel closes or we
     /// encounter an error.
     ///
     /// Once this function returns, the channel is dead, and can't be
     /// used again.
+    #[instrument(level = "trace", skip_all)]
     pub async fn run(mut self) -> Result<()> {
         trace!(channel_id = %self, "Running reactor");
         let result: Result<()> = loop {
@@ -197,6 +257,7 @@ impl<S: SleepProvider> Reactor<S> {
     }
 
     /// Helper for run(): handles only one action.
+    #[instrument(level = "trace", skip_all)]
     async fn run_once(&mut self) -> std::result::Result<(), ReactorError> {
         select! {
 
@@ -209,7 +270,7 @@ impl<S: SleepProvider> Reactor<S> {
                     // See reasoning below.
                     // eprintln!("PADDING - SENDING NEOGIATION: {:?}", &l);
                     self.padding_timer.as_mut().note_cell_sent();
-                    return Some(l)
+                    return Some((l, None));
                 }
 
                 select_biased! {
@@ -235,12 +296,28 @@ impl<S: SleepProvider> Reactor<S> {
                     },
                     p = self.padding_timer.as_mut().next() => {
                         // eprintln!("PADDING - SENDING PADDING: {:?}", &p);
-                        Some(p.into())
+
+                        // Note that we treat padding from the padding_timer as a normal cell,
+                        // since it doesn't have a padding machine.
+                        self.padding_ctrl.queued_data(HopNum::from(0));
+
+                        self.padding_timer.as_mut().note_cell_sent();
+                        Some((p.into(), None))
                     },
                 }
             }) => {
-                let (msg, sendable) = ret?;
-                let msg = msg.ok_or(ReactorError::Shutdown)?;
+                self.padding_ctrl.flushed_channel_cell();
+                let (queued, sendable) = ret?;
+                let (msg, cell_padding_info) = queued.ok_or(ReactorError::Shutdown)?;
+                // Tell the relevant circuit padder that this cell is getting flushed.
+                // Note that, technically, it won't go onto the network for a while longer:
+                // it has to go through the TLS buffer, and the kernel TCP buffer.
+                // We've got to live with that.
+                // TODO: conceivably we could defer this even longer, but it would take
+                // some tricky hacking!
+                if let (Some(cell_padding_info), Some(circid)) = (cell_padding_info, msg.circid()) {
+                    self.circs.note_cell_flushed(circid, cell_padding_info);
+                }
                 sendable.send(msg)?;
             }
 
@@ -252,6 +329,11 @@ impl<S: SleepProvider> Reactor<S> {
                 self.handle_control(ctrl).await?;
             }
 
+            ret = self.padding_event_stream.next() => {
+                let event = ret.ok_or_else(|| Error::from(internal!("Padding event stream was exhausted")))?;
+                self.handle_padding_event(event).await?;
+            }
+
             ret = self.input.next() => {
                 let item = ret
                     .ok_or(ReactorError::Shutdown)??;
@@ -264,6 +346,7 @@ impl<S: SleepProvider> Reactor<S> {
     }
 
     /// Handle a CtrlMsg other than Shutdown.
+    #[instrument(level = "trace", skip(self))] // Intentionally omitting skip_all, msg is useful and not sensitive
     async fn handle_control(&mut self, msg: CtrlMsg) -> Result<()> {
         trace!(
             channel_id = %self,
@@ -282,10 +365,22 @@ impl<S: SleepProvider> Reactor<S> {
                 let mut rng = rand::rng();
                 let my_unique_id = self.unique_id;
                 let circ_unique_id = self.circ_unique_id_ctx.next(my_unique_id);
+                // NOTE: This is a very weird place to be calling new_padding, but:
+                //  - we need to do it here or earlier, so we can add it as part of the CircEnt to
+                //    our map.
+                //  - We need to do it at some point where we have a runtime, which implies in a
+                //    reactor.
+                //
+                // TODO circpad: We might want to lazy-allocate this somehow, or try harder to make
+                // it a no-op when we aren't padding on a particular circuit.
+                let (padding_ctrl, padding_stream) = crate::client::circuit::padding::new_padding(
+                    // TODO: avoid using DynTimeProvider at some point, and re-parameterize for efficiency.
+                    DynTimeProvider::new(self.runtime.clone()),
+                );
                 let ret: Result<_> = self
                     .circs
-                    .add_ent(&mut rng, created_sender, sender)
-                    .map(|id| (id, circ_unique_id));
+                    .add_ent(&mut rng, created_sender, sender, padding_ctrl.clone())
+                    .map(|id| (id, circ_unique_id, padding_ctrl, padding_stream));
                 let _ = tx.send(ret); // don't care about other side going away
                 self.update_disused_since();
             }
@@ -322,12 +417,139 @@ impl<S: SleepProvider> Reactor<S> {
                 }
             }
             CtrlMsg::KistConfigUpdate(kist) => self.apply_kist_params(&kist),
+            #[cfg(feature = "circ-padding-manual")]
+            CtrlMsg::SetChannelPadder { padder, sender } => {
+                self.padding_ctrl
+                    .install_padder_padding_at_hop(HopNum::from(0), padder);
+                let _ignore = sender.send(Ok(()));
+            }
         }
         Ok(())
     }
 
+    /// Take the padding action described in `action`.
+    ///
+    /// (With circuit padding disabled, PaddingEvent can't be constructed.)
+    #[cfg(not(feature = "circ-padding"))]
+    async fn handle_padding_event(&mut self, action: PaddingEvent) -> Result<()> {
+        void::unreachable(action.0)
+    }
+
+    /// Take the padding action described in `action`.
+    #[cfg(feature = "circ-padding")]
+    async fn handle_padding_event(&mut self, action: PaddingEvent) -> Result<()> {
+        use PaddingEvent as PE;
+        match action {
+            PE::SendPadding(send_padding) => {
+                self.handle_send_padding(send_padding).await?;
+            }
+            PE::StartBlocking(start_blocking) => {
+                if self.output.is_unlimited() {
+                    self.output.set_blocked();
+                }
+                self.padding_blocker = Some(start_blocking);
+            }
+            PE::StopBlocking => {
+                self.output.set_unlimited();
+            }
+        }
+        Ok(())
+    }
+
+    /// Send the padding described in `padding`.
+    #[cfg(feature = "circ-padding")]
+    async fn handle_send_padding(&mut self, padding: SendPadding) -> Result<()> {
+        // TODO circpad: This is somewhat duplicative of the logic in `Circuit::send_padding` and
+        // `Circuit::padding_disposition`.  It might be good to unify them at some point.
+        // For now (Oct 2025), though, they have slightly different inputs and behaviors.
+
+        use crate::client::circuit::padding::{Bypass::*, Replace::*};
+        // multihop padding belongs in circuit padders, not here.
+        let hop = HopNum::from(0);
+        assert_eq!(padding.hop, hop);
+
+        // If true, there is blocking, but we are allowed to bypass it.
+        let blocking_bypassed = matches!(
+            (&self.padding_blocker, padding.may_bypass_block()),
+            (
+                Some(StartBlocking {
+                    is_bypassable: true
+                }),
+                BypassBlocking
+            )
+        );
+        // If true, there is blocking, and we can't bypass it.
+        let this_padding_blocked = self.padding_blocker.is_some() && !blocking_bypassed;
+
+        if padding.may_replace_with_data() == Replaceable {
+            if self.output_is_full().await? {
+                // When the output buffer is full,
+                // we _always_ treat it as satisfying our replaceable padding.
+                //
+                // TODO circpad: It would be better to check whether
+                // the output has any bytes at all, but futures_codec doesn't seem to give us a
+                // way to check that.  If we manage to do so in the future, we should change the
+                // logic in this function.
+                self.padding_ctrl
+                    .replaceable_padding_already_queued(hop, padding);
+                return Ok(());
+            } else if self.cells.approx_count() > 0 {
+                // We can replace the padding with outbound cells!
+                if this_padding_blocked {
+                    // In the blocked case, we just declare that the pending data _is_ the queued padding.
+                    self.padding_ctrl
+                        .replaceable_padding_already_queued(hop, padding);
+                } else {
+                    // Otherwise we report that queued data _became_ padding,
+                    // and we allow it to pass any blocking that's present.
+                    self.padding_ctrl.queued_data_as_padding(hop, padding);
+                    if blocking_bypassed {
+                        self.output.allow_n_additional_items(1);
+                    }
+                }
+                return Ok(());
+            } else {
+                // There's nothing to replace this with, so fall through.
+            }
+        }
+
+        // There's no replacement, so we queue unconditionally.
+        self.special_outgoing.queue_padding_cell();
+        self.padding_ctrl.queued_padding(hop, padding);
+        if blocking_bypassed {
+            self.output.allow_n_additional_items(1);
+        }
+
+        Ok(())
+    }
+
+    /// Return true if the output stream is full.
+    ///
+    /// We use this in circuit padding to implement replaceable padding.
+    //
+    // TODO circpad: We'd rather check whether there is any data at all queued in self.output,
+    // but futures_codec doesn't give us a way to do that.
+    #[cfg(feature = "circ-padding")]
+    async fn output_is_full(&mut self) -> Result<bool> {
+        use futures::future::poll_fn;
+        use std::task::Poll;
+        // We use poll_fn to get a cx that we can pass to poll_ready_unpin.
+        poll_fn(|cx| {
+            Poll::Ready(match self.output.poll_ready_unpin(cx) {
+                // If if's ready to send, it isn't full.
+                Poll::Ready(Ok(())) => Ok(false),
+                // If it isn't ready to send, it's full.
+                Poll::Pending => Ok(true),
+                // Propagate errors:
+                Poll::Ready(Err(e)) => Err(e),
+            })
+        })
+        .await
+    }
+
     /// Helper: process a cell on a channel.  Most cell types get ignored
     /// or rejected; a few get delivered to circuits.
+    #[instrument(level = "trace", skip_all)]
     async fn handle_cell(&mut self, cell: AnyChanCell) -> Result<()> {
         let (circid, msg) = cell.into_circid_and_msg();
         use AnyChanMsg::*;
@@ -342,6 +564,15 @@ impl<S: SleepProvider> Reactor<S> {
             ),
         }
 
+        // Report the message to the padding controller.
+        match msg {
+            Padding(_) | Vpadding(_) => {
+                // We always accept channel padding, even if we haven't negotiated any.
+                let _always_acceptable = self.padding_ctrl.decrypted_padding(HopNum::from(0));
+            }
+            _ => self.padding_ctrl.decrypted_data(HopNum::from(0)),
+        }
+
         match msg {
             // These are allowed, and need to be handled.
             Relay(_) => self.deliver_relay(circid, msg).await,
@@ -368,7 +599,7 @@ impl<S: SleepProvider> Reactor<S> {
             .ok_or_else(|| Error::ChanProto("Relay cell on nonexistent circuit".into()))?;
 
         match &mut *ent {
-            CircEnt::Open(s) => {
+            CircEnt::Open { cell_sender: s, .. } => {
                 // There's an open circuit; we can give it the RELAY cell.
                 if s.send(msg.try_into()?).await.is_err() {
                     drop(ent);
@@ -377,7 +608,7 @@ impl<S: SleepProvider> Reactor<S> {
                 }
                 Ok(())
             }
-            CircEnt::Opening(_, _) => Err(Error::ChanProto(
+            CircEnt::Opening { .. } => Err(Error::ChanProto(
                 "Relay cell on pending circuit before CREATED* received".into(),
             )),
             CircEnt::DestroySent(hs) => hs.receive_cell(),
@@ -415,9 +646,12 @@ impl<S: SleepProvider> Reactor<S> {
         match entry {
             // If the circuit is waiting for CREATED, tell it that it
             // won't get one.
-            Some(CircEnt::Opening(oneshot, _)) => {
+            Some(CircEnt::Opening {
+                create_response_sender,
+                ..
+            }) => {
                 trace!(channel_id = %self, "Passing destroy to pending circuit {}", circid);
-                oneshot
+                create_response_sender
                     .send(msg.try_into()?)
                     // TODO(nickm) I think that this one actually means the other side
                     // is closed. See arti#269.
@@ -426,9 +660,12 @@ impl<S: SleepProvider> Reactor<S> {
                     })
             }
             // It's an open circuit: tell it that it got a DESTROY cell.
-            Some(CircEnt::Open(mut sink)) => {
+            Some(CircEnt::Open {
+                mut cell_sender, ..
+            }) => {
                 trace!(channel_id = %self, "Passing destroy to open circuit {}", circid);
-                sink.send(msg.try_into()?)
+                cell_sender
+                    .send(msg.try_into()?)
                     .await
                     // TODO(nickm) I think that this one actually means the other side
                     // is closed. See arti#269.
@@ -515,14 +752,15 @@ pub(crate) mod test {
     use super::*;
     use crate::channel::{ChannelType, ClosedUnexpectedly, UniqId};
     use crate::client::circuit::CircParameters;
+    use crate::client::circuit::padding::new_padding;
     use crate::fake_mpsc;
-    use crate::util::fake_mq;
+    use crate::util::{DummyTimeoutEstimator, fake_mq};
     use futures::sink::SinkExt;
     use futures::stream::StreamExt;
-    use futures::task::SpawnExt;
     use tor_cell::chancell::msg;
     use tor_linkspec::OwnedChanTarget;
-    use tor_rtcompat::{NoOpStreamOpsHandle, Runtime};
+    use tor_rtcompat::SpawnExt;
+    use tor_rtcompat::{DynTimeProvider, NoOpStreamOpsHandle, Runtime};
 
     pub(crate) type CodecResult = std::result::Result<AnyChanCell, Error>;
 
@@ -613,7 +851,10 @@ pub(crate) mod test {
             let (chan, mut reactor, mut output, _input) = new_reactor(rt.clone());
             assert!(chan.duration_unused().is_some()); // unused yet
 
-            let (ret, reac) = futures::join!(chan.new_tunnel(), reactor.run_once());
+            let (ret, reac) = futures::join!(
+                chan.new_tunnel(Arc::new(DummyTimeoutEstimator)),
+                reactor.run_once()
+            );
             let (pending, circr) = ret.unwrap();
             rt.spawn(async {
                 let _ignore = circr.run().await;
@@ -624,7 +865,7 @@ pub(crate) mod test {
             let id = pending.peek_circid();
 
             let ent = reactor.circs.get_mut(id);
-            assert!(matches!(*ent.unwrap(), CircEnt::Opening(_, _)));
+            assert!(matches!(*ent.unwrap(), CircEnt::Opening { .. }));
             assert!(chan.duration_unused().is_none()); // in use
 
             // Now drop the circuit; this should tell the reactor to remove
@@ -651,7 +892,10 @@ pub(crate) mod test {
         tor_rtcompat::test_with_all_runtimes!(|rt| async move {
             let (chan, mut reactor, mut output, mut input) = new_reactor(rt.clone());
 
-            let (ret, reac) = futures::join!(chan.new_tunnel(), reactor.run_once());
+            let (ret, reac) = futures::join!(
+                chan.new_tunnel(Arc::new(DummyTimeoutEstimator)),
+                reactor.run_once()
+            );
             let (pending, circr) = ret.unwrap();
             rt.spawn(async {
                 let _ignore = circr.run().await;
@@ -664,7 +908,7 @@ pub(crate) mod test {
             let id = pending.peek_circid();
 
             let ent = reactor.circs.get_mut(id);
-            assert!(matches!(*ent.unwrap(), CircEnt::Opening(_, _)));
+            assert!(matches!(*ent.unwrap(), CircEnt::Opening { .. }));
 
             #[allow(clippy::clone_on_copy)]
             let rtc = rt.clone();
@@ -733,22 +977,33 @@ pub(crate) mod test {
     #[test]
     fn deliver_relay() {
         tor_rtcompat::test_with_all_runtimes!(|rt| async move {
-            use crate::client::circuit::celltypes::ClientCircChanMsg;
+            use crate::circuit::celltypes::ClientCircChanMsg;
             use oneshot_fused_workaround as oneshot;
 
-            let (_chan, mut reactor, _output, mut input) = new_reactor(rt);
+            let (_chan, mut reactor, _output, mut input) = new_reactor(rt.clone());
+
+            let (padding_ctrl, _padding_stream) = new_padding(DynTimeProvider::new(rt));
 
             let (_circ_stream_7, mut circ_stream_13) = {
                 let (snd1, _rcv1) = oneshot::channel();
                 let (snd2, rcv2) = fake_mpsc(64);
-                reactor
-                    .circs
-                    .put_unchecked(CircId::new(7).unwrap(), CircEnt::Opening(snd1, snd2));
+                reactor.circs.put_unchecked(
+                    CircId::new(7).unwrap(),
+                    CircEnt::Opening {
+                        create_response_sender: snd1,
+                        cell_sender: snd2,
+                        padding_ctrl: padding_ctrl.clone(),
+                    },
+                );
 
                 let (snd3, rcv3) = fake_mpsc(64);
-                reactor
-                    .circs
-                    .put_unchecked(CircId::new(13).unwrap(), CircEnt::Open(snd3));
+                reactor.circs.put_unchecked(
+                    CircId::new(13).unwrap(),
+                    CircEnt::Open {
+                        cell_sender: snd3,
+                        padding_ctrl,
+                    },
+                );
 
                 reactor.circs.put_unchecked(
                     CircId::new(23).unwrap(),
@@ -818,22 +1073,33 @@ pub(crate) mod test {
     #[test]
     fn deliver_destroy() {
         tor_rtcompat::test_with_all_runtimes!(|rt| async move {
-            use crate::client::circuit::celltypes::*;
+            use crate::circuit::celltypes::*;
             use oneshot_fused_workaround as oneshot;
 
-            let (_chan, mut reactor, _output, mut input) = new_reactor(rt);
+            let (_chan, mut reactor, _output, mut input) = new_reactor(rt.clone());
+
+            let (padding_ctrl, _padding_stream) = new_padding(DynTimeProvider::new(rt));
 
             let (circ_oneshot_7, mut circ_stream_13) = {
                 let (snd1, rcv1) = oneshot::channel();
                 let (snd2, _rcv2) = fake_mpsc(64);
-                reactor
-                    .circs
-                    .put_unchecked(CircId::new(7).unwrap(), CircEnt::Opening(snd1, snd2));
+                reactor.circs.put_unchecked(
+                    CircId::new(7).unwrap(),
+                    CircEnt::Opening {
+                        create_response_sender: snd1,
+                        cell_sender: snd2,
+                        padding_ctrl: padding_ctrl.clone(),
+                    },
+                );
 
                 let (snd3, rcv3) = fake_mpsc(64);
-                reactor
-                    .circs
-                    .put_unchecked(CircId::new(13).unwrap(), CircEnt::Open(snd3));
+                reactor.circs.put_unchecked(
+                    CircId::new(13).unwrap(),
+                    CircEnt::Open {
+                        cell_sender: snd3,
+                        padding_ctrl: padding_ctrl.clone(),
+                    },
+                );
 
                 reactor.circs.put_unchecked(
                     CircId::new(23).unwrap(),
