@@ -4,18 +4,18 @@ pub(crate) mod circhop;
 pub(super) mod create;
 pub(super) mod extender;
 
-use crate::channel::{Channel, ChannelSender};
+use crate::channel::Channel;
 use crate::circuit::UniqId;
-use crate::client::circuit::celltypes::{ClientCircChanMsg, CreateResponse};
-#[cfg(feature = "counter-galois-onion")]
-use crate::client::circuit::handshake::RelayCryptLayerProtocol;
+use crate::circuit::cell_sender::CircuitCellSender;
+use crate::circuit::celltypes::{ClientCircChanMsg, CreateResponse};
+use crate::circuit::circhop::HopSettings;
 use crate::client::circuit::handshake::{BoxedClientLayer, HandshakeRole};
-use crate::client::circuit::{CircuitRxReceiver, MutableState, StreamMpscReceiver};
-use crate::client::circuit::{HopSettings, path};
+use crate::client::circuit::padding::{
+    self, PaddingController, PaddingEventStream, QueuedCellPaddingInfo,
+};
+use crate::client::circuit::{CircuitRxReceiver, MutableState};
+use crate::client::circuit::{TimeoutEstimator, path};
 use crate::client::reactor::MetaCellDisposition;
-use crate::client::stream::queue::{StreamQueueSender, stream_queue};
-use crate::client::stream::{AnyCmdChecker, DrainRateRequest, StreamRateLimit, StreamStatus};
-use crate::client::streammap;
 use crate::congestion::CongestionSignals;
 use crate::congestion::sendme;
 use crate::crypto::binding::CircuitBinding;
@@ -28,18 +28,21 @@ use crate::crypto::handshake::ntor::{NtorClient, NtorPublicKey};
 use crate::crypto::handshake::ntor_v3::{NtorV3Client, NtorV3PublicKey};
 use crate::crypto::handshake::{ClientHandshake, KeyGenerator};
 use crate::memquota::{CircuitAccount, SpecificAccount as _, StreamAccount};
+use crate::stream::StreamMpscReceiver;
+use crate::stream::cmdcheck::{AnyCmdChecker, StreamStatus};
+use crate::stream::flow_ctrl::state::StreamRateLimit;
+use crate::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
+use crate::stream::queue::{StreamQueueSender, stream_queue};
+use crate::streammap;
 use crate::tunnel::TunnelScopedCircId;
-use crate::util::SinkExt as _;
 use crate::util::err::ReactorError;
 use crate::util::notify::NotifySender;
-use crate::util::sometimes_unbounded_sink::SometimesUnboundedSink;
 use crate::{ClockSkew, Error, Result};
 
 use tor_async_utils::{SinkTrySend as _, SinkTrySendError as _};
 use tor_cell::chancell::msg::{AnyChanMsg, HandshakeType, Relay};
 use tor_cell::chancell::{AnyChanCell, ChanCmd, CircId};
 use tor_cell::chancell::{BoxedCellBody, ChanMsg};
-use tor_cell::relaycell::extend::{CcRequest, CircRequestExt};
 use tor_cell::relaycell::msg::{AnyRelayMsg, End, Sendme, SendmeTag, Truncated};
 use tor_cell::relaycell::{
     AnyRelayMsgOuter, RelayCellDecoderResult, RelayCellFormat, RelayCmd, StreamId, UnparsedRelayMsg,
@@ -49,45 +52,45 @@ use tor_linkspec::RelayIds;
 use tor_llcrypto::pk;
 use tor_memquota::mq_queue::{ChannelSpec as _, MpscSpec};
 
-use futures::{SinkExt as _, Stream};
+use futures::SinkExt as _;
 use oneshot_fused_workaround as oneshot;
 use postage::watch;
 use safelog::sensitive as sv;
-use tor_rtcompat::DynTimeProvider;
-use tracing::{debug, trace, warn};
+use tor_rtcompat::{DynTimeProvider, SleepProvider as _};
+use tracing::{debug, instrument, trace, warn};
 
 use super::{
     CellHandlers, CircuitHandshake, CloseStreamBehavior, ReactorResultChannel, SendRelayCell,
 };
 
+use crate::conflux::msghandler::ConfluxStatus;
+
 use std::borrow::Borrow;
 use std::pin::Pin;
 use std::result::Result as StdResult;
 use std::sync::Arc;
-use std::task::Poll;
-use std::time::{Duration, SystemTime};
+use std::time::{Duration, Instant, SystemTime};
 
 use create::{Create2Wrap, CreateFastWrap, CreateHandshakeWrap};
 use extender::HandshakeAuxDataHandler;
 
 #[cfg(feature = "hs-service")]
 use {
-    crate::client::stream::{DataCmdChecker, IncomingStreamRequest},
+    crate::circuit::CircSyncView,
+    crate::client::stream::{InboundDataCmdChecker, IncomingStreamRequest},
     tor_cell::relaycell::msg::Begin,
 };
 
 #[cfg(feature = "conflux")]
 use {
-    super::conflux::ConfluxMsgHandler,
-    super::conflux::{ConfluxAction, OooRelayMsg},
-    crate::client::reactor::RemoveLegReason,
+    crate::conflux::msghandler::{ConfluxAction, ConfluxCmd, ConfluxMsgHandler, OooRelayMsg},
     crate::tunnel::TunnelId,
 };
 
 pub(super) use circhop::{CircHop, CircHopList};
 
 /// Initial value for outbound flow-control window on streams.
-pub(super) const SEND_WINDOW_INIT: u16 = 500;
+pub(crate) const SEND_WINDOW_INIT: u16 = 500;
 /// Initial value for inbound flow-control window on streams.
 pub(crate) const RECV_WINDOW_INIT: u16 = 500;
 /// Size of the buffer used between the reactor and a `StreamReader`.
@@ -110,7 +113,7 @@ pub(crate) struct Circuit {
     ///
     /// NOTE: Control messages could potentially add unboundedly to this, although that's
     ///       not likely to happen (and isn't triggereable from the network, either).
-    pub(super) chan_sender: SometimesUnboundedSink<AnyChanCell, ChannelSender>,
+    pub(super) chan_sender: CircuitCellSender,
     /// Input stream, on which we receive ChanMsg objects from this circuit's
     /// channel.
     ///
@@ -123,7 +126,7 @@ pub(crate) struct Circuit {
     /// The cryptographic state for this circuit for outbound cells.
     crypto_out: OutboundClientCrypt,
     /// List of hops state objects used by the reactor
-    hops: CircHopList,
+    pub(super) hops: CircHopList,
     /// Mutable information about this circuit,
     /// shared with the reactor's `ConfluxSet`.
     mutable: Arc<MutableState>,
@@ -137,6 +140,23 @@ pub(crate) struct Circuit {
     /// using [`Reactor::handle_link_circuits`](super::Reactor::handle_link_circuits).
     #[cfg(feature = "conflux")]
     conflux_handler: Option<ConfluxMsgHandler>,
+    /// A padding controller to which padding-related events should be reported.
+    padding_ctrl: PaddingController,
+    /// An event stream telling us about padding-related events.
+    //
+    // TODO: it would be nice to have all of these streams wrapped in a single
+    // SelectAll, but we can't really do that, since we need the ability to move them
+    // from one conflux set to another, and a SelectAll doesn't let you actually
+    // remove one of its constituent streams.  This issue might get solved along
+    // with the the rest of the next reactor refactoring.
+    pub(super) padding_event_stream: PaddingEventStream,
+    /// Current rules for blocking traffic, according to the padding controller.
+    #[cfg(feature = "circ-padding")]
+    padding_block: Option<padding::StartBlocking>,
+    /// The circuit timeout estimator.
+    ///
+    /// Used for computing half-stream expiration.
+    timeouts: Arc<dyn TimeoutEstimator>,
     /// Memory quota account
     #[allow(dead_code)] // Partly here to keep it alive as long as the circuit
     memquota: CircuitAccount,
@@ -149,7 +169,7 @@ pub(crate) struct Circuit {
 /// of the circuit the `CircuitCmd` came from.
 ///
 /// This type gets mapped to a `RunOnceCmdInner` in the circuit reactor.
-#[derive(Debug)]
+#[derive(Debug, derive_more::From)]
 pub(super) enum CircuitCmd {
     /// Send a RELAY cell on the circuit leg this command originates from.
     Send(SendRelayCell),
@@ -171,20 +191,9 @@ pub(super) enum CircuitCmd {
         /// The reason for closing the stream.
         reason: streammap::TerminateReason,
     },
-    /// Remove this circuit from the conflux set.
-    ///
-    /// Returned by `ConfluxMsgHandler::handle_conflux_msg` for invalid messages
-    /// (originating from wrong hop), and for messages that are rejected
-    /// by its inner `AbstractMsgHandler`.
+    /// Perform an action resulting from handling a conflux cell.
     #[cfg(feature = "conflux")]
-    ConfluxRemove(RemoveLegReason),
-    /// This circuit has completed the conflux handshake,
-    /// and wants to send the specified cell.
-    ///
-    /// Returned by an `AbstractMsgHandler` to signal to the reactor that
-    /// the conflux handshake is complete.
-    #[cfg(feature = "conflux")]
-    ConfluxHandshakeComplete(SendRelayCell),
+    Conflux(ConfluxCmd),
     /// Perform a clean shutdown on this circuit.
     CleanShutdown,
     /// Enqueue an out-of-order cell in the reactor.
@@ -222,6 +231,7 @@ pub(super) use unsupported_client_cell;
 
 impl Circuit {
     /// Create a new non-multipath circuit.
+    #[allow(clippy::too_many_arguments)]
     pub(super) fn new(
         runtime: DynTimeProvider,
         channel: Arc<Channel>,
@@ -230,8 +240,11 @@ impl Circuit {
         input: CircuitRxReceiver,
         memquota: CircuitAccount,
         mutable: Arc<MutableState>,
+        padding_ctrl: PaddingController,
+        padding_event_stream: PaddingEventStream,
+        timeouts: Arc<dyn TimeoutEstimator>,
     ) -> Self {
-        let chan_sender = SometimesUnboundedSink::new(channel.sender());
+        let chan_sender = CircuitCellSender::from_channel_sender(channel.sender());
 
         let crypto_out = OutboundClientCrypt::new();
         Circuit {
@@ -247,6 +260,11 @@ impl Circuit {
             mutable,
             #[cfg(feature = "conflux")]
             conflux_handler: None,
+            padding_ctrl,
+            padding_event_stream,
+            #[cfg(feature = "circ-padding")]
+            padding_block: None,
+            timeouts,
             memquota,
         }
     }
@@ -296,7 +314,7 @@ impl Circuit {
         }
 
         let cell = SendRelayCell {
-            hop,
+            hop: Some(hop),
             early: false,
             cell,
         };
@@ -311,7 +329,7 @@ impl Circuit {
 
     /// Get the wallclock time when the handshake on this circuit is supposed to time out.
     ///
-    /// Returns `None` if this handler hasn't started the handshake yet.
+    /// Returns `None` if the handshake is not currently in progress.
     pub(super) fn conflux_hs_timeout(&self) -> Option<SystemTime> {
         cfg_if::cfg_if! {
             if #[cfg(feature = "conflux")] {
@@ -349,7 +367,7 @@ impl Circuit {
 
         let settings = HopSettings::from_params_and_caps(
             // This is for testing only, so we'll assume full negotiation took place.
-            crate::client::circuit::HopNegotiationType::Full,
+            crate::circuit::circhop::HopNegotiationType::Full,
             params,
             &[named::FLOWCTRL_CC].into_iter().collect::<Protocols>(),
         )
@@ -399,7 +417,22 @@ impl Circuit {
     /// NOTE: the reactor should not call this function directly, only via
     /// [`ConfluxSet::send_relay_cell_on_leg`](super::conflux::ConfluxSet::send_relay_cell_on_leg),
     /// which will reroute the message, if necessary to the primary leg.
+    #[instrument(level = "trace", skip_all)]
     pub(super) async fn send_relay_cell(&mut self, msg: SendRelayCell) -> Result<()> {
+        self.send_relay_cell_inner(msg, None).await
+    }
+
+    /// As [`send_relay_cell`](Self::send_relay_cell), but takes an optional
+    /// [`QueuedCellPaddingInfo`] in `padding_info`.
+    ///
+    /// If `padding_info` is None, `msg` must be non-padding: we report it as such to the
+    /// padding controller.
+    #[instrument(level = "trace", skip_all)]
+    async fn send_relay_cell_inner(
+        &mut self,
+        msg: SendRelayCell,
+        padding_info: Option<QueuedCellPaddingInfo>,
+    ) -> Result<()> {
         let SendRelayCell {
             hop,
             early,
@@ -419,18 +452,19 @@ impl Circuit {
         let runtime = self.runtime.clone();
         let c_t_w = sendme::cmd_counts_towards_windows(msg.cmd());
         let stream_id = msg.stream_id();
+        let hop = hop.expect("missing hop in client SendRelayCell?!");
         let circhop = self.hops.get_mut(hop).ok_or(Error::NoSuchHop)?;
 
         // We might be out of capacity entirely; see if we are about to hit a limit.
         //
         // TODO: If we ever add a notion of _recoverable_ errors below, we'll
-        // need a way to restore this limit, and similarly for take_capacity_to_send().
+        // need a way to restore this limit, and similarly for about_to_send().
         circhop.decrement_outbound_cell_limit()?;
 
         // We need to apply stream-level flow control *before* encoding the message.
         if c_t_w {
             if let Some(stream_id) = stream_id {
-                circhop.take_capacity_to_send(stream_id, msg.msg())?;
+                circhop.about_to_send(stream_id, msg.msg())?;
             }
         }
 
@@ -443,7 +477,7 @@ impl Circuit {
         //            the whole circuit (e.g. by returning an error).
         let (msg, tag) = Self::encode_relay_cell(
             &mut self.crypto_out,
-            circhop.relay_format(),
+            circhop.relay_cell_format(),
             hop,
             early,
             msg,
@@ -451,10 +485,13 @@ impl Circuit {
         // The cell counted for congestion control, inform our algorithm of such and pass down the
         // tag for authenticated SENDMEs.
         if c_t_w {
-            circhop.ccontrol_mut().note_data_sent(&runtime, &tag)?;
+            circhop.ccontrol().note_data_sent(&runtime, &tag)?;
         }
 
-        self.send_msg(msg).await?;
+        // Remember that we've enqueued this cell.
+        let padding_info = padding_info.or_else(|| self.padding_ctrl.queued_data(hop));
+
+        self.send_msg(msg, padding_info).await?;
 
         #[cfg(feature = "conflux")]
         if let Some(conflux) = self.conflux_handler.as_mut() {
@@ -537,6 +574,12 @@ impl Circuit {
     ) -> Result<Vec<CircuitCmd>> {
         let (hopnum, tag, decode_res) = self.decode_relay_cell(cell)?;
 
+        if decode_res.is_padding() {
+            self.padding_ctrl.decrypted_padding(hopnum)?;
+        } else {
+            self.padding_ctrl.decrypted_data(hopnum);
+        }
+
         // Check whether we are allowed to receive more data for this circuit hop.
         self.hop_mut(hopnum)
             .ok_or_else(|| internal!("nonexistent hop {:?}", hopnum))?
@@ -549,7 +592,7 @@ impl Circuit {
         let send_circ_sendme = if c_t_w {
             self.hop_mut(hopnum)
                 .ok_or_else(|| Error::CircProto("Sendme from nonexistent hop".into()))?
-                .ccontrol_mut()
+                .ccontrol()
                 .note_data_received()?
         } else {
             false
@@ -565,7 +608,7 @@ impl Circuit {
             let sendme = Sendme::from(tag);
             let cell = AnyRelayMsgOuter::new(None, sendme.into());
             circ_cmds.push(CircuitCmd::Send(SendRelayCell {
-                hop: hopnum,
+                hop: Some(hopnum),
                 early: false,
                 cell,
             }));
@@ -578,7 +621,7 @@ impl Circuit {
                         hopnum
                     ))
                 })?
-                .ccontrol_mut()
+                .ccontrol()
                 .note_sendme_sent()?;
         }
 
@@ -676,17 +719,26 @@ impl Circuit {
         streamid: StreamId,
         msg: UnparsedRelayMsg,
     ) -> Result<Option<CircuitCmd>> {
+        let now = self.runtime.now();
+
         #[cfg(feature = "conflux")]
         if let Some(conflux) = self.conflux_handler.as_mut() {
             conflux.inc_last_seq_delivered(&msg);
         }
 
-        // Returns the original message if it's an an incoming stream request
+        let path = self.mutable.path();
+
+        let nonexistent_hop_err = || Error::CircProto("Cell from nonexistent hop!".into());
+        let hop = self.hop_mut(hopnum).ok_or_else(nonexistent_hop_err)?;
+
+        let hop_detail = path
+            .iter()
+            .nth(usize::from(hopnum))
+            .ok_or_else(nonexistent_hop_err)?;
+
+        // Returns the original message if it's an incoming stream request
         // that we need to handle.
-        let hop = self
-            .hop_mut(hopnum)
-            .ok_or_else(|| Error::CircProto("Cell from nonexistent hop!".into()))?;
-        let res = hop.handle_msg(cell_counts_toward_windows, streamid, msg)?;
+        let res = hop.handle_msg(hop_detail, cell_counts_toward_windows, streamid, msg, now)?;
 
         // If it was an incoming stream request, we don't need to worry about
         // sending an XOFF as there's no stream data within this message.
@@ -704,7 +756,7 @@ impl Circuit {
         if let Some(cell) = hop.maybe_send_xoff(streamid)? {
             let cell = AnyRelayMsgOuter::new(Some(streamid), cell.into());
             let cell = SendRelayCell {
-                hop: hopnum,
+                hop: Some(hopnum),
                 early: false,
                 cell,
             };
@@ -726,7 +778,7 @@ impl Circuit {
         &mut self,
         hop: HopNum,
         msg: UnparsedRelayMsg,
-    ) -> Result<Option<CircuitCmd>> {
+    ) -> Result<Option<ConfluxCmd>> {
         let Some(conflux_handler) = self.conflux_handler.as_mut() else {
             // If conflux is not enabled, tear down the circuit
             // (see 4.2.1. Cell Injection Side Channel Mitigations in prop329)
@@ -849,7 +901,7 @@ impl Circuit {
             //
             // This means it's very important not to call this function while any of the hop's
             // stream map mutex is held.
-            let view = ClientCircSyncView::new(&self.hops);
+            let view = CircSyncView::new_client(ClientCircSyncView::new(&self.hops));
 
             match handler.filter.as_mut().disposition(&ctx, &view)? {
                 Accept => {}
@@ -857,7 +909,7 @@ impl Circuit {
                 RejectRequest(end) => {
                     let end_msg = AnyRelayMsgOuter::new(Some(stream_id), end.into());
                     let cell = SendRelayCell {
-                        hop: hop_num,
+                        hop: Some(hop_num),
                         early: false,
                         cell: end_msg,
                     };
@@ -869,7 +921,7 @@ impl Circuit {
         // TODO: Sadly, we need to look up `&mut hop` yet again,
         // since we needed to pass `&self.hops` by reference to our filter above. :(
         let hop = self.hops.get_mut(hop_num).ok_or(Error::CircuitClosed)?;
-        let relay_cell_format = hop.relay_format();
+        let relay_cell_format = hop.relay_cell_format();
 
         let memquota = StreamAccount::new(&self.memquota)?;
 
@@ -877,11 +929,11 @@ impl Circuit {
             #[cfg(not(feature = "flowctl-cc"))]
             STREAM_READER_BUFFER,
             &memquota,
-            self.chan_sender.as_inner().time_provider(),
+            self.chan_sender.time_provider(),
         )?;
 
         let (msg_tx, msg_rx) = MpscSpec::new(CIRCUIT_BUFFER_SIZE).new_mq(
-            self.chan_sender.as_inner().time_provider().clone(),
+            self.chan_sender.time_provider().clone(),
             memquota.as_raw_account(),
         )?;
 
@@ -893,7 +945,7 @@ impl Circuit {
         let mut drain_rate_request_tx = NotifySender::new_typed();
         let drain_rate_request_rx = drain_rate_request_tx.subscribe();
 
-        let cmd_checker = DataCmdChecker::new_connected();
+        let cmd_checker = InboundDataCmdChecker::new_connected();
         hop.add_ent_with_id(
             sender,
             msg_rx,
@@ -928,7 +980,7 @@ impl Circuit {
                 );
 
                 let cell = SendRelayCell {
-                    hop: hop_num,
+                    hop: Some(hop_num),
                     early: false,
                     cell: end_msg,
                 };
@@ -1037,7 +1089,7 @@ impl Circuit {
             create = %create_cell.cmd(),
             "Extending to hop 1",
         );
-        self.send_msg(create_cell).await?;
+        self.send_msg(create_cell, None).await?;
 
         let reply = recvcreated
             .await
@@ -1127,7 +1179,7 @@ impl Circuit {
         self.channel.check_match(&target)?;
 
         // Set the client extensions.
-        let client_extensions = circ_extensions_from_settings(&settings)?;
+        let client_extensions = settings.circuit_request_extensions()?;
         let wrap = Create2Wrap {
             handshake_type: HandshakeType::NTOR_V3,
         };
@@ -1235,6 +1287,17 @@ impl Circuit {
             return Ok(Some(CircuitCmd::CleanShutdown));
         }
 
+        if msg.cmd() == RelayCmd::DROP {
+            cfg_if::cfg_if! {
+                if #[cfg(feature = "circ-padding")] {
+                    return Ok(None);
+                } else {
+                    use crate::util::err::ExcessPadding;
+                    return Err(Error::ExcessPadding(ExcessPadding::NoPaddingNegotiated, hopnum));
+                }
+            }
+        }
+
         trace!(circ_id = %self.unique_id, cell = ?msg, "Received meta-cell");
 
         #[cfg(feature = "conflux")]
@@ -1245,7 +1308,8 @@ impl Circuit {
                 | RelayCmd::CONFLUX_LINKED_ACK
                 | RelayCmd::CONFLUX_SWITCH
         ) {
-            return self.handle_conflux_msg(hopnum, msg);
+            let cmd = self.handle_conflux_msg(hopnum, msg)?;
+            return Ok(cmd.map(CircuitCmd::from));
         }
 
         if self.is_conflux_pending() {
@@ -1301,6 +1365,7 @@ impl Circuit {
     }
 
     /// Handle a RELAY_SENDME cell on this circuit with stream ID 0.
+    #[instrument(level = "trace", skip_all)]
     pub(super) fn handle_sendme(
         &mut self,
         hopnum: HopNum,
@@ -1321,7 +1386,7 @@ impl Circuit {
                 // but we don't support those any longer.
                  Error::CircProto("missing tag on circuit sendme".into()))?;
         // Update the CC object that we received a SENDME along with possible congestion signals.
-        hop.ccontrol_mut()
+        hop.ccontrol()
             .note_sendme_received(&runtime, tag, signals)?;
         Ok(None)
     }
@@ -1331,49 +1396,32 @@ impl Circuit {
     /// If the channel is ready to accept messages, it will be sent immediately. If not, the message
     /// will be enqueued for sending at a later iteration of the reactor loop.
     ///
+    /// `info` is the status returned from the padding controller when we told it we were queueing
+    /// this data.  It should be provided whenever possible.
+    ///
     /// # Note
     ///
     /// Making use of the enqueuing capabilities of this function is discouraged! You should first
     /// check whether the channel is ready to receive messages (`self.channel.poll_ready`), and
     /// ideally use this to implement backpressure (such that you do not read from other sources
     /// that would send here while you know you're unable to forward the messages on).
-    async fn send_msg(&mut self, msg: AnyChanMsg) -> Result<()> {
+    #[instrument(level = "trace", skip_all)]
+    async fn send_msg(
+        &mut self,
+        msg: AnyChanMsg,
+        info: Option<QueuedCellPaddingInfo>,
+    ) -> Result<()> {
         let cell = AnyChanCell::new(Some(self.channel_id), msg);
         // Note: this future is always `Ready`, so await won't block.
-        Pin::new(&mut self.chan_sender).send_unbounded(cell).await?;
+        Pin::new(&mut self.chan_sender)
+            .send_unbounded((cell, info))
+            .await?;
         Ok(())
     }
 
-    /// Returns a [`Stream`] of [`CircuitCmd`] to poll from the main loop.
-    ///
-    /// The iterator contains at most one [`CircuitCmd`] for each hop,
-    /// (excluding the `exclude` hop, if specified),
-    /// representing the instructions for handling the ready-item, if any,
-    /// of its highest priority stream.
-    ///
-    /// IMPORTANT: this stream locks the stream map mutexes of each `CircHop`!
-    /// To avoid contention, never create more than one [`Circuit::ready_streams_iterator`]
-    /// stream at a time!
-    ///
-    /// This is cancellation-safe.
-    pub(super) fn ready_streams_iterator(
-        &self,
-        exclude: Option<HopNum>,
-    ) -> impl Stream<Item = Result<CircuitCmd>> + use<> {
-        self.hops.ready_streams_iterator(exclude)
-    }
-
-    /// Return the congestion signals for this reactor. This is used by congestion control module.
-    ///
-    /// Note: This is only async because we need a Context to check the sink for readiness.
-    pub(super) async fn congestion_signals(&mut self) -> CongestionSignals {
-        futures::future::poll_fn(|cx| -> Poll<CongestionSignals> {
-            Poll::Ready(CongestionSignals::new(
-                self.chan_sender.poll_ready_unpin_bool(cx).unwrap_or(false),
-                self.chan_sender.n_queued(),
-            ))
-        })
-        .await
+    /// Remove all halfstreams that are expired at `now`.
+    pub(super) fn remove_expired_halfstreams(&mut self, now: Instant) {
+        self.hops.remove_expired_halfstreams(now);
     }
 
     /// Return a reference to the hop corresponding to `hopnum`, if there is one.
@@ -1417,15 +1465,17 @@ impl Circuit {
     }
 
     /// Close the specified stream
+    #[instrument(level = "trace", skip_all)]
     pub(super) async fn close_stream(
         &mut self,
         hop_num: HopNum,
         sid: StreamId,
         behav: CloseStreamBehavior,
         reason: streammap::TerminateReason,
+        expiry: Instant,
     ) -> Result<()> {
         if let Some(hop) = self.hop_mut(hop_num) {
-            let res = hop.close_stream(sid, behav, reason)?;
+            let res = hop.close_stream(sid, behav, reason, expiry)?;
             if let Some(cell) = res {
                 self.send_relay_cell(cell).await?;
             }
@@ -1437,7 +1487,7 @@ impl Circuit {
     ///
     /// Important: this function locks the stream map of its each of the [`CircHop`]s
     /// in this circuit, so it must **not** be called from any function where the
-    /// stream map lock is held (such as [`ready_streams_iterator`](Self::ready_streams_iterator).
+    /// stream map lock is held.
     pub(super) fn has_streams(&self) -> bool {
         self.hops.has_streams()
     }
@@ -1523,17 +1573,168 @@ impl Circuit {
             .as_ref()
             .map(|handler| handler.init_rtt())?
     }
+
+    /// Start or stop padding at the given hop.
+    ///
+    /// Replaces any previous padder at that hop.
+    ///
+    /// Return an error if that hop doesn't exist.
+    #[cfg(feature = "circ-padding-manual")]
+    pub(super) fn set_padding_at_hop(
+        &self,
+        hop: HopNum,
+        padder: Option<padding::CircuitPadder>,
+    ) -> Result<()> {
+        if self.hop(hop).is_none() {
+            return Err(Error::NoSuchHop);
+        }
+        self.padding_ctrl.install_padder_padding_at_hop(hop, padder);
+        Ok(())
+    }
+
+    /// Determine how exactly to handle a request to handle padding.
+    ///
+    /// This is fairly complicated; see the maybenot documentation for more information.
+    ///
+    /// ## Limitations
+    ///
+    /// In our current padding implementation, a circuit is either blocked or not blocked:
+    /// we do not keep track of which hop is actually doing the blocking.
+    #[cfg(feature = "circ-padding")]
+    fn padding_disposition(&self, send_padding: &padding::SendPadding) -> CircPaddingDisposition {
+        use CircPaddingDisposition::*;
+        use padding::Bypass::*;
+        use padding::Replace::*;
+
+        // If true, and we are trying to send Replaceable padding,
+        // we should let any data in the queue count as the queued padding instead,
+        // if it is queued for our target hop (or any subsequent hop).
+        //
+        // TODO circpad: In addition to letting currently-queued data count as padding,
+        // maybenot also permits us to send currently pending data from our streams
+        // (or from our next hop, if we're a relay).  We don't have that implemented yet.
+        //
+        // TODO circpad: This will usually be false, since we try not to queue data
+        // when there isn't space to write it.  If we someday add internal per-circuit
+        // Buffers to chan_sender, this test is more likely to trigger.
+        let have_queued_cell_for_hop = self
+            .chan_sender
+            .have_queued_cell_for_hop_or_later(send_padding.hop);
+
+        match &self.padding_block {
+            Some(blocking) if blocking.is_bypassable => {
+                match (
+                    send_padding.may_replace_with_data(),
+                    send_padding.may_bypass_block(),
+                ) {
+                    (NotReplaceable, DoNotBypass) => QueuePaddingNormally,
+                    (NotReplaceable, BypassBlocking) => QueuePaddingAndBypass,
+                    (Replaceable, DoNotBypass) => {
+                        if have_queued_cell_for_hop {
+                            TreatQueuedCellAsPadding
+                        } else {
+                            QueuePaddingNormally
+                        }
+                    }
+                    (Replaceable, BypassBlocking) => {
+                        if have_queued_cell_for_hop {
+                            TreatQueuedCellAsPadding
+                        } else {
+                            QueuePaddingAndBypass
+                        }
+                    }
+                }
+            }
+            Some(_) | None => match send_padding.may_replace_with_data() {
+                Replaceable => {
+                    if have_queued_cell_for_hop {
+                        TreatQueuedCellAsPadding
+                    } else {
+                        QueuePaddingNormally
+                    }
+                }
+                NotReplaceable => QueuePaddingNormally,
+            },
+        }
+    }
+
+    /// Handle a request from our padding subsystem to send a padding packet.
+    #[cfg(feature = "circ-padding")]
+    pub(super) async fn send_padding(&mut self, send_padding: padding::SendPadding) -> Result<()> {
+        use CircPaddingDisposition::*;
+
+        let target_hop = send_padding.hop;
+
+        match self.padding_disposition(&send_padding) {
+            QueuePaddingNormally => {
+                let queue_info = self.padding_ctrl.queued_padding(target_hop, send_padding);
+                self.queue_padding_cell_for_hop(target_hop, queue_info)
+                    .await?;
+            }
+            QueuePaddingAndBypass => {
+                let queue_info = self.padding_ctrl.queued_padding(target_hop, send_padding);
+                self.queue_padding_cell_for_hop(target_hop, queue_info)
+                    .await?;
+            }
+            TreatQueuedCellAsPadding => {
+                self.padding_ctrl
+                    .replaceable_padding_already_queued(target_hop, send_padding);
+            }
+        }
+        Ok(())
+    }
+
+    /// Generate and encrypt a padding cell, and send it to a targeted hop.
+    ///
+    /// Ignores any padding-based blocking.
+    #[cfg(feature = "circ-padding")]
+    async fn queue_padding_cell_for_hop(
+        &mut self,
+        target_hop: HopNum,
+        queue_info: Option<QueuedCellPaddingInfo>,
+    ) -> Result<()> {
+        use tor_cell::relaycell::msg::Drop as DropMsg;
+        let msg = SendRelayCell {
+            hop: Some(target_hop),
+            // TODO circpad: we will probably want padding machines that can send EARLY cells.
+            early: false,
+            cell: AnyRelayMsgOuter::new(None, DropMsg::default().into()),
+        };
+        self.send_relay_cell_inner(msg, queue_info).await
+    }
+
+    /// Enable padding-based blocking,
+    /// or change the rule for padding-based blocking to the one in `block`.
+    #[cfg(feature = "circ-padding")]
+    pub(super) fn start_blocking_for_padding(&mut self, block: padding::StartBlocking) {
+        self.chan_sender.start_blocking();
+        self.padding_block = Some(block);
+    }
+
+    /// Disable padding-based blocking.
+    #[cfg(feature = "circ-padding")]
+    pub(super) fn stop_blocking_for_padding(&mut self) {
+        self.chan_sender.stop_blocking();
+        self.padding_block = None;
+    }
+
+    /// The estimated circuit build timeout for a circuit of the specified length.
+    pub(super) fn estimate_cbt(&self, length: usize) -> Duration {
+        self.timeouts.circuit_build_timeout(length)
+    }
 }
 
-/// The conflux status of a conflux [`Circuit`].
-#[derive(Copy, Clone, Debug, PartialEq, Eq)]
-pub(super) enum ConfluxStatus {
-    /// Circuit has not begun the conflux handshake yet.
-    Unlinked,
-    /// Conflux handshake is in progress.
-    Pending,
-    /// A linked conflux circuit.
-    Linked,
+/// A possible way to handle a request to send padding.
+#[derive(Copy, Clone, Debug)]
+enum CircPaddingDisposition {
+    /// Enqueue the padding normally.
+    QueuePaddingNormally,
+    /// Enqueue the padding, and allow one cell of data on our outbound queue
+    /// to bypass the current block.
+    QueuePaddingAndBypass,
+    /// Do not take any actual padding action:
+    /// existing data on our outbound queue will count as padding.
+    TreatQueuedCellAsPadding,
 }
 
 /// Return the stream ID of `msg`, if it has one.
@@ -1558,60 +1759,3 @@ impl Drop for Circuit {
         let _ = self.channel.close_circuit(self.channel_id);
     }
 }
-
-/// Return the client circuit-creation extensions that we should use in order to negotiate
-/// a given set of circuit hop parameters.
-#[allow(clippy::unnecessary_wraps)]
-pub(super) fn circ_extensions_from_settings(params: &HopSettings) -> Result<Vec<CircRequestExt>> {
-    // allow 'unused_mut' because of the combinations of `cfg` conditions below
-    #[allow(unused_mut)]
-    let mut client_extensions = Vec::new();
-
-    #[allow(unused, unused_mut)]
-    let mut cc_extension_set = false;
-
-    if params.ccontrol.is_enabled() {
-        cfg_if::cfg_if! {
-            if #[cfg(feature = "flowctl-cc")] {
-                client_extensions.push(CircRequestExt::CcRequest(CcRequest::default()));
-                cc_extension_set = true;
-            } else {
-                return Err(
-                    tor_error::internal!(
-                        "Congestion control is enabled on this circuit, but 'flowctl-cc' feature is not enabled"
-                    )
-                    .into()
-                );
-            }
-        }
-    }
-
-    // See whether we need to send a list of required protocol capabilities.
-    // These aren't "negotiated" per se; they're simply demanded.
-    // The relay will refuse the circuit if it doesn't support all of them,
-    // and if any of them isn't supported in the SubprotocolRequest extension.
-    //
-    // (In other words, don't add capabilities here just because you want the
-    // relay to have them! They must be explicitly listed as supported for use
-    // with this extension. For the current list, see
-    // https://spec.torproject.org/tor-spec/create-created-cells.html#subproto-request)
-    //
-    #[allow(unused_mut)]
-    let mut required_protocol_capabilities: Vec<tor_protover::NamedSubver> = Vec::new();
-
-    #[cfg(feature = "counter-galois-onion")]
-    if matches!(params.relay_crypt_protocol(), RelayCryptLayerProtocol::Cgo) {
-        if !cc_extension_set {
-            return Err(tor_error::internal!("Tried to negotiate CGO without CC.").into());
-        }
-        required_protocol_capabilities.push(tor_protover::named::RELAY_CRYPT_CGO);
-    }
-
-    if !required_protocol_capabilities.is_empty() {
-        client_extensions.push(CircRequestExt::SubprotocolRequest(
-            required_protocol_capabilities.into_iter().collect(),
-        ));
-    }
-
-    Ok(client_extensions)
-}
