@@ -6,18 +6,24 @@ use super::{
     RunOnceCmdInner, SendRelayCell,
 };
 use crate::Result;
-use crate::client::circuit::celltypes::CreateResponse;
-use crate::client::circuit::{HopSettings, path};
-use crate::client::reactor::circuit::circ_extensions_from_settings;
+use crate::circuit::celltypes::CreateResponse;
+use crate::circuit::circhop::HopSettings;
+#[cfg(feature = "circ-padding-manual")]
+use crate::client::circuit::padding;
+use crate::client::circuit::path;
 use crate::client::reactor::{NoJoinPointError, NtorClient, ReactorError};
-use crate::client::stream::queue::StreamQueueSender;
-use crate::client::stream::{AnyCmdChecker, DrainRateRequest, StreamRateLimit};
-use crate::client::{HopLocation, TargetHop, streammap};
+use crate::client::{HopLocation, TargetHop};
 use crate::crypto::binding::CircuitBinding;
 use crate::crypto::cell::{InboundClientLayer, OutboundClientLayer};
 use crate::crypto::handshake::ntor_v3::{NtorV3Client, NtorV3PublicKey};
+use crate::stream::cmdcheck::AnyCmdChecker;
+use crate::stream::flow_ctrl::state::StreamRateLimit;
+use crate::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
+use crate::stream::queue::StreamQueueSender;
+use crate::streammap;
 use crate::util::notify::NotifySender;
 use crate::util::skew::ClockSkew;
+use crate::util::tunnel_activity::TunnelActivity;
 #[cfg(test)]
 use crate::{circuit::UniqId, client::circuit::CircParameters, crypto::cell::HopNum};
 use postage::watch;
@@ -41,8 +47,8 @@ use super::{Circuit, ConfluxLinkResultChannel};
 
 use oneshot_fused_workaround as oneshot;
 
-use crate::client::circuit::StreamMpscReceiver;
 use crate::crypto::handshake::ntor::NtorPublicKey;
+use crate::stream::StreamMpscReceiver;
 use tor_linkspec::{EncodedLinkSpec, OwnedChanTarget};
 
 use std::result::Result as StdResult;
@@ -293,6 +299,25 @@ pub(crate) enum CtrlCmd {
         /// or an error if the reactor's tunnel is multi-path.
         answer: oneshot::Sender<StdResult<Circuit, Bug>>,
     },
+
+    /// Install or remove a [`padding::CircuitPadder`] for a given hop.
+    ///
+    /// Any existing `CircuitPadder` at that hop is replaced.
+    #[cfg(feature = "circ-padding-manual")]
+    SetPadder {
+        /// The hop to modify.
+        hop: HopLocation,
+        /// The Padder to install, or None to remove any existing padder.
+        padder: Option<padding::CircuitPadder>,
+        /// A sender to alert after we've changed the padding.
+        sender: oneshot::Sender<Result<()>>,
+    },
+
+    /// Yield the most active [`TunnelActivity`] for any hop on any leg of this tunnel.
+    GetTunnelActivity {
+        /// A sender to receive the reply.
+        sender: oneshot::Sender<TunnelActivity>,
+    },
 }
 
 /// A flow control update message.
@@ -403,7 +428,7 @@ impl<'a> ControlHandler<'a> {
                     return Ok(None);
                 };
 
-                let client_extensions = circ_extensions_from_settings(&settings)?;
+                let client_extensions = settings.circuit_request_extensions()?;
 
                 let (extender, cell) = CircuitExtender::<NtorV3Client>::begin(
                     peer_id,
@@ -544,7 +569,7 @@ impl<'a> ControlHandler<'a> {
                         let cell = AnyRelayMsgOuter::new(Some(stream_id), sendme.into());
 
                         let cell = SendRelayCell {
-                            hop: hop_num,
+                            hop: Some(hop_num),
                             early: false,
                             cell,
                         };
@@ -577,7 +602,7 @@ impl<'a> ControlHandler<'a> {
 
                 let cell = AnyRelayMsgOuter::new(None, msg);
                 let cell = SendRelayCell {
-                    hop: hop_num,
+                    hop: Some(hop_num),
                     early: false,
                     cell,
                 };
@@ -760,6 +785,21 @@ impl<'a> ControlHandler<'a> {
             CtrlCmd::ShutdownAndReturnCircuit { answer } => {
                 self.reactor.handle_shutdown_and_return_circuit(answer)
             }
+            #[cfg(feature = "circ-padding-manual")]
+            CtrlCmd::SetPadder {
+                hop,
+                padder,
+                sender,
+            } => {
+                let result = self.reactor.set_padding_at_hop(hop, padder);
+                let _ = sender.send(result);
+                Ok(())
+            }
+            CtrlCmd::GetTunnelActivity { sender } => {
+                let count = self.reactor.circuits.tunnel_activity();
+                let _ = sender.send(count);
+                Ok(())
+            }
         }
     }
 }
