@@ -55,7 +55,7 @@ pub(crate) trait CongestionControlAlgorithm: Send + std::fmt::Debug {
     fn can_send(&self) -> bool;
     /// Return the congestion window object. The reason is returns an Option is because not all
     /// algorithm uses one and so we avoid acting on it if so.
-    fn cwnd(&self) -> Option<&CongestionWindow>;
+    fn cwnd(&self) -> Option<CongestionWindow>;
 
     /// Inform the algorithm that we just got a DATA cell.
     ///
@@ -130,7 +130,7 @@ impl State {
 
 /// A congestion window. This is generic for all algorithms but their parameters' value will differ
 /// depending on the selected algorithm.
-#[derive(Clone, Debug)]
+#[derive(Clone, Copy, Debug)]
 pub(crate) struct CongestionWindow {
     /// Congestion window parameters from the consensus.
     params: CongestionWindowParams,
@@ -142,10 +142,10 @@ pub(crate) struct CongestionWindow {
 
 impl CongestionWindow {
     /// Constructor taking consensus parameters.
-    fn new(params: &CongestionWindowParams) -> Self {
+    fn new(params: CongestionWindowParams) -> Self {
         Self {
             value: params.cwnd_init(),
-            params: params.clone(),
+            params,
             is_full: false,
         }
     }
@@ -329,7 +329,7 @@ impl CongestionControl {
         // doesn't use it and so no need for the RTT.
         if let Some(cwnd) = self.algorithm.cwnd() {
             self.rtt
-                .update(now, &self.state, cwnd)
+                .update(now, &self.state, &cwnd)
                 .map_err(|e| Error::CircProto(e.to_string()))?;
         }
 
@@ -389,14 +389,11 @@ impl CongestionControl {
     ///
     /// Optional, because not all algorithms track this.
     #[cfg(feature = "conflux")]
-    pub(crate) fn cwnd(&self) -> Option<&CongestionWindow> {
+    pub(crate) fn cwnd(&self) -> Option<CongestionWindow> {
         self.algorithm.cwnd()
     }
 
     /// Return a reference to the RTT estimator.
-    ///
-    /// Used for conflux, for choosing the best circuit to send on.
-    #[cfg(feature = "conflux")]
     pub(crate) fn rtt(&self) -> &RoundtripTimeEstimator {
         &self.rtt
     }
@@ -419,7 +416,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
