@@ -137,8 +137,8 @@ impl CongestionControlAlgorithm for Vegas {
         self.num_inflight < self.cwnd.get()
     }
 
-    fn cwnd(&self) -> Option<&CongestionWindow> {
-        Some(&self.cwnd)
+    fn cwnd(&self) -> Option<CongestionWindow> {
+        Some(self.cwnd)
     }
 
     /// Called when a SENDME cell is received.
@@ -338,7 +338,7 @@ pub(crate) mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -439,7 +439,7 @@ pub(crate) mod test {
             let ret = self.rtt.update(
                 now + Duration::from_micros(p.got_sendme_usec_in),
                 &self.state,
-                self.vegas.cwnd().expect("No CWND"),
+                &self.vegas.cwnd().expect("No CWND"),
             );
             assert!(ret.is_ok());
 
