@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -58,14 +58,19 @@ pub mod bench_utils;
 pub mod channel;
 pub mod circuit;
 pub mod client;
+pub(crate) mod conflux;
 mod congestion;
 mod crypto;
 pub mod memquota;
+mod stream;
+pub(crate) mod streammap;
 pub(crate) mod tunnel;
 mod util;
 
 #[cfg(feature = "relay")]
-pub(crate) mod relay;
+pub mod relay;
+#[cfg(feature = "relay")]
+pub use relay::channel::{RelayChannelBuilder, RelayIdentities};
 
 pub use util::err::{Error, ResolveError};
 pub use util::skew::ClockSkew;
@@ -74,6 +79,7 @@ pub use channel::params::ChannelPaddingInstructions;
 pub use client::{ClientTunnel, HopLocation, TargetHop};
 pub use congestion::params as ccparams;
 pub use crypto::cell::{HopNum, HopNumDisplay};
+pub use stream::flow_ctrl::params::{CellCount, FlowCtrlParameters};
 #[cfg(feature = "send-control-msg")]
 #[cfg_attr(docsrs, doc(cfg(feature = "send-control-msg")))]
 pub use {
@@ -179,7 +185,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
