@@ -4,7 +4,7 @@
 #![allow(unreachable_pub)]
 
 use std::{
-    collections::{hash_map, HashMap},
+    collections::{HashMap, hash_map},
     hash::Hash,
     pin::Pin,
     sync::Arc,
@@ -13,8 +13,8 @@ use std::{
 
 use futures::future::FutureExt;
 use futures::{
-    channel::mpsc::{UnboundedReceiver, UnboundedSender},
     Future,
+    channel::mpsc::{UnboundedReceiver, UnboundedSender},
 };
 use pin_project::pin_project;
 
@@ -249,14 +249,14 @@ mod tests {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
     use std::task::Waker;
 
-    use futures::{executor::block_on, future::poll_fn, StreamExt as _};
+    use futures::{StreamExt as _, executor::block_on, future::poll_fn};
     use oneshot_fused_workaround as oneshot;
     use tor_rtmock::MockRuntime;
 
