@@ -8,13 +8,16 @@ use std::{
     future::Future,
     hash::Hash,
     pin::Pin,
-    task::{Context, Poll},
+    task::{Context, Poll, Waker},
 };
 
-use futures::{FutureExt, StreamExt as _, task::noop_waker_ref};
+use futures::{FutureExt, StreamExt as _};
 use tor_async_utils::peekable_stream::PeekableStream;
 
-use crate::util::keyed_futures_unordered::KeyedFuturesUnordered;
+use crate::util::{
+    keyed_futures_unordered::KeyedFuturesUnordered,
+    tunnel_activity::{InTunnelActivity, TunnelActivity},
+};
 
 /// A future that wraps a [`PeekableStream`], and yields the stream
 /// when an item becomes available.
@@ -83,14 +86,14 @@ pub struct StreamPollSet<K, P, S>
 where
     S: PeekableStream + Unpin,
 {
-    /// Priority for each stream in the set.
+    /// Priority for each stream in the set, and associated InTunnelActivity token.
     // We keep the priority for each stream here instead of bundling it together
     // with the stream, so that the priority can easily be changed even while a
     // future waiting on the stream is still pending (e.g. to support rescaling
     // priorities for EWMA).
     // Invariants:
     // * Every key is also present in exactly one of `ready_values` or `pending_streams`.
-    priorities: HashMap<K, P>,
+    priorities: HashMap<K, (P, InTunnelActivity)>,
     /// Streams that have a result ready, in ascending order by priority.
     // Invariants:
     // * Keys are a (non-strict) subset of those in `priorities`.
@@ -99,6 +102,10 @@ where
     // Invariants:
     // * Keys are a (non-strict) subset of those in `priorities`.
     pending_streams: KeyedFuturesUnordered<K, PeekableReady<S>>,
+
+    /// Information about how active this particular hop has been,
+    /// with respect to tracking overall tunnel activity.
+    tunnel_activity: TunnelActivity,
 }
 
 impl<K, P, S> StreamPollSet<K, P, S>
@@ -113,6 +120,7 @@ where
             priorities: Default::default(),
             ready_streams: Default::default(),
             pending_streams: KeyedFuturesUnordered::new(),
+            tunnel_activity: TunnelActivity::never_used(),
         }
     }
 
@@ -142,14 +150,16 @@ where
             // By `pending_streams` invariant that keys are a subset of those in
             // `priorities`.
             .unwrap_or_else(|_| panic!("Unexpected duplicate key"));
-        v.insert(priority);
+        let token = self.tunnel_activity.inc_streams();
+        v.insert((priority, token));
         Ok(())
     }
 
     /// Remove the entry for `key`, if any. This is the key, priority, buffered
     /// poll_next result, and stream.
     pub fn remove(&mut self, key: &K) -> Option<(K, P, S)> {
-        let priority = self.priorities.remove(key)?;
+        let (priority, token) = self.priorities.remove(key)?;
+        self.tunnel_activity.dec_streams(token);
         if let Some((key, fut)) = self.pending_streams.remove(key) {
             // Validate `priorities` invariant that keys are also present in exactly one of
             // `pending_streams` and `ready_values`.
@@ -258,7 +268,7 @@ where
     ) -> impl Iterator<Item = (&'a K, &'a P, &'a mut S)> + 'a + use<'a, K, P, S> {
         // First poll for ready streams
         while let Poll::Ready(Some((key, stream))) = self.pending_streams.poll_next_unpin(cx) {
-            let priority = self
+            let (priority, _) = self
                 .priorities
                 .get(&key)
                 // By `pending_streams` invariant that all keys are also in `priorities`.
@@ -291,7 +301,7 @@ where
             // Key isn't present at all.
             return None;
         };
-        let priority_mut = priority_entry.get_mut();
+        let (priority_mut, _) = priority_entry.get_mut();
         let Some(((_p, key), mut stream)) = self
             .ready_streams
             .remove_entry(&(priority_mut.clone(), key.clone()))
@@ -299,9 +309,7 @@ where
             // This stream isn't in the ready list.
             return None;
         };
-        match Pin::new(&mut stream)
-            .poll_peek(&mut Context::from_waker(&futures::task::noop_waker()))
-        {
+        match Pin::new(&mut stream).poll_peek(&mut Context::from_waker(Waker::noop())) {
             Poll::Ready(Some(_val)) => (), // Stream is ready, and has an item. Proceed.
             Poll::Ready(None) => {
                 // Stream is ready, but is terminated.
@@ -343,13 +351,13 @@ where
     // This will be used for packing and fragmentation, to take part of a DATA message.
     #[allow(unused)]
     pub fn peek_mut<'a>(&'a mut self, key: &K) -> Option<Poll<Option<&'a mut S::Item>>> {
-        let priority = self.priorities.get(key)?;
+        let (priority, _) = self.priorities.get(key)?;
         let Some(peekable) = self.ready_streams.get_mut(&(priority.clone(), key.clone())) else {
             return Some(Poll::Pending);
         };
         // We don't have a waker registered here, so we can just use the noop waker.
         // TODO: Create a mut future for `PeekableStream`.
-        Some(Pin::new(peekable).poll_peek_mut(&mut Context::from_waker(noop_waker_ref())))
+        Some(Pin::new(peekable).poll_peek_mut(&mut Context::from_waker(Waker::noop())))
     }
 
     /// Get a reference to the stream for `key`.
@@ -364,7 +372,7 @@ where
             debug_assert!(s.is_some(), "Unexpected missing pending stream");
             return s;
         }
-        let priority = self.priorities.get(key)?;
+        let (priority, _) = self.priorities.get(key)?;
         self.ready_streams.get(&(priority.clone(), key.clone()))
     }
 
@@ -393,7 +401,7 @@ where
             debug_assert!(s.is_some(), "Unexpected missing pending stream");
             return s;
         }
-        let priority = self.priorities.get(key)?;
+        let (priority, _) = self.priorities.get(key)?;
         self.ready_streams.get_mut(&(priority.clone(), key.clone()))
     }
 
@@ -401,6 +409,23 @@ where
     pub fn len(&self) -> usize {
         self.priorities.len()
     }
+
+    /// Return a `TunnelActivity` for this hop.
+    pub fn tunnel_activity(&self) -> TunnelActivity {
+        assert_eq!(self.len(), self.tunnel_activity.n_open_streams());
+        self.tunnel_activity
+    }
+}
+
+impl<K, P, S> Drop for StreamPollSet<K, P, S>
+where
+    S: PeekableStream + Unpin,
+{
+    fn drop(&mut self) {
+        self.priorities
+            .drain()
+            .for_each(|(_key, (_prio, token))| token.consume_and_forget());
+    }
 }
 
 /// Error returned by [`StreamPollSet::try_insert`].
@@ -429,7 +454,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
