@@ -4,11 +4,11 @@
 //! version 1, and communicate with them in order to specify configuration parameters and
 //! receive updates as to the current state of the PT.
 
+use crate::PtClientMethod;
 use crate::err;
 use crate::err::PtError;
-use crate::PtClientMethod;
-use futures::channel::mpsc::Receiver;
 use futures::StreamExt;
+use futures::channel::mpsc::Receiver;
 use itertools::Itertools;
 use std::borrow::Cow;
 use std::collections::HashMap;
@@ -535,7 +535,7 @@ pub(crate) mod sealed {
                         return Err(PtError::ProtocolViolation(format!(
                             "unknown CMETHOD protocol '{}'",
                             x
-                        )))
+                        )));
                     }
                 },
                 None => SocksVersion::V5,
@@ -1141,7 +1141,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
