@@ -218,6 +218,7 @@ pub(crate) enum BuiltinVariant {
 
 #[derive(Deserialize, Clone, Debug)]
 #[serde(bound = "R::Path : Deserialize<'de>, AddrWithStr<R::SocketAddr> : Deserialize<'de>")]
+#[allow(clippy::missing_docs_in_private_items)]
 pub(crate) struct Connect<R: Addresses> {
     /// The address of the socket at which the client should try to reach the RPC server,
     /// and which the RPC server should bind.
@@ -437,7 +438,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
