@@ -1,15 +1,18 @@
 //! Implementation for using Rustls with a runtime.
+//!
+//! #
 
-use crate::traits::{CertifiedConn, TlsConnector, TlsProvider};
 use crate::StreamOps;
+use crate::traits::{CertifiedConn, TlsConnector, TlsProvider};
 
 use async_trait::async_trait;
 use futures::{AsyncRead, AsyncWrite};
 use futures_rustls::rustls::{self, crypto::CryptoProvider};
 use rustls::client::danger;
-use rustls::crypto::{verify_tls12_signature, verify_tls13_signature, WebPkiSupportedAlgorithms};
+use rustls::crypto::{WebPkiSupportedAlgorithms, verify_tls12_signature, verify_tls13_signature};
 use rustls::{CertificateError, Error as TLSError};
 use rustls_pki_types::{CertificateDer, ServerName};
+use tracing::instrument;
 use webpki::EndEntityCert; // this is actually rustls_webpki.
 
 use std::{
@@ -21,12 +24,31 @@ use std::{
 ///
 /// It supports wrapping any reasonable stream type that implements `AsyncRead` + `AsyncWrite`.
 ///
-/// The application is responsible for calling `CryptoProvider::install_default_provider()`
-/// before constructing one of these providers.  If they do not, we will issue a warning,
-/// and install a default (ring) provider.
+/// # Cryptographic providers
+///
+/// The application is responsible for calling [`CryptoProvider::install_default()`]
+/// before constructing [`TlsProvider`].  If they do not, we will issue a warning,
+/// and install a default ([ring]) provider.
+///
+/// We choose ring because, of the two builtin providers that ship with rustls,
+/// it has the best license.
+/// We _could_ instead use [aws-lc-rs] (for its early MLKEM768 support),
+/// but it is [still under the old OpenSSL license][aws-lc-license], which is GPL-incompatible.
+/// (Although Arti isn't under the GPL itself, we are trying to stay compatible with it.)
+///
+/// See the [rustls documentation][all-providers] for a list of other rustls
+/// cryptography providcers.
+///
+/// [ring]: https://crates.io/crates/ring
+/// [aws-lc-rs]: https://github.com/aws/aws-lc-rs
+/// [aws-lc-license]: https://github.com/aws/aws-lc/issues/2203
+/// [all-providers]: https://docs.rs/rustls/latest/rustls/#cryptography-providers
 #[cfg_attr(
     docsrs,
-    doc(cfg(all(feature = "rustls", any(feature = "tokio", feature = "async-std"))))
+    doc(cfg(all(
+        feature = "rustls",
+        any(feature = "tokio", feature = "async-std", feature = "smol")
+    )))
 )]
 #[derive(Clone)]
 #[non_exhaustive]
@@ -81,6 +103,7 @@ where
 {
     type Conn = futures_rustls::client::TlsStream<S>;
 
+    #[instrument(skip_all, level = "trace")]
     async fn negotiate_unvalidated(&self, stream: S, sni_hostname: &str) -> IoResult<Self::Conn> {
         let name: ServerName<'_> = sni_hostname
             .try_into()
@@ -129,7 +152,7 @@ fn ensure_provider_installed() {
 }
 
 impl RustlsProvider {
-    /// Construct a new [`RustlsProvider`.]
+    /// Construct a new [`RustlsProvider`].
     pub(crate) fn new() -> Self {
         ensure_provider_installed();
 
@@ -248,7 +271,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
